@@ -10,13 +10,20 @@
 /* define if extended WM hints are enabled */
 #undef HAVE_EWMH
 
-/* Where to search for images.  */
+/* Where to search for images. */
 #undef FVWM_IMAGEPATH
 
-/* Define if Xpm library is used.  */
+/* Define if Xpm library is used. */
 #undef XPM
 
-/* Define if rplay library is used.  */
+/* Define if ppm library is used. */
+#undef HAVE_PNG
+
+/* Define if your processor stores words with the most significant
+   byte first (like Motorola and SPARC, unlike Intel and VAX). */
+#undef WORDS_BIGENDIAN
+
+/* Define if rplay library is used. */
 #undef HAVE_RPLAY
 
 /* Define if Xinerama should be emulated on a single screen. */
@@ -25,6 +32,18 @@
 /* Define if Xinerama library is used. */
 #undef HAVE_XINERAMA
 
+/* Define if Xrender library is used. */
+#undef HAVE_XRENDER
+
+/* Define if Xft library is used. */
+#undef HAVE_XFT
+
+/* Define if Xft 2 library is used. */
+#undef HAVE_XFT2
+
+/* Define if Xft library can handle utf8 encoding */
+#undef HAVE_XFT_UTF8
+
 /* Define if stroke library is used. */
 #undef HAVE_STROKE
 
@@ -36,7 +55,7 @@
 #    define STROKE_CODE(x)
 #endif
 
-/* Define if readline is available.  */
+/* Define if readline is available. */
 #undef HAVE_READLINE
 
 /* Define if iconv (in the libc) or libiconv is available */
@@ -49,6 +68,12 @@
 /* define if second arg of iconv use const */
 #undef ICONV_ARG_USE_CONST
 
+#ifdef USE_LIBICONV
+/* define to use locale_charset in the place of nl_langinfog if libiconv
+ * is used */
+#undef HAVE_LIBCHARSET
+#endif
+
 /* Define if nl_langinfo is available */
 #undef HAVE_CODESET
 
@@ -66,22 +91,29 @@
  * don't normally use a shaped window, you have to decide for yourself.
  *
  * Note: if it is compiled in, run time detection is used to make sure that
- * the currently running X server supports it.  */
+ * the currently running X server supports it. */
 #undef SHAPE
 
+/* Define if fribidi library is used. */
+#undef HAVE_BIDI
+
 /* Enables the "MiniIcon" Style option to specify a small pixmap which
  * can be used as one of the title-bar buttons, shown in window list,
  * utilized by modules, etc.  Requires PIXMAP_BUTTONS to be defined
- * (see below).  */
+ * (see below). */
 /* #undef MINI_ICONS */
 /* NOTE: hard coded to 1 */
-#define MINI_ICONS 1
+#if 1
+#define FMiniIconsSupported 1
+#else
+#define FMiniIconsSupported 0
+#endif
 
 /* Enables tagged general decoration styles which can be assigned to
  * windows using the UseDecor Style option, or dynamically updated
  * with ChangeDecor.  To create and destroy "decor" definitions, see
  * the man page entries for AddToDecor and DestroyDecor.  There is a
- * slight memory penalty for each additionally defined decor.  */
+ * slight memory penalty for each additionally defined decor. */
 /* #undef USEDECOR */
 /* NOTE: hard coded to 1 */
 #define USEDECOR 1
@@ -92,24 +124,24 @@
 /* Enables session management functionality. */
 #undef SESSION
 
-/* Enables X11 multibyte character support */
-#undef I18N_MB
-
 /* Enables to use setlocale() provided by X */
 #undef X_LOCALE
 
-/* Specify a type for sig_atomic_t if it's not available.  */
+/* Enable X output method */
+#undef HAVE_XOUTPUT_METHOD
+
+/* Specify a type for sig_atomic_t if it's not available. */
 #undef sig_atomic_t
 
-/* Define to the type used in argument 1 `select'.  Usually this is an `int'.  */
+/* Define to the type used in argument 1 `select'.  Usually this is an `int'. */
 #undef fd_set_size_t
 
 /* Define to the type used in arguments 2-4 of `select', if not set by system
-   headers.  */
+   headers. */
 #undef fd_set
 
 /* Define a suitable cast for arguments 2-4 of `select'.  On most systems,
-   this will be the empty string, as select usually takes pointers to fd_set.  */
+   this will be the empty string, as select usually takes pointers to fd_set. */
 #undef SELECT_FD_SET_CAST
 
 
@@ -193,7 +225,7 @@
        #pragma alloca
 #  else
 #    ifndef alloca /* predefined by HP cc +Olibcalls */
-         char *alloca ();
+	 char *alloca ();
 #    endif
 #  endif
 #endif
