@@ -16,16 +16,21 @@
 #ifndef _MOVE_RESIZE_
 #define _MOVE_RESIZE_
 
+#include "menus.h"
+
 void switch_move_resize_grid(Bool state);
 void AnimatedMoveOfWindow(
   Window w,int startX,int startY,int endX, int endY,Bool fWarpPointerToo,
-  int cusDelay, float *ppctMovement, Bool ParentalMenu);
+  int cusDelay, float *ppctMovement, MenuRoot *menu_root);
 void AnimatedMoveFvwmWindow(
-  FvwmWindow *tmp_win, Window w, int startX, int startY, int endX, int endY,
+  FvwmWindow *fw, Window w, int startX, int startY, int endX, int endY,
   Bool fWarpPointerToo, int cmsDelay, float *ppctMovement);
-Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
+Bool moveLoop(FvwmWindow *fw, int XOffset, int YOffset, int Width,
 	      int Height, int *FinalX, int *FinalY,Bool do_move_opaque);
 void handle_stick(F_CMD_ARGS, int toggle);
 void resize_geometry_window(void);
+void __move_icon(
+	FvwmWindow *fw, int x, int y, int old_x, int old_y,
+	Bool do_move_animated, Bool do_warp_pointer);
 
 #endif /* _MOVE_RESIZE_ */
