@@ -30,6 +30,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -84,7 +85,7 @@ static int SmartPlacement(
   int tx = 0;
   int ty = 0;
   int th = 0;
-  FvwmWindow *test_window;
+  FvwmWindow *test_fw;
   int stickyx;
   int stickyy;
   rectangle g;
@@ -101,42 +102,42 @@ static int SmartPlacement(
     while (test_x + temp_w < PageRight && !loc_ok)
     {
       loc_ok = True;
-      test_window = Scr.FvwmRoot.next;
-      for ( ; test_window != NULL && loc_ok; test_window = test_window->next)
+      test_fw = Scr.FvwmRoot.next;
+      for ( ; test_fw != NULL && loc_ok; test_fw = test_fw->next)
       {
-	if (t == test_window || IS_EWMH_DESKTOP(test_window->w))
+	if (t == test_fw || IS_EWMH_DESKTOP(FW_W(test_fw)))
 	  continue;
 
-        /*  RBW - account for sticky windows...  */
-        if (test_window->Desk == t->Desk || IS_STICKY(test_window))
-        {
-          if (IS_STICKY(test_window))
+	/*  RBW - account for sticky windows...  */
+	if (test_fw->Desk == t->Desk || IS_STICKY(test_fw))
+	{
+	  if (IS_STICKY(test_fw))
 	  {
 	    stickyx = pdeltax;
 	    stickyy = pdeltay;
 	  }
-          else
+	  else
 	  {
 	    stickyx = 0;
 	    stickyy = 0;
 	  }
-	  rc = get_visible_window_or_icon_geometry(test_window, &g);
+	  rc = get_visible_window_or_icon_geometry(test_fw, &g);
 	  if (rc == True &&
-	      (PLACEMENT_AVOID_ICON == 0 || !IS_ICONIFIED(test_window)))
+	      (PLACEMENT_AVOID_ICON == 0 || !IS_ICONIFIED(test_fw)))
 	  {
 	    tx = g.x - stickyx;
 	    ty = g.y - stickyy;
 	    tw = g.width;
 	    th = g.height;
-            if (tx < test_x + width  && test_x < tx + tw &&
+	    if (tx < test_x + width  && test_x < tx + tw &&
 		ty < test_y + height && test_y < ty + th)
-            {
+	    {
 	      /* window overlaps, look for a different place */
-              loc_ok = False;
-              test_x = tx + tw - 1;
-            }
+	      loc_ok = False;
+	      test_x = tx + tw - 1;
+	    }
 	  }
-        } /* if (test_window->Desk == t->Desk || IS_STICKY(test_window)) */
+	} /* if (test_fw->Desk == t->Desk || IS_STICKY(test_fw)) */
       } /* for */
       if (!loc_ok)
 	test_x +=1;
@@ -253,7 +254,7 @@ static int get_next_x(
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
     if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)) ||
-	IS_EWMH_DESKTOP(testw->w))
+	IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
     if (IS_STICKY(testw))
@@ -349,7 +350,7 @@ static int get_next_y(
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
     if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
-	|| IS_EWMH_DESKTOP(testw->w))
+	|| IS_EWMH_DESKTOP(FW_W(testw)))
       continue;
 
     if (IS_STICKY(testw))
@@ -439,7 +440,7 @@ static float test_fit(
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
     if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
-	|| IS_EWMH_DESKTOP(testw->w))
+	|| IS_EWMH_DESKTOP(FW_W(testw)))
        continue;
 
     if (IS_STICKY(testw))
@@ -468,15 +469,15 @@ static float test_fit(
       yb = MIN(y12, y22);
       anew = (xr - xl) * (yb - yt);
       if (IS_ICONIFIED(testw))
-        avoidance_factor = ICON_PLACEMENT_PENALTY(testw);
+	avoidance_factor = ICON_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) > 0)
-        avoidance_factor = ONTOP_PLACEMENT_PENALTY(testw);
+	avoidance_factor = ONTOP_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) < 0)
-        avoidance_factor = BELOW_PLACEMENT_PENALTY(testw);
+	avoidance_factor = BELOW_PLACEMENT_PENALTY(testw);
       else if(IS_STICKY(testw))
-        avoidance_factor = STICKY_PLACEMENT_PENALTY(testw);
+	avoidance_factor = STICKY_PLACEMENT_PENALTY(testw);
       else
-        avoidance_factor = NORMAL_PLACEMENT_PENALTY(testw);
+	avoidance_factor = NORMAL_PLACEMENT_PENALTY(testw);
 
       if (use_percent)
       {
@@ -511,7 +512,7 @@ static float test_fit(
       aoi += anew;
       if (aoi > aoimin && aoimin != -1)
       {
-        return aoi;
+	return aoi;
       }
     }
   }
@@ -537,8 +538,9 @@ static float test_fit(
  *
  **************************************************************************/
 Bool PlaceWindow(
-  FvwmWindow *tmp_win, style_flags *sflags,
-  int Desk, int PageX, int PageY, int XineramaScreen, int mode)
+	FvwmWindow *fw, style_flags *sflags, rectangle *attr_g,
+	int Desk, int PageX, int PageY, int XineramaScreen, int mode,
+	initial_window_options_type *win_opts)
 {
   FvwmWindow *t;
   int xl = -1;
@@ -554,6 +556,7 @@ Bool PlaceWindow(
   int PageRight;
   int PageBottom;
   rectangle screen_g;
+  size_borders b;
   Bool rc = False;
   struct
   {
@@ -564,7 +567,6 @@ Bool PlaceWindow(
     unsigned do_not_use_wm_placement : 1;
   } flags;
   extern Bool Restarting;
-  extern Bool PPosOverride;
 
   memset(&flags, 0, sizeof(flags));
 
@@ -588,7 +590,8 @@ Bool PlaceWindow(
      * Honor the flag unless...
      * it's a restart or recapture, and that option's disallowed...
      */
-    if (PPosOverride && (Restarting || (Scr.flags.windows_captured)) &&
+    if (win_opts->flags.do_override_ppos &&
+	(Restarting || (Scr.flags.windows_captured)) &&
 	!SRECAPTURE_HONORS_STARTS_ON_PAGE(sflags))
     {
       flags.do_honor_starts_on_page = False;
@@ -597,7 +600,8 @@ Bool PlaceWindow(
     /*
      * it's a cold start window capture, and that's disallowed...
      */
-    if (PPosOverride && (!Restarting && !(Scr.flags.windows_captured)) &&
+    if (win_opts->flags.do_override_ppos &&
+	(!Restarting && !(Scr.flags.windows_captured)) &&
 	!SCAPTURE_HONORS_STARTS_ON_PAGE(sflags))
     {
       flags.do_honor_starts_on_page = False;
@@ -606,12 +610,12 @@ Bool PlaceWindow(
     /*
      * it's ActivePlacement and SkipMapping, and that's disallowed.
      */
-    if (!PPosOverride &&
-	(DO_NOT_SHOW_ON_MAP(tmp_win) &&
+    if (!win_opts->flags.do_override_ppos &&
+	(DO_NOT_SHOW_ON_MAP(fw) &&
 	 (SPLACEMENT_MODE(sflags) == PLACE_MANUAL ||
 	  SPLACEMENT_MODE(sflags) == PLACE_MANUAL_B ||
 	  SPLACEMENT_MODE(sflags) == PLACE_TILEMANUAL) &&
-         !SMANUAL_PLACEMENT_HONORS_STARTS_ON_PAGE(sflags)))
+	 !SMANUAL_PLACEMENT_HONORS_STARTS_ON_PAGE(sflags)))
     {
       flags.do_honor_starts_on_page = False;
       fvwm_msg(WARN, "PlaceWindow",
@@ -651,7 +655,7 @@ Bool PlaceWindow(
   if (SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAPPERCENT &&
       SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAP)
   {
-    EWMH_GetWorkAreaIntersection(tmp_win,
+    EWMH_GetWorkAreaIntersection(fw,
       &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height,
       SEWMH_PLACEMENT_MODE(sflags));
   }
@@ -663,50 +667,50 @@ Bool PlaceWindow(
   yt = PageTop;
 
   /* Don't alter the existing desk location during Capture/Recapture.  */
-  if (!PPosOverride)
+  if (!win_opts->flags.do_override_ppos)
   {
-    tmp_win->Desk = Scr.CurrentDesk;
+    fw->Desk = Scr.CurrentDesk;
   }
-  if (SIS_STICKY(*sflags))
-    tmp_win->Desk = Scr.CurrentDesk;
+  if (S_IS_STICKY(SFC(*sflags)))
+    fw->Desk = Scr.CurrentDesk;
   else if (SUSE_START_ON_DESK(sflags) && Desk && flags.do_honor_starts_on_page)
-    tmp_win->Desk = (Desk > -1) ? Desk - 1 : Desk;
+    fw->Desk = (Desk > -1) ? Desk - 1 : Desk;
   else
   {
-    if ((DO_USE_WINDOW_GROUP_HINT(tmp_win)) &&
-	(tmp_win->wmhints) && (tmp_win->wmhints->flags & WindowGroupHint)&&
-	(tmp_win->wmhints->window_group != None) &&
-	(tmp_win->wmhints->window_group != Scr.Root))
+    if ((DO_USE_WINDOW_GROUP_HINT(fw)) &&
+	(fw->wmhints) && (fw->wmhints->flags & WindowGroupHint)&&
+	(fw->wmhints->window_group != None) &&
+	(fw->wmhints->window_group != Scr.Root))
     {
       /* Try to find the group leader or another window in the group */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if (t->w == tmp_win->wmhints->window_group)
+	if (FW_W(t) == fw->wmhints->window_group)
 	{
 	  /* found the group leader, break out */
-	  tmp_win->Desk = t->Desk;
+	  fw->Desk = t->Desk;
 	  break;
 	}
 	else if (t->wmhints && (t->wmhints->flags & WindowGroupHint) &&
-		 (t->wmhints->window_group == tmp_win->wmhints->window_group))
+		 (t->wmhints->window_group == fw->wmhints->window_group))
 	{
 	  /* found a window from the same group, but keep looking for the group
 	   * leader */
-	  tmp_win->Desk = t->Desk;
+	  fw->Desk = t->Desk;
 	}
       }
     }
-    if ((IS_TRANSIENT(tmp_win))&&(tmp_win->transientfor!=None)&&
-	(tmp_win->transientfor != Scr.Root))
+    if ((IS_TRANSIENT(fw))&&(FW_W_TRANSIENTFOR(fw)!=None)&&
+	(FW_W_TRANSIENTFOR(fw) != Scr.Root))
     {
       /* Try to find the parent's desktop */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if (t->w == tmp_win->transientfor)
-        {
-	  tmp_win->Desk = t->Desk;
-          break;
-        }
+	if (FW_W(t) == FW_W_TRANSIENTFOR(fw))
+	{
+	  fw->Desk = t->Desk;
+	  break;
+	}
       }
     }
 
@@ -720,12 +724,12 @@ Bool PlaceWindow(
 
       if (
 	XGetWindowProperty(
-	  dpy, tmp_win->w, _XA_WM_DESKTOP, 0L, 1L, True, _XA_WM_DESKTOP,
+	  dpy, FW_W(fw), _XA_WM_DESKTOP, 0L, 1L, True, _XA_WM_DESKTOP,
 	  &atype, &aformat, &nitems, &bytes_remain, &prop) == Success)
       {
 	if (prop != NULL)
 	{
-	  tmp_win->Desk = *(unsigned long *)prop;
+	  fw->Desk = *(unsigned long *)prop;
 	  XFree(prop);
 	}
       }
@@ -735,16 +739,16 @@ Bool PlaceWindow(
   /* I think it would be good to switch to the selected desk
    * whenever a new window pops up, except during initialization */
   /*  RBW - 11/02/1998  --  I dont. */
-  if ((!PPosOverride)&&(!DO_NOT_SHOW_ON_MAP(tmp_win)))
+  if ((!win_opts->flags.do_override_ppos)&&(!DO_NOT_SHOW_ON_MAP(fw)))
   {
-    goto_desk(tmp_win->Desk);
+    goto_desk(fw->Desk);
   }
 
   /* Don't move viewport if SkipMapping, or if recapturing the window,
    * adjust the coordinates later. Otherwise, just switch to the target
    * page - it's ever so much simpler.
    */
-  if (!SIS_STICKY(*sflags) && SUSE_START_ON_DESK(sflags))
+  if (!S_IS_STICKY(SFC(*sflags)) && SUSE_START_ON_DESK(sflags))
   {
     if (PageX && PageY)
     {
@@ -752,7 +756,7 @@ Bool PlaceWindow(
       py = PageY - 1;
       px *= Scr.MyDisplayWidth;
       py *= Scr.MyDisplayHeight;
-      if ( (!PPosOverride) && (!DO_NOT_SHOW_ON_MAP(tmp_win)))
+      if ( (!win_opts->flags.do_override_ppos) && (!DO_NOT_SHOW_ON_MAP(fw)))
       {
 	MoveViewport(px,py,True);
       }
@@ -792,7 +796,7 @@ Bool PlaceWindow(
     Bool has_ppos = False;
     Bool has_uspos = False;
 
-    if (IS_TRANSIENT(tmp_win))
+    if (IS_TRANSIENT(fw))
     {
       override_ppos = SUSE_NO_TRANSIENT_PPOSITION(sflags);
       override_uspos = SUSE_NO_TRANSIENT_USPOSITION(sflags);
@@ -802,11 +806,11 @@ Bool PlaceWindow(
       override_ppos = SUSE_NO_PPOSITION(sflags);
       override_uspos = SUSE_NO_USPOSITION(sflags);
     }
-    if ((tmp_win->hints.flags & PPosition) && !override_ppos)
+    if ((fw->hints.flags & PPosition) && !override_ppos)
     {
       has_ppos = True;
     }
-    if ((tmp_win->hints.flags & USPosition) && !override_uspos)
+    if ((fw->hints.flags & USPosition) && !override_uspos)
     {
       has_uspos = True;
     }
@@ -819,13 +823,13 @@ Bool PlaceWindow(
     {
       flags.do_not_use_wm_placement = True;
     }
-    else if (PPosOverride)
+    else if (win_opts->flags.do_override_ppos)
     {
       flags.do_not_use_wm_placement = True;
     }
     else if (!flags.do_honor_starts_on_page &&
-	     tmp_win->wmhints && (tmp_win->wmhints->flags & StateHint) &&
-	     tmp_win->wmhints->initial_state == IconicState)
+	     fw->wmhints && (fw->wmhints->flags & StateHint) &&
+	     fw->wmhints->initial_state == IconicState)
     {
       flags.do_forbid_manual_placement = True;
     }
@@ -836,7 +840,7 @@ Bool PlaceWindow(
     unsigned int placement_mode = SPLACEMENT_MODE(sflags);
 
     /* override if Manual placement happen */
-    SET_PLACED_BY_FVWM(tmp_win,True);
+    SET_PLACED_BY_FVWM(fw,True);
 
     if (flags.do_forbid_manual_placement)
     {
@@ -860,7 +864,7 @@ Bool PlaceWindow(
     case PLACE_TILEMANUAL:
       flags.is_smartly_placed =
 	SmartPlacement(
-	  tmp_win, &screen_g, tmp_win->frame_g.width, tmp_win->frame_g.height,
+	  fw, &screen_g, fw->frame_g.width, fw->frame_g.height,
 	  &xl, &yt, pdeltax, pdeltay);
       if (flags.is_smartly_placed)
 	break;
@@ -881,26 +885,26 @@ Bool PlaceWindow(
 
 	/* Grabbed the pointer - continue */
 	MyXGrabServer(dpy);
-	if (XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY,
+	if (XGetGeometry(dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY,
 			 (unsigned int *)&DragWidth,
 			 (unsigned int *)&DragHeight,
 			 &JunkBW,  &JunkDepth) == 0)
-        {
+	{
 	  MyXUngrabServer(dpy);
-          UngrabEm(GRAB_NORMAL);
+	  UngrabEm(GRAB_NORMAL);
 	  return False;
 	}
-	SET_PLACED_BY_FVWM(tmp_win,False);
+	SET_PLACED_BY_FVWM(fw,False);
 	MyXGrabKeyboard(dpy);
-	DragWidth = tmp_win->frame_g.width;
-	DragHeight = tmp_win->frame_g.height;
+	DragWidth = fw->frame_g.width;
+	DragHeight = fw->frame_g.height;
 
 	if (Scr.SizeWindow != None)
 	{
 	  XMapRaised(dpy, Scr.SizeWindow);
 	}
 	FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
-	if (moveLoop(tmp_win, mx, my, DragWidth, DragHeight,
+	if (moveLoop(fw, mx, my, DragWidth, DragHeight,
 		     &xl, &yt, False))
 	{
 	  /* resize too */
@@ -928,62 +932,82 @@ Bool PlaceWindow(
       }
       if (flags.do_honor_starts_on_page)
       {
-        xl -= pdeltax;
-        yt -= pdeltay;
+	xl -= pdeltax;
+	yt -= pdeltay;
       }
-      tmp_win->attr.y = yt;
-      tmp_win->attr.x = xl;
+      attr_g->y = yt;
+      attr_g->x = xl;
       break;
     case PLACE_MINOVERLAPPERCENT:
-      CleverPlacement(tmp_win, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
+      CleverPlacement(
+	      fw, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
       flags.is_smartly_placed = True;
       break;
     case PLACE_TILECASCADE:
       flags.is_smartly_placed =
 	SmartPlacement(
-	  tmp_win, &screen_g, tmp_win->frame_g.width, tmp_win->frame_g.height,
+	  fw, &screen_g, fw->frame_g.width, fw->frame_g.height,
 	  &xl, &yt, pdeltax, pdeltay);
       if (flags.is_smartly_placed)
 	break;
       /* fall through to cascade placement */
     case PLACE_CASCADE:
     case PLACE_CASCADE_B:
-      /*  either "smart" placement fail and the second choice is "random"
-	  placement (TileCascade) or we have a "random" placement in any case
+      /*  either "smart" placement fail and the second choice is "cascade"
+	  placement (TileCascade) or we have a "cascade" placement in any case
 	  (Cascade) or we have a crazy SPLACEMENT_MODE(sflags) value set with
 	  the old Style Dumb/Smart, Random/Active, Smart/SmartOff (i.e.:
 	  Dumb+Random+Smart or Dumb+Active+Smart)
       */
-      if ((Scr.randomx += tmp_win->title_g.height) > screen_g.width / 2)
+      if (Scr.cascade_window != NULL)
       {
-	Scr.randomx = tmp_win->title_g.height;
+	Scr.cascade_x += fw->title_thickness;
+	Scr.cascade_y += 2 * fw->title_thickness;
       }
-      if ((Scr.randomy += 2 * tmp_win->title_g.height) > screen_g.height / 2)
+      Scr.cascade_window = fw;
+      if (Scr.cascade_x > screen_g.width / 2)
       {
-	Scr.randomy = 2 * tmp_win->title_g.height;
+	Scr.cascade_x = fw->title_thickness;
       }
-      tmp_win->attr.x = Scr.randomx + PageLeft - pdeltax;
-      tmp_win->attr.y = Scr.randomy + PageTop - pdeltay;
-      /* try to keep the window on the screen */
-      tmp_win->frame_g.x = PageLeft + tmp_win->attr.x + tmp_win->old_bw + 10;
-      tmp_win->frame_g.y = PageTop + tmp_win->attr.y + tmp_win->old_bw + 10;
+      if (Scr.cascade_y > screen_g.height / 2)
+      {
+	Scr.cascade_y = 2 * fw->title_thickness;
+      }
+      attr_g->x = Scr.cascade_x + PageLeft - pdeltax;
+      attr_g->y = Scr.cascade_y + PageTop - pdeltay;
 
-      if (tmp_win->attr.x + tmp_win->frame_g.width >= PageRight)
+      /* migo: what should these crazy calculations mean? */
+#if 0
+      fw->frame_g.x = PageLeft + attr_g->x + fw->attr_backup.border_width + 10;
+      fw->frame_g.y = PageTop + attr_g->y + fw->attr_backup.border_width + 10;
+#endif
+
+      /* try to keep the window on the screen */
+      get_window_borders(fw, &b);
+      if (attr_g->x + fw->frame_g.width >= PageRight)
       {
-	tmp_win->attr.x = PageRight - tmp_win->attr.width
-	  - tmp_win->old_bw - 2*tmp_win->boundary_width;
-	Scr.randomx = 0;
+	attr_g->x = PageRight - attr_g->width - b.total_size.width;
+	Scr.cascade_x = fw->title_thickness;
       }
-      if (tmp_win->attr.y + tmp_win->frame_g.height >= PageBottom)
+      if (attr_g->y + fw->frame_g.height >= PageBottom)
       {
-	tmp_win->attr.y = PageBottom - tmp_win->attr.height
-	  - tmp_win->old_bw - tmp_win->title_g.height -
-	  2*tmp_win->boundary_width;
-	Scr.randomy = 0;
+	attr_g->y = PageBottom - attr_g->height - b.total_size.height;
+	Scr.cascade_y = 2 * fw->title_thickness;
+      }
+
+      /* the left and top sides are more important in huge windows */
+      if (attr_g->x < PageLeft)
+      {
+	attr_g->x = PageLeft;
+      }
+      if (attr_g->y < PageTop)
+      {
+	attr_g->y = PageTop;
       }
       break;
     case PLACE_MINOVERLAP:
-      CleverPlacement(tmp_win, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
+      CleverPlacement(
+	      fw, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
       flags.is_smartly_placed = True;
       break;
     default:
@@ -994,14 +1018,14 @@ Bool PlaceWindow(
     if (flags.is_smartly_placed)
     {
       /* "smart" placement succed, we have done ... */
-      tmp_win->attr.x = xl;
-      tmp_win->attr.y = yt;
+      attr_g->x = xl;
+      attr_g->y = yt;
     }
   } /* !flags.do_not_use_wm_placement */
   else
   {
-    if (!PPosOverride)
-      SET_PLACED_BY_FVWM(tmp_win,False);
+    if (!win_opts->flags.do_override_ppos)
+      SET_PLACED_BY_FVWM(fw,False);
     /* the USPosition was specified, or the window is a transient,
      * or it starts iconic so place it automatically */
 
@@ -1022,27 +1046,26 @@ Bool PlaceWindow(
        */
 
       FScreenTranslateCoordinates(
-	NULL, XineramaScreen, NULL, FSCREEN_GLOBAL,
-	&tmp_win->attr.x, &tmp_win->attr.y);
+	NULL, XineramaScreen, NULL, FSCREEN_GLOBAL, &attr_g->x, &attr_g->y);
     }
 
     /*
      *  If SkipMapping, and other legalities are observed, adjust for
      * StartsOnPage.
      */
-    if ( ( DO_NOT_SHOW_ON_MAP(tmp_win) && flags.do_honor_starts_on_page )  &&
+    if ( ( DO_NOT_SHOW_ON_MAP(fw) && flags.do_honor_starts_on_page )  &&
 
-	 ( (!(IS_TRANSIENT(tmp_win)) ||
+	 ( (!(IS_TRANSIENT(fw)) ||
 	    SUSE_START_ON_PAGE_FOR_TRANSIENT(sflags)) &&
 
 	   ((SUSE_NO_PPOSITION(sflags)) ||
-	    !(tmp_win->hints.flags & PPosition)) &&
+	    !(fw->hints.flags & PPosition)) &&
 
-           /*  RBW - allow StartsOnPage to go through, even if iconic.  */
-           ( ((!((tmp_win->wmhints)&&
-		 (tmp_win->wmhints->flags & StateHint)&&
-		 (tmp_win->wmhints->initial_state == IconicState)))
-              || (flags.do_honor_starts_on_page)) )
+	   /*  RBW - allow StartsOnPage to go through, even if iconic.  */
+	   ( ((!((fw->wmhints)&&
+		 (fw->wmhints->flags & StateHint)&&
+		 (fw->wmhints->initial_state == IconicState)))
+	      || (flags.do_honor_starts_on_page)) )
 
 	   ) )
     {
@@ -1055,32 +1078,30 @@ Bool PlaceWindow(
        * were only one page, no virtual desktop), then 2) readjust
        * relative to the current page.
        */
-      if (tmp_win->attr.x < 0)
+      if (attr_g->x < 0)
       {
-	tmp_win->attr.x =
-	  ((Scr.MyDisplayWidth + tmp_win->attr.x) % Scr.MyDisplayWidth);
+	attr_g->x = ((Scr.MyDisplayWidth + attr_g->x) % Scr.MyDisplayWidth);
       }
       else
       {
-	tmp_win->attr.x = tmp_win->attr.x % Scr.MyDisplayWidth;
+	attr_g->x = attr_g->x % Scr.MyDisplayWidth;
       }
       /*
        * Noticed a quirk here. With some apps (e.g., xman), we find the
        * placement has moved 1 pixel away from where we originally put it when
        * we come through here. Why is this happening?
-       * Probably old_bw, try xclock -borderwidth 100
+       * Probably attr_backup.border_width, try xclock -borderwidth 100
        */
-      if (tmp_win->attr.y < 0)
+      if (attr_g->y < 0)
       {
-	tmp_win->attr.y =
-	  ((Scr.MyDisplayHeight + tmp_win->attr.y) % Scr.MyDisplayHeight);
+	attr_g->y = ((Scr.MyDisplayHeight + attr_g->y) % Scr.MyDisplayHeight);
       }
       else
       {
-	tmp_win->attr.y = tmp_win->attr.y % Scr.MyDisplayHeight;
+	attr_g->y = attr_g->y % Scr.MyDisplayHeight;
       }
-      tmp_win->attr.x -= pdeltax;
-      tmp_win->attr.y -= pdeltay;
+      attr_g->x -= pdeltax;
+      attr_g->y -= pdeltay;
     }
 
     /* put it where asked, mod title bar */
@@ -1090,9 +1111,9 @@ Bool PlaceWindow(
       int gravx;
       int gravy;
 
-      gravity_get_offsets(tmp_win->hints.win_gravity, &gravx, &gravy);
-      final_g.x = tmp_win->attr.x + gravx * tmp_win->old_bw;
-      final_g.y = tmp_win->attr.y + gravy * tmp_win->old_bw;
+      gravity_get_offsets(fw->hints.win_gravity, &gravx, &gravy);
+      final_g.x = attr_g->x + gravx * fw->attr_backup.border_width;
+      final_g.y = attr_g->y + gravy * fw->attr_backup.border_width;
       /* Virtually all applications seem to share a common bug: they request
        * the top left pixel of their *border* as their origin instead of the
        * top left pixel of their client window, e.g. 'xterm -g +0+0' creates an
@@ -1101,54 +1122,93 @@ Bool PlaceWindow(
        * at (1 1) instead.  This clearly violates the ICCCM, but trying to
        * change this is like tilting at windmills.  So we have to add the
        * border width here. */
-      final_g.x += tmp_win->old_bw;
-      final_g.y += tmp_win->old_bw;
+      final_g.x += fw->attr_backup.border_width;
+      final_g.y += fw->attr_backup.border_width;
       final_g.width = 0;
       final_g.height = 0;
       if (mode == PLACE_INITIAL)
       {
-        gravity_resize(
-          tmp_win->hints.win_gravity, &final_g,
-          2 * tmp_win->boundary_width,
-          2 * tmp_win->boundary_width + tmp_win->title_g.height);
+	get_window_borders(fw, &b);
+	gravity_resize(
+	  fw->hints.win_gravity, &final_g,
+	  b.total_size.width, b.total_size.height);
       }
-      tmp_win->attr.x = final_g.x;
-      tmp_win->attr.y = final_g.y;
+      attr_g->x = final_g.x;
+      attr_g->y = final_g.y;
     }
   }
 
   return rc;
 }
 
-
+#include "icons.h"
 void CMD_PlaceAgain(F_CMD_ARGS)
 {
-  int x;
-  int y;
-  char *token;
-  float noMovement[1] = {1.0};
-  float *ppctMovement = noMovement;
-  window_style style;
+	char *token;
+	float noMovement[1] = {1.0};
+	float *ppctMovement = noMovement;
+	window_style style;
+	rectangle attr_g;
+	XWindowAttributes attr;
+	Bool do_move_animated = False;
+	Bool do_place_icon = False;
 
-  if (DeferExecution(eventp, &w, &tmp_win, &context, CRS_SELECT,
-		     ButtonRelease))
-    return;
+	if (DeferExecution(
+		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
+	{
+		return;
+	}
+	if (!XGetWindowAttributes(dpy, FW_W(fw), &attr))
+	{
+		return;
+	}
+	while ((token = PeekToken(action, &action)) != NULL)
+	{
+		if (StrEquals("Anim", token))
+		{
+			ppctMovement = NULL;
+			do_move_animated = True;
+		}
+		else if (StrEquals("icon", token))
+		{
+			do_place_icon = True;
+		}
+	}
+	if (IS_ICONIFIED(fw) && do_place_icon)
+	{
+		rectangle new_g;
+		rectangle old_g;
 
-  lookup_style(tmp_win, &style);
-  PlaceWindow(
-    tmp_win, &style.flags,
-    SGET_START_DESK(style), SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
-    SGET_START_SCREEN(style), PLACE_AGAIN);
-  x = tmp_win->attr.x;
-  y = tmp_win->attr.y;
+		if (IS_ICON_SUPPRESSED(fw))
+		{
+			return;
+		}
+		get_icon_geometry(fw, &old_g);
+		SET_ICON_MOVED(fw, 0);
+		AutoPlaceIcon(fw, NULL, False);
+		get_icon_geometry(fw, &new_g);
+		__move_icon(
+			fw, new_g.x, new_g.y, old_g.x, old_g.y,
+			do_move_animated, False);
+	}
+	else
+	{
+		initial_window_options_type win_opts;
 
-  /* Possibly animate the movement */
-  token = PeekToken(action, NULL);
-  if(token && StrEquals("ANIM", token))
-    ppctMovement = NULL;
+		memset(&win_opts, 0, sizeof(win_opts));
+		lookup_style(fw, &style);
+		attr_g.x = attr.x;
+		attr_g.y = attr.y;
+		attr_g.width = attr.width;
+		attr_g.height = attr.height;
+		PlaceWindow(
+			fw, &style.flags, &attr_g, SGET_START_DESK(style),
+			SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
+			SGET_START_SCREEN(style), PLACE_AGAIN, &win_opts);
+		AnimatedMoveFvwmWindow(
+			fw, FW_W_FRAME(fw), -1, -1, attr_g.x, attr_g.y, False,
+			-1, ppctMovement);
+	}
 
-  AnimatedMoveFvwmWindow(tmp_win, tmp_win->frame, -1, -1, x, y, False, -1,
-			 ppctMovement);
-
-  return;
+	return;
 }
