@@ -57,15 +57,16 @@
 #ifdef FANCY_TITLEBARS
 #define TITLE_PADDING 5
 enum tb_pixmap_enum {TBP_MAIN, TBP_LEFT_MAIN, TBP_RIGHT_MAIN, TBP_UNDER_TEXT,
-                     TBP_LEFT_OF_TEXT, TBP_RIGHT_OF_TEXT, TBP_LEFT_END,
-                     TBP_RIGHT_END, TBP_BUTTONS, TBP_LEFT_BUTTONS,
-                     TBP_RIGHT_BUTTONS, NUM_TB_PIXMAPS};
+		     TBP_LEFT_OF_TEXT, TBP_RIGHT_OF_TEXT, TBP_LEFT_END,
+		     TBP_RIGHT_END, TBP_BUTTONS, TBP_LEFT_BUTTONS,
+		     TBP_RIGHT_BUTTONS, NUM_TB_PIXMAPS};
 #endif
 
 typedef struct
 {
   Window win;
   int isMapped;
+  char * command ; /* command which is executed when the pan frame is entered */
 } PanFrame;
 
 typedef enum
@@ -80,9 +81,7 @@ typedef enum
 #ifdef FANCY_TITLEBARS
     MultiPixmap               ,
 #endif
-#ifdef MINI_ICONS
     MiniIconButton            ,
-#endif
     SolidButton
 } DecorFaceType;
 
@@ -98,11 +97,7 @@ typedef enum
 
 typedef struct
 {
-#ifdef FANCY_TITLEBARS
   unsigned face_type : 4;
-#else
-  unsigned face_type : 3; /* was DecorFaceType : 3 */
-#endif
   struct
   {
     unsigned h_justification : 2; /* was JustificationType : 2 */
@@ -133,83 +128,86 @@ typedef struct
 
 typedef struct DecorFace
 {
-  DecorFaceStyle style;
-  struct
-  {
-    Picture *p;
+	DecorFaceStyle style;
+	struct
+	{
+		FvwmPicture *p;
 #ifdef FANCY_TITLEBARS
-    Picture **multi_pixmaps;
-    short multi_stretch_flags;
+		FvwmPicture **multi_pixmaps;
+		short multi_stretch_flags;
 #endif
-    Pixel back;
-    struct
-    {
-      int npixels;
-      Pixel *pixels;
-      char gradient_type;
-    } grad;
-    struct vector_coords
-    {
-      int num;
-      int *x;
-      int *y;
-      unsigned long line_style;
-      unsigned long use_fgbg;
-    } vector;
-  } u;
+		Pixel back;
+		struct
+		{
+			int npixels;
+			XColor *xcs;
+			int do_dither;
+			Pixel *d_pixels;
+			int d_npixels;
+			char gradient_type;
+		} grad;
+		struct vector_coords
+		{
+			int num;
+			signed char *x;
+			signed char *y;
+			signed char *c;
+			unsigned use_fgbg : 1;
+		} vector;
+	} u;
 
-  struct DecorFace *next;
+	struct DecorFace *next;
 
-  struct
-  {
-    unsigned has_changed : 1;
-  } flags;
+	struct
+	{
+		unsigned has_changed : 1;
+	} flags;
 } DecorFace;
 
-enum ButtonState
-{
-  BS_All = -1,
-  BS_ActiveUp,
-  BS_ActiveDown,
-  BS_Inactive,
-  BS_ToggledActiveUp,
-  BS_ToggledActiveDown,
-  BS_ToggledInactive,
-  BS_MaxButtonState
-};
-
 typedef enum
 {
-  /* The first five are used in title buttons.  These can't be renumbered
-   * without extending the mwm_decor_flags member below and adapting the style
-   * structure. */
-  MWM_DECOR_MENU     = 0x1,
-  MWM_DECOR_MINIMIZE = 0x2,
-  MWM_DECOR_MAXIMIZE = 0x4,
-  MWM_DECOR_SHADE    = 0x8,
-  MWM_DECOR_STICK    = 0x10,
-  /* --- */
-  MWM_DECOR_BORDER   = 0x20,
-  MWM_DECOR_RESIZEH  = 0x40,
-  MWM_DECOR_TITLE    = 0x80,
-  MWM_DECOR_ALL      = 0x100,
-  MWM_DECOR_EVERYTHING = 0xff
+	BS_All = -1,
+	BS_ActiveUp,
+	BS_ActiveDown,
+	BS_Inactive,
+	BS_ToggledActiveUp,
+	BS_ToggledActiveDown,
+	BS_ToggledInactive,
+	BS_MaxButtonState
+} ButtonState;
+
+typedef enum
+{
+	/* The first five are used in title buttons.  These can't be
+	 * renumbered without extending the mwm_decor_flags member below and
+	 * adapting the style structure. */
+	MWM_DECOR_MENU     = 0x1,
+	MWM_DECOR_MINIMIZE = 0x2,
+	MWM_DECOR_MAXIMIZE = 0x4,
+	MWM_DECOR_SHADE    = 0x8,
+	MWM_DECOR_STICK    = 0x10,
+	/* --- */
+	MWM_DECOR_BORDER   = 0x20,
+	MWM_DECOR_RESIZEH  = 0x40,
+	MWM_DECOR_TITLE    = 0x80,
+	MWM_DECOR_ALL      = 0x100,
+	MWM_DECOR_EVERYTHING = 0xff
 } mwm_flags;
 
 typedef struct
 {
-  unsigned just : 2; /* was JustificationType : 2 */
-  int layer;
-  struct
-  {
-    unsigned has_changed : 1;
-    mwm_flags mwm_decor_flags : 9;
-    /* Support {ButtonStyle <number> - Layer 4} construction, so button
-     * can be rendered 'pressed in' when the window is assigned to a
-     * particular layer. */
-    unsigned has_layer : 1;
-  } flags;
-  DecorFace state[BS_MaxButtonState];
+	unsigned just : 2; /* was JustificationType : 2 */
+	int layer;
+	struct
+	{
+		unsigned has_changed : 1;
+		mwm_flags mwm_decor_flags : 9;
+		/* Support {ButtonStyle <number> - Layer 4} construction, so
+		 * button can be rendered 'pressed in' when the window is
+		 * assigned to a particular layer. */
+		unsigned has_layer : 1;
+	} flags;
+	DecorFace state[BS_MaxButtonState];
 } TitleButton;
 
 #define TB_FLAGS(tb)              ((tb).flags)
@@ -233,7 +231,7 @@ typedef struct
 typedef struct FvwmDecor
 {
 #ifdef USEDECOR
-  char *tag;			/* general style tag */
+  char *tag;                    /* general style tag */
 #endif
   int title_height;           /* explicitly specified title bar height */
   /* titlebar buttons */
@@ -244,7 +242,7 @@ typedef struct FvwmDecor
     DecorFace active, inactive;
   } BorderStyle;
 #ifdef USEDECOR
-  struct FvwmDecor *next;	/* additional user-defined styles */
+  struct FvwmDecor *next;       /* additional user-defined styles */
 #endif
   struct
   {
@@ -281,12 +279,12 @@ typedef struct ScreenInfo
   unsigned long screen;
   Screen *pscreen;
   int NumberOfScreens;          /* number of screens on display */
-  int MyDisplayWidth;		/* my copy of DisplayWidth(dpy, screen) */
-  int MyDisplayHeight;	        /* my copy of DisplayHeight(dpy, screen) */
+  int MyDisplayWidth;           /* my copy of DisplayWidth(dpy, screen) */
+  int MyDisplayHeight;          /* my copy of DisplayHeight(dpy, screen) */
 
-  FvwmWindow FvwmRoot;		/* the head of the fvwm window list */
-  Window Root;		        /* the root window */
-  Window SizeWindow;		/* the resize dimensions window */
+  FvwmWindow FvwmRoot;          /* the head of the fvwm window list */
+  Window Root;                  /* the root window */
+  Window SizeWindow;            /* the resize dimensions window */
   Window NoFocusWin;            /* Window which will own focus when no other
 				 * windows have it */
   PanFrame PanFrameTop;
@@ -301,11 +299,11 @@ typedef struct ScreenInfo
 
   Binding *AllBindings;
 
-  int root_pushes;		/* current push level to install root
+  int root_pushes;              /* current push level to install root
 				   colormap windows */
-  int fvwm_pushes;		/* current push level to install fvwm
+  int fvwm_pushes;              /* current push level to install fvwm
 				   colormap windows */
-  FvwmWindow *pushed_window;	/* saved window to install when pushes drops
+  FvwmWindow *pushed_window;    /* saved window to install when pushes drops
 				   to zero */
   Cursor *FvwmCursors;
   int BusyCursor;               /* context where we display the busy cursor */
@@ -315,9 +313,9 @@ typedef struct ScreenInfo
   int DefaultLayer;
   int BottomLayer;
 
-  FvwmFunction *functions;
+  struct FvwmFunction *functions;
 
-  FvwmFont DefaultFont;     	/* font structure */
+  FlocaleFont *DefaultFont;             /* font structure */
 
   GC TransMaskGC;               /* GC for transparency masks */
   Pixel StdFore, StdBack, StdHilite, StdShadow; /* don't change the order */
@@ -328,33 +326,37 @@ typedef struct ScreenInfo
   Pixmap ScratchMonoPixmap;     /* A scratch 1x1x1 pixmap */
   GC MonoGC;                    /* GC for drawing into depth 1 drawables */
 
-  GC XorGC;			/* GC to draw lines for move and resize */
+  GC XorGC;                     /* GC to draw lines for move and resize */
   GC ScratchGC1;
   GC ScratchGC2;
   GC ScratchGC3;
   GC ScratchGC4;
   GC TitleGC;
-  int SizeStringWidth;	        /* minimum width of size window */
+  GC TileGC;                    /* Used for tiling (clipmask and tile) */
+  int SizeStringWidth;          /* minimum width of size window */
 
-  FvwmDecor DefaultDecor;	/* decoration style(s) */
+  FvwmDecor DefaultDecor;       /* decoration style(s) */
   FvwmDecor *cur_decor;
 
-  int nr_left_buttons;		/* number of left-side title-bar buttons */
-  int nr_right_buttons;		/* number of right-side title-bar buttons */
+  int nr_left_buttons;          /* number of left-side title-bar buttons */
+  int nr_right_buttons;         /* number of right-side title-bar buttons */
 
-  FvwmWindow *Hilite;		/* the fvwm window that is highlighted
+  FvwmWindow *Hilite;           /* the fvwm window that is highlighted
 				 * except for networking delays, this is the
 				 * window which REALLY has the focus */
   Window UnknownWinFocused;      /* None, if the focus is nowhere or on an fvwm
 				 * managed window. Set to id of otherwindow
 				 * with focus otherwise */
-  FvwmWindow *Ungrabbed;
+  Window StolenFocusWin;        /* The window that the UnknownWinFocused window
+				 * stole the focus from. */
+  FvwmWindow *focus_in_pending_window;
   int EdgeScrollX;              /* #pixels to scroll on screen edge */
   int EdgeScrollY;              /* #pixels to scroll on screen edge */
   unsigned char buttons2grab;   /* buttons to grab in click to focus mode */
   int NumBoxes;
-  int randomx;                  /* values used for randomPlacement */
-  int randomy;
+  int cascade_x;                /* values used for CascadePlacement */
+  int cascade_y;
+  FvwmWindow *cascade_window;
   int VxMax;                    /* Max location for top left of virt desk*/
   int VyMax;
   int Vx;                       /* Current loc for top left of virt desk */
@@ -442,7 +444,8 @@ typedef struct ScreenInfo
 } ScreenInfo;
 
 /* A macro to to simplify he "ewmh desktop code" */
-#define IS_EWMH_DESKTOP(win) (Scr.EwmhDesktop && win == Scr.EwmhDesktop->w)
+#define IS_EWMH_DESKTOP(win) \
+  (Scr.EwmhDesktop && win == Scr.EwmhDesktop->wins.client)
 
 /*
    Macro which gets specific decor or default decor.
