@@ -16,13 +16,14 @@
 #ifndef _VIRTUAL_
 #define _VIRTUAL_
 
-Bool HandlePaging(int, int, int *, int *, int *, int *, Bool, Bool, Bool);
+int HandlePaging(int, int, int *, int *, int *, int *, Bool, Bool, Bool);
 void checkPanFrames(void);
 void raisePanFrames(void);
 void initPanFrames(void);
+Bool is_pan_frame(Window w);
 void MoveViewport(int newx, int newy,Bool);
 void goto_desk(int desk);
-void do_move_window_to_desk(FvwmWindow *tmp_win, int desk);
+void do_move_window_to_desk(FvwmWindow *fw, int desk);
 Bool get_page_arguments(char *action, int *page_x, int *page_y);
 char *GetDesktopName(int desk);
 
