@@ -17,7 +17,6 @@
 
 #include <stdio.h>
 #include <signal.h>
-#include <string.h>
 #include <ctype.h>
 #include <unistd.h>
 
@@ -38,25 +37,33 @@ static Bool is_grabbing_everything = False;
 /* The keys must be in lower case! */
 static struct charstring win_contexts[]=
 {
-  {'w',C_WINDOW},
-  {'d',C_EWMH_DESKTOP},
-  {'t',C_TITLE},
-  {'i',C_ICON},
-  {'r',C_ROOT},
-  {'f',C_FRAME},
-  {'s',C_SIDEBAR},
-  {'1',C_L1},
-  {'2',C_R1},
-  {'3',C_L2},
-  {'4',C_R2},
-  {'5',C_L3},
-  {'6',C_R3},
-  {'7',C_L4},
-  {'8',C_R4},
-  {'9',C_L5},
-  {'0',C_R5},
-  {'a',C_ALL},
-  {0,0}
+  {'d', C_EWMH_DESKTOP},
+  {'f', C_FRAME},
+  {'<', C_F_TOPLEFT},
+  {'^', C_F_TOPRIGHT},
+  {'>', C_F_BOTTOMRIGHT},
+  {'v', C_F_BOTTOMLEFT},
+  {'i', C_ICON},
+  {'r', C_ROOT},
+  {'s', C_SIDEBAR},
+  {'[', C_SB_LEFT},
+  {']', C_SB_RIGHT},
+  {'-', C_SB_TOP},
+  {'_', C_SB_BOTTOM},
+  {'t', C_TITLE},
+  {'w', C_WINDOW},
+  {'1', C_L1},
+  {'2', C_R1},
+  {'3', C_L2},
+  {'4', C_R2},
+  {'5', C_L3},
+  {'6', C_R3},
+  {'7', C_L4},
+  {'8', C_R4},
+  {'9', C_L5},
+  {'0', C_R5},
+  {'a', C_ALL},
+  {0, 0}
 };
 
 /* The keys musat be in lower case! */
@@ -76,6 +83,22 @@ static struct charstring key_modifiers[]=
   {0,0}
 };
 
+/* Table to translate a modifier map index to a modifier that we define that
+ * generates that index.  This mapping can be chosen by each client, but the
+ * settings below try to emulate the usual terminal behaviour. */
+static unsigned int modifier_mapindex_to_mask[8] =
+{
+	ShiftMask,
+	Mod3Mask, /* Alt Gr */
+	Mod3Mask | ShiftMask,
+	/* Just guessing below here - LockMask is not used anywhere*/
+	ControlMask,
+	Mod1Mask, /* Alt/Meta on XFree86 */
+	Mod2Mask, /* Num lock on XFree86 */
+	Mod4Mask,
+	Mod5Mask, /* Scroll lock on XFree86 */
+};
+
 static int key_min;
 static int key_max;
 
@@ -85,8 +108,8 @@ static int key_max;
  * true/false values (bits)
  *
  ****************************************************************************/
-static Bool find_context( const char *string, int *output,
-			  struct charstring *table )
+static Bool find_context(
+	const char *string, int *output, struct charstring *table)
 {
   int i;
   int len = strlen( string );
@@ -232,7 +255,7 @@ Bool RemoveMatchingBinding(
   for (t = *pblist, prev = NULL; t; prev = t, t = t->NextBinding)
   {
     if (MatchBindingExactly(t, STROKE_ARG(stroke)
-                            button, keycode, modifiers, contexts, type))
+			    button, keycode, modifiers, contexts, type))
     {
       /* found a matching binding - remove it */
       UnlinkBinding(pblist, t, prev);
@@ -263,7 +286,7 @@ Bool MatchBinding(Display *dpy, Binding *b,
   static int kc_maxlen = 0;
   static KeySym last_keysym = NoSymbol;
 
-  modifier &= used_modifiers;
+  modifier &= (used_modifiers & ALL_MODIFIERS);
   if (((b->Modifier & used_modifiers) != modifier &&
        b->Modifier != AnyModifier) ||
       (b->Context & Context) == 0 ||
@@ -350,6 +373,7 @@ int AddBinding(
   int max;
   int maxmods;
   int m;
+  int mask;
   int count = 0;
   KeySym tkeysym;
   Binding *temp;
@@ -401,24 +425,23 @@ int AddBinding(
 	  if (modifiers != AnyModifier && !(modifiers & ShiftMask))
 	  {
 	    add_modifiers = ShiftMask;
-	    bind_mask = 2;
-	    /* but don't bind it again if already bound without modifiers */
-	    check_bound_mask = 1;
-	  }
-	  break;
-	case 2:
-	  /* key generates the key sym with caps-lock depressed */
-	  if (modifiers != AnyModifier && !(modifiers & LockMask))
-	  {
-	    add_modifiers = LockMask;
-	    bind_mask = 4;
+	    bind_mask = (1 << m);
 	    /* but don't bind it again if already bound without modifiers */
 	    check_bound_mask = 1;
 	  }
 	  break;
 	default:
-	  /* key generates the key sym with unknown modifiers depressed -
-	  * can't map that to specific modifiers - trat as no modifiers */
+	  /* key generates the key sym with undefined modifiers depressed -
+	   * let's make an educated guess at what modifiers the user expected
+	   * based on the XFree86 default configuration. */
+	  mask = modifier_mapindex_to_mask[m - 1];
+	  if (modifiers != AnyModifier && !(modifiers & mask) != mask)
+	  {
+	    add_modifiers = mask;
+	    bind_mask = (1 << m);
+	    /* but don't bind it again if already bound without modifiers */
+	    check_bound_mask = 1;
+	  }
 	  break;
 	}
 	if ((bind_mask & bound_mask) || (check_bound_mask & bound_mask))
@@ -441,17 +464,6 @@ int AddBinding(
 	(*pblist)->Action = (action) ? stripcpy(action) : NULL;
 	(*pblist)->Action2 = (action2) ? stripcpy(action2) : NULL;
 	(*pblist)->NextBinding = temp;
-#if 0
-        fprintf(stderr,
-   "Bindings: added binding type %c, key %X, key name %s, ctx %X mod %X act1 %s, act2 %s\n",
-                (*pblist)->type,
-                (*pblist)->Button_Key,
-                (*pblist)->key_name ? (*pblist)->key_name : "",
-                (*pblist)->Context,
-                (*pblist)->Modifier,
-                (*pblist)->Action ? (*pblist)->Action : "",
-                (*pblist)->Action2 ? (*pblist)->Action2 : "");
-#endif
 	bound_mask |= bind_mask;
 	count++;
       }
@@ -551,7 +563,7 @@ void *CheckBinding(Binding *blist,
   Binding *b;
   unsigned int used_modifiers = ~dead_modifiers;
 
-  modifier &= used_modifiers;
+  modifier &= (used_modifiers & ALL_MODIFIERS);
 
   for (b = blist; b != NULL; b = b->NextBinding)
   {
@@ -580,18 +592,19 @@ Bool MatchBindingExactly(
   int button, KeyCode keycode, unsigned int modifier, int Context,
   BindingType type)
 {
+  modifier &= ALL_MODIFIERS;
   if (b->type == type)
   {
     if (
-        (
-          (type == KEY_BINDING && b->Button_Key == keycode) ||
-          STROKE_CODE(
-            (type == STROKE_BINDING && b->Button_Key == button &&
-             (strcmp(b->Stroke_Seq,stroke) == 0)) ||
-              )
-          (type == MOUSE_BINDING && b->Button_Key == button)) &&
-        (b->Context == Context) &&
-        (b->Modifier == modifier))
+	(
+	  (type == KEY_BINDING && b->Button_Key == keycode) ||
+	  STROKE_CODE(
+	    (type == STROKE_BINDING && b->Button_Key == button &&
+	     (strcmp(b->Stroke_Seq,stroke) == 0)) ||
+	      )
+	  (type == MOUSE_BINDING && b->Button_Key == button)) &&
+	(b->Context == Context) &&
+	(b->Modifier == modifier))
     {
       return True;
     }
@@ -603,12 +616,12 @@ Bool MatchBindingExactly(
 /***********************************************************************
  *
  *  Procedure:
- *	GrabWindowKey        - grab needed keys for the window for one binding
- *	GrabAllWindowKeys    - grab needed keys for the window for all bindings
+ *      GrabWindowKey        - grab needed keys for the window for one binding
+ *      GrabAllWindowKeys    - grab needed keys for the window for all bindings
  *                             in blist
- *	GrabWindowButton     - same for mouse buttons
- *	GrabAllWindowButtons - same for mouse buttons
- *	GrabAllWindowKeysAndButtons - both of the above
+ *      GrabWindowButton     - same for mouse buttons
+ *      GrabAllWindowButtons - same for mouse buttons
+ *      GrabAllWindowKeysAndButtons - both of the above
  *
  *  Inputs:
  *   w              - the window to use (the frame window)
@@ -626,6 +639,7 @@ void GrabWindowKey(Display *dpy, Window w, Binding *binding,
 {
   /* remove unnecessary bits from dead_modifiers */
   dead_modifiers &= ~(binding->Modifier & dead_modifiers);
+  dead_modifiers &= ALL_MODIFIERS;
 
   if((binding->Context & contexts) && (binding->type == KEY_BINDING))
   {
@@ -684,6 +698,7 @@ void GrabWindowButton(Display *dpy, Window w, Binding *binding,
     return;
 
   dead_modifiers &= ~(binding->Modifier & dead_modifiers); /* dje */
+  dead_modifiers &= ALL_MODIFIERS;
 
   if ((binding->Context & contexts) &&
       ((binding->type == MOUSE_BINDING)
@@ -804,7 +819,7 @@ void GrabWindowKeyOrButton(
 /***********************************************************************
  *
  *  Procedure:
- *	FvwmStringToKeysym
+ *      FvwmStringToKeysym
  *
  *  Like XStringToKeysym, but allows some typos and does some additional
  *  error checking.
