@@ -29,8 +29,9 @@
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
 #include "libs/Colorset.h"
-#include "libs/Picture.h"
 #include "libs/safemalloc.h"
+#include "libs/PictureBase.h"
+#include "libs/PictureGraphics.h"
 
 /* globals */
 colorset_struct *Colorset = NULL;
@@ -51,21 +52,15 @@ void AllocColorset(int n)
   Colorset = (colorset_struct *)saferealloc((char *)Colorset,
 					    ++n * sizeof(colorset_struct));
 
-  /* zero out the new members */
-  memset(&Colorset[nColorsets], 0, (n - nColorsets) * sizeof(colorset_struct));
-
-  /* copy colorset 0 pixels into new members so that if undefined ones are
-   * referenced at least they don't give black on black */
-  if (n > 1)
-  {
+  /* zero out colorset 0
+     it's always defined so will be filled in during module startup */
+  if (n == 0)
+    memset(&Colorset[nColorsets], 0, (n - nColorsets) * sizeof(colorset_struct));
+  else
+    /* copy colorset 0 into new members so that if undefined ones are
+     * referenced at least they don't give black on black */
     for ( ; nColorsets < n; nColorsets++)
-    {
-      Colorset[nColorsets].fg = Colorset[0].fg;
-      Colorset[nColorsets].bg = Colorset[0].bg;
-      Colorset[nColorsets].hilite = Colorset[0].hilite;
-      Colorset[nColorsets].shadow = Colorset[0].shadow;
-    }
-  }
+      memcpy(&Colorset[nColorsets], Colorset, sizeof(colorset_struct));
 
   nColorsets = n;
 }
@@ -73,14 +68,16 @@ void AllocColorset(int n)
 /*****************************************************************************
  * DumpColorset() returns a char * to the colorset contents in printable form
  *****************************************************************************/
-static char csetbuf[160];
+static char csetbuf[184];
 char *DumpColorset(int n, colorset_struct *cs)
 {
   sprintf(csetbuf,
-	  "Colorset %x %lx %lx %lx %lx %lx %lx %x %x %x %x %x %x",
-	  n, cs->fg, cs->bg, cs->hilite, cs->shadow, cs->pixmap,
-	  cs->shape_mask, cs->width, cs->height, cs->pixmap_type,
-	  cs->shape_width, cs->shape_height, cs->shape_type);
+	  "Colorset "
+	  "%x %lx %lx %lx %lx %lx %lx %lx %lx %x %x %x %x %x %x %x %x %x %x",
+	  n, cs->fg, cs->bg, cs->hilite, cs->shadow, cs->fgsh, cs->icon_tint,
+	  cs->pixmap, cs->shape_mask, cs->fg_alpha, cs->width, cs->height,
+	  cs->pixmap_type, cs->shape_width, cs->shape_height, cs->shape_type,
+	  cs->do_dither_icon, cs->icon_tint_percent, cs->icon_alpha);
   return csetbuf;
 }
 
@@ -91,35 +88,48 @@ int LoadColorset(char *line)
 {
   colorset_struct *cs;
   unsigned int n, chars;
-  Pixel fg, bg, hilite, shadow;
+  Pixel fg, bg, hilite, shadow, fgsh, icon_tint;
   Pixmap pixmap;
   Pixmap shape_mask;
-  unsigned int width, height, pixmap_type;
+  unsigned int fg_alpha, width, height, pixmap_type;
   unsigned int shape_width, shape_height, shape_type;
+  unsigned int do_dither_icon, icon_tint_percent, icon_alpha;
 
   if (line == NULL)
     return -1;
   if (sscanf(line, "%x%n", &n, &chars) < 1)
     return -1;
   line += chars;
-  if (sscanf(line, "%lx %lx %lx %lx %lx %lx %x %x %x %x %x %x",
-	     &fg, &bg, &hilite, &shadow, &pixmap, &shape_mask, &width, &height,
-	     &pixmap_type, &shape_width, &shape_height, &shape_type) != 12)
+  if (sscanf(line,
+	     "%lx %lx %lx %lx %lx %lx %lx %lx " 
+	     "%x %x %x %x %x %x %x %x %x %x",
+	     &fg, &bg, &hilite, &shadow, &fgsh, &icon_tint, &pixmap, &shape_mask,
+	     &fg_alpha, &width, &height, &pixmap_type, &shape_width,
+	     &shape_height, &shape_type, &do_dither_icon, &icon_tint_percent,
+	     &icon_alpha) != 18)
     return -1;
+
   AllocColorset(n);
   cs = &Colorset[n];
   cs->fg = fg;
   cs->bg = bg;
   cs->hilite = hilite;
   cs->shadow = shadow;
+  cs->fgsh = fgsh;
+  cs->icon_tint = icon_tint;
   cs->pixmap = pixmap;
   cs->shape_mask = shape_mask;
+  cs->fg_alpha = fg_alpha;
   cs->width = width;
   cs->height = height;
   cs->pixmap_type = pixmap_type;
   cs->shape_width = shape_width;
   cs->shape_height = shape_height;
   cs->shape_type = shape_type;
+  cs->do_dither_icon = do_dither_icon;
+  cs->icon_tint_percent = icon_tint_percent;
+  cs->icon_alpha = icon_alpha;
+
   return n;
 }
 
@@ -335,7 +345,7 @@ Pixmap CreateBackgroundPixmap(Display *dpy, Window win, int width, int height,
     /* make a pixmap big enough to cover the destination but with the aspect
      * ratio of the cs_pixmap */
     big_width = trim_side ? height * cs_width / cs_height : width;
-    big_height = trim_side ? height : width * cs_height / cs_width;
+    big_height = trim_side ? height : width * cs_width / cs_width;
     big_pixmap = CreateStretchPixmap(
       dpy, cs_pixmap, cs_width, cs_height, depth, big_width, big_height, gc);
 
