@@ -20,6 +20,7 @@
 */
 
 #include <X11/X.h>
+#include "libs/fvwmlib.h"
 
 
 /**
@@ -39,6 +40,11 @@ typedef struct {
 } FvwmPacket;
 
 
+/*
+ * If you modify constants here, please regenerate Constants.pm in perllib.
+ */
+
+
 /** All size values in units of "unsigned long" **/
 #define FvwmPacketHeaderSize        4
 #define FvwmPacketBodySize(p)       ((p).size - FvwmPacketHeaderSize)
@@ -56,6 +62,8 @@ typedef struct {
 /* Value of start_pattern */
 #define START_FLAG 0xffffffff
 
+#define ModuleFinishedStartupResponse "NOP FINISHED STARTUP"
+#define ModuleUnlockResponse          "NOP UNLOCK"
 
 /* Possible values of type */
 #define M_NEW_PAGE               (1)
@@ -111,12 +119,18 @@ typedef struct {
 #define MX_VISIBLE_ICON_NAME      ((1<<0) | M_EXTENDED_MSG)
 #define MX_ENTER_WINDOW           ((1<<1) | M_EXTENDED_MSG)
 #define MX_LEAVE_WINDOW           ((1<<2) | M_EXTENDED_MSG)
-#define MAX_EXTENDED_MESSAGES     3
+#define MX_PROPERTY_CHANGE        ((1<<3) | M_EXTENDED_MSG)
+#define MAX_EXTENDED_MESSAGES     4
 #define DEFAULT_XMSG_MASK         0x00000000
-#define MAX_XMSG_MASK             0x00000007
+#define MAX_XMSG_MASK             0x0000000f
 
 #define MAX_TOTAL_MESSAGES   (MAX_MESSAGES + MAX_EXTENDED_MESSAGES)
 
+/* for MX_PROPERTY_CHANGE */
+#define MX_PROPERTY_CHANGE_NONE        0
+#define MX_PROPERTY_CHANGE_BACKGROUND  1
+#define MX_PROPERTY_CHANGE_SWALLOW     2
+
 /**
  * Reads a single packet of info from FVWM.
  * The packet is stored into static memory that is reused during
@@ -215,6 +229,11 @@ void InitGetConfigLine(int *fd, char *match);
 void GetConfigLine(int *fd, char **line);
 
 
+/* expands certain variables in a command to be sent by a module */
+char *module_expand_action(
+	Display *dpy, int screen , char *in_action, rectangle *r,
+	char *forecolor, char *backcolor);
+
 /**
  * Parse the command line arguments given to the module by FVWM.
  * Input is the argc & argv from main(), and a flag to indicate
