@@ -30,7 +30,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/wait.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -43,36 +43,38 @@
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 
-#include <X11/xpm.h>
-
 #include <libs/fvwmlib.h>
 #include <libs/Picture.h>
 #include <libs/Module.h>
 #include <libs/FScreen.h>
 #include <libs/FShape.h>
-
+#include <libs/FRenderInit.h>
 
 /* migo (16-Sep-1999): How about to do this configurable? */
 /*#include "fvwm-logo-old.xpm"*/
 /*#include "fvwm-logo-colorful.xpm"*/
 #include "fvwm-logo-current.xpm"
+#include "fvwm-logo-current.xbm"
 
-typedef struct _XpmIcon {
-    Pixmap pixmap;
-    Pixmap mask;
-    XpmAttributes attributes;
-}        XpmIcon;
+typedef struct _PImageIcon {
+	Pixmap pixmap;
+	Pixmap mask;
+	int width;
+	int height;
+	int depth;
+}        PImageIcon;
 
 /**************************************************************************
  * A few function prototypes
  **************************************************************************/
 void RedrawWindow(void);
 void GetXPMData(char **);
-void GetXPMFile(char *,char *);
+void GetXBMData(void);
+void GetImageFile(char *,char *);
 void change_window_name(char *str);
 static void parseOptions (int fd[2]);
 
-XpmIcon view;
+PImageIcon view;
 Window win;
 
 char *imagePath = NULL;
@@ -82,7 +84,7 @@ static int MyNameLen;
 
 int timeout = 3000000; /* default time of 3 seconds */
 
-Display *dpy;			/* which display are we talking to */
+Display *dpy;                   /* which display are we talking to */
 Window Root;
 int screen;
 int x_fd;
@@ -114,8 +116,8 @@ int main(int argc, char **argv)
   string = strrchr (argv[0], '/');
   if (string != (char *) 0) string++;
 
-  MyNameLen=strlen(string)+1;		/* account for '*' */
-  MyName = safemalloc(MyNameLen+1);	/* account for \0 */
+  MyNameLen=strlen(string)+1;           /* account for '*' */
+  MyName = safemalloc(MyNameLen+1);     /* account for \0 */
   *MyName='*';
   strcpy (MyName+1, string);
 
@@ -149,9 +151,10 @@ int main(int argc, char **argv)
   Root = RootWindow(dpy, screen);
   x_fd = XConnectionNumber(dpy);
 
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   FShapeInit(dpy);
+  FRenderInit(dpy);
   parseOptions(fd);
 
   /* chick in the neck situation:
@@ -169,16 +172,34 @@ int main(int argc, char **argv)
 
   /* Get the xpm banner */
   if (imageName)
-    GetXPMFile(imageName,imagePath);
+    GetImageFile(imageName,imagePath);
   else
     GetXPMData(fvwm_logo_xpm);
+
+  if (view.depth != Pdepth)
+  {
+	  XGCValues gcv;
+	  GC gc;
+	  Pixmap temp;
+
+	  gcv.background= WhitePixel(dpy, screen);
+	  gcv.foreground= BlackPixel(dpy, screen);
+	  gc = fvwmlib_XCreateGC(dpy, win,
+				 GCForeground | GCBackground, &gcv);
+	  temp = XCreatePixmap(dpy, win, view.width, view.height, Pdepth);
+	  XCopyPlane(dpy, view.pixmap, temp, gc,
+		     0,0,view.width,view.height, 0,0,1);
+	  XFreePixmap(dpy, view.pixmap);
+	  XFreeGC(dpy,gc);
+	  view.pixmap = temp;
+  }
   XSetWindowBackgroundPixmap(dpy, win, view.pixmap);
 
   /* Create a window to hold the banner */
   mysizehints.flags = USSize | USPosition | PWinGravity | PResizeInc
 		     | PBaseSize | PMinSize | PMaxSize;
-  mysizehints.width = view.attributes.width;
-  mysizehints.height = view.attributes.height;
+  mysizehints.width = view.width;
+  mysizehints.height = view.height;
   mysizehints.width_inc = 1;
   mysizehints.height_inc = 1;
   mysizehints.base_height = mysizehints.height;
@@ -191,7 +212,7 @@ int main(int argc, char **argv)
 
   FScreenCenterOnScreen(
     NULL, FSCREEN_PRIMARY, &mysizehints.x, &mysizehints.y,
-    view.attributes.width, view.attributes.height);
+    view.width, view.height);
 
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
@@ -261,45 +282,60 @@ int main(int argc, char **argv)
  * Looks for a color XPM icon file
  *
  ****************************************************************************/
-void GetXPMData(char **data)
+void GetXBMData(void)
 {
-  view.attributes.valuemask = XpmReturnPixels | XpmCloseness | XpmExtensions
-			      | XpmVisual | XpmColormap | XpmDepth;
-  view.attributes.closeness = 40000 /* Allow for "similar" colors */;
-  view.attributes.visual = Pvisual;
-  view.attributes.colormap = Pcmap;
-  view.attributes.depth = Pdepth;
-  if(XpmCreatePixmapFromData(dpy, win, data, &view.pixmap, &view.mask,
-			     &view.attributes)!=XpmSuccess)
+  if ((view.pixmap =
+       XCreatePixmapFromBitmapData(dpy, win, (char *)fvwm_logo_xbm_bits,
+				   fvwm_logo_xbm_width, fvwm_logo_xbm_height,
+				   BlackPixel(dpy,screen),
+				   WhitePixel(dpy,screen),
+				   Pdepth)) == None)
   {
     fprintf(stderr,"FvwmBanner: ERROR couldn't convert data to pixmap\n");
     exit(1);
   }
+  view.width = fvwm_logo_xbm_width;
+  view.height = fvwm_logo_xbm_height;
+  view.depth = Pdepth;
+  view.mask = None;
 }
-void GetXPMFile(char *file, char *path)
+
+void GetXPMData(char **data)
 {
-  char *full_file = NULL;
-
-  view.attributes.valuemask = XpmReturnPixels | XpmCloseness | XpmExtensions
-			      | XpmVisual | XpmColormap | XpmDepth;
-  view.attributes.closeness = 40000 /* Allow for "similar" colors */;
-  view.attributes.visual = Pvisual;
-  view.attributes.colormap = Pcmap;
-  view.attributes.depth = Pdepth;
-
-  if (file)
-    full_file = findImageFile(file,path,R_OK);
-
-  if (full_file)
+  if(!PImageLoadPixmapFromXpmData(dpy, win, 0, data,
+				  &view.pixmap, &view.mask,
+				  &view.width, &view.height,
+				  &view.depth))
   {
-    if(XpmReadFileToPixmap(dpy, Root, full_file, &view.pixmap, &view.mask,
-			   &view.attributes) == XpmSuccess)
-      return;
-    fprintf(stderr,"FvwmBanner: ERROR reading pixmap file\n");
+    GetXBMData();
   }
-  else
-    fprintf(stderr,"FvwmBanner: ERROR finding pixmap file in ImagePath\n");
-  GetXPMData(fvwm_logo_xpm);
+}
+
+void GetImageFile(char *file, char *path)
+{
+	char *full_file = NULL;
+	FvwmPictureAttributes fpa;
+
+	fpa.mask = FPAM_NO_ALLOC_PIXELS | FPAM_DITHER | FPAM_NO_ALPHA;
+	if (file)
+		full_file = PictureFindImageFile(file,path,R_OK);
+
+	if (full_file)
+	{
+		if(PImageLoadPixmapFromFile(
+			dpy, Root, full_file, &view.pixmap, &view.mask, None,
+			&view.width, &view.height, &view.depth, 0, NULL, fpa))
+		{
+			return;
+		}
+		fprintf(stderr,"FvwmBanner: ERROR loading image file\n");
+	}
+	else
+	{
+		fprintf(stderr,
+			"FvwmBanner: ERROR finding image file in ImagePath\n");
+		GetXPMData(fvwm_logo_xpm);
+	}
 }
 
 void nocolor(char *a, char *b)
@@ -325,18 +361,18 @@ static void parseOptions(int fd[2])
       }
       if (StrEquals(tok, "ImagePath"))
       {
-        CopyString(&imagePath, tline);
-        if (imagePath[0] == 0)
+	CopyString(&imagePath, tline);
+	if (imagePath[0] == 0)
 	{
-          free (imagePath);
-          imagePath = (char *) 0;
-        }
-        continue;
+	  free (imagePath);
+	  imagePath = (char *) 0;
+	}
+	continue;
       }
       if (strncasecmp(tok,MyName,MyNameLen))
       {
 	/* if not for me: ignore it*/
-        continue;
+	continue;
       }
       /* start of interesting part */
       tok += MyNameLen;
@@ -351,17 +387,17 @@ static void parseOptions(int fd[2])
 	    imageName = (char *) 0;
 	  }
 	}
-        continue;
+	continue;
       }
       if (StrEquals(tok, "NoDecor"))
       {
-        no_wm = True;
-        continue;
+	no_wm = True;
+	continue;
       }
       if (StrEquals(tok, "Timeout"))
       {
-        timeout = atoi(tline) * 1000000;
-        continue;
+	timeout = atoi(tline) * 1000000;
+	continue;
       }
     }
   }
@@ -389,7 +425,7 @@ void change_window_name(char *str)
 /***********************************************************************
  *
  *  Procedure:
- *	SIGPIPE handler - SIGPIPE means fvwm is dying
+ *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
  ***********************************************************************/
 
