@@ -45,13 +45,13 @@
 
 /* ---------------------------- compatibility ------------------------------ */
 
-#define OLD_EXPOSE     /* Try this if resizing/exposes screw up */
+/*#define OLD_EXPOSE*/     /* Try this if resizing/exposes screw up */
 
 /* -------------------------------- more  ---------------------------------- */
 
 #include "libs/fvwmlib.h"
 #include "libs/Picture.h"
-
+#include "libs/Flocale.h"
 
 /* ------------------------------- structs --------------------------------- */
 
@@ -83,17 +83,20 @@
 #define b_ColorsetParent 0x00400000 /* Parent has a colorset background */
 #define b_Panel      0x00800000 /* similar to swallow, but drawn differently */
 #define b_ActionIgnoresClientWindow \
-                     0x01000000 /* Actions work only on the background of a
+		     0x01000000 /* Actions work only on the background of a
 				 * button with a swallowed app. */
+#define b_ActionOnPress \
+		     0x02000000 /* By default this only done on Popup */
 
 /* Flags for b->swallow */
-#define b_Count       0x03 /* Init counter for swallowing */
-#define b_NoHints     0x04 /* Ignore window hints from swallowed window */
-#define b_NoClose     0x08 /* Don't close window when exiting, unswallow it */
-#define b_Kill        0x10 /* Don't close window when exiting, kill it */
-#define b_Respawn     0x20 /* Respawn if swallowed window dies */
-#define b_UseOld      0x40 /* Try to capture old window, don't spawn it */
-#define b_UseTitle    0x80 /* Allow window to write to b->title */
+#define b_Count       0x0003 /* Init counter for swallowing */
+#define b_NoHints     0x0004 /* Ignore window hints from swallowed window */
+#define b_NoClose     0x0008 /* Don't close window when exiting, unswallow it */
+#define b_Kill        0x0010 /* Don't close window when exiting, kill it */
+#define b_Respawn     0x0020 /* Respawn if swallowed window dies */
+#define b_UseOld      0x0040 /* Try to capture old window, don't spawn it */
+#define b_UseTitle    0x0080 /* Allow window to write to b->title */
+#define b_FvwmModule  0x0100 /* consider the swallowed app as module */
 
 /* Flags for b->justify */
 #define b_TitleHoriz  0x03 /* Mask for title x positioning info */
@@ -117,17 +120,14 @@ struct container_info_struct
   int width;
   int height;
 
-  unsigned long flags;     /* Which data are set in this container? */
-  byte justify;            /* b_Justify */
-  byte justify_mask;       /* b_Justify */
-  byte swallow;            /* b_Swallow */
-  byte swallow_mask;       /* b_Swallow */
-  byte xpad,ypad;          /* b_Padding */
-  signed char framew;             /* b_Frame */
-  XFontStruct *font;       /* b_Font */
-#ifdef I18N_MB
-  XFontSet fontset;        /* b_Font */
-#endif
+  unsigned long flags;       /* Which data are set in this container? */
+  byte justify;              /* b_Justify */
+  byte justify_mask;         /* b_Justify */
+  unsigned int swallow;      /* b_Swallow */
+  unsigned int swallow_mask; /* b_Swallow */
+  byte xpad,ypad;            /* b_Padding */
+  signed char framew;        /* b_Frame */
+  FlocaleFont *Ffont;        /* b_Font */
   char *font_string;       /* b_Font */
   char *back;              /* b_Back && !b_IconBack */
   char *back_file;         /* b_Back && b_IconBack */
@@ -135,7 +135,7 @@ struct container_info_struct
   int colorset;            /* b_Colorset */
   Pixel fc;                /* b_Fore */
   Pixel bc,hc,sc;          /* b_Back && !b_IconBack */
-  Picture *backicon;       /* b_Back && b_IconBack */
+  FvwmPicture *backicon;   /* b_Back && b_IconBack */
   ushort minx,miny;        /* b_Size */
 };
 
@@ -161,10 +161,7 @@ struct button_info_struct
   int n;                   /* number in parent */
 
   /* conditional fields */ /* applicable if these flags are set */
-  XFontStruct *font;       /* b_Font */
-#ifdef I18N_MB
-  XFontSet fontset;        /* b_Font */
-#endif
+  FlocaleFont *Ffont;      /* b_Font */
   char *font_string;       /* b_Font */
   char *back;              /* b_Back */
   char *fore;              /* b_Fore */
@@ -181,20 +178,20 @@ struct button_info_struct
   Pixel fc;                /* b_Fore */
   Pixel bc,hc,sc;          /* b_Back && !b_IconBack */
   ushort minx,miny;        /* b_Size */
-  Picture *icon;           /* b_Icon */
-  Picture *backicon;       /* b_Back && b_IconBack */
+  FvwmPicture *icon;       /* b_Icon */
+  FvwmPicture *backicon;   /* b_Back && b_IconBack */
   Window IconWin;          /* b_Icon || b_Swallow */
   Window PanelWin;         /* b_Panel */
   Window BackIconWin;      /* b_Back && b_IconBack */
 
-  byte swallow;            /* b_Swallow */
-  byte swallow_mask;       /* b_Swallow */
-  int icon_w,icon_h;       /* b_Swallow */
-  Window IconWinParent;    /* b_Swallow */
-  XSizeHints *hints;       /* b_Swallow && !b_NoHints */
-  char *spawn;             /* b_Swallow */
-  int x,y;                 /* b_Swallow */
-  ushort w,h,bw;           /* b_Swallow */
+  unsigned int swallow;       /* b_Swallow */
+  unsigned int swallow_mask;  /* b_Swallow */
+  int icon_w,icon_h;          /* b_Swallow */
+  Window IconWinParent;       /* b_Swallow */
+  XSizeHints *hints;          /* b_Swallow && !b_NoHints */
+  char *spawn;                /* b_Swallow */
+  int x,y;                    /* b_Swallow */
+  ushort w,h,bw;              /* b_Swallow */
 
   struct
   {
@@ -253,15 +250,7 @@ extern int fd[];
 extern int new_desk;
 extern GC  NormalGC;
 extern GC  ShadowGC;
-
-/* ------------------------------ I18N_MB stuff ---------------------------- */
-#ifdef I18N_MB
-#ifdef __STDC__
-#define XTextWidth(x,y,z)       XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z)       XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
+extern FlocaleWinString *FwinString;
 
 /* ---------------------------------- misc --------------------------------- */
 
