@@ -28,9 +28,11 @@
 #include "libs/fvwmlib.h"
 
 #define MYNAME "FvwmCommand"
+#define MAXHOSTNAME 32
 
 static int  Fdr, Fdw;  /* file discriptor for fifo */
-static int  Fdrun;
+static FILE *Frun;     /* File contains pid */
+static struct stat stat_buf;
 static char *Fr_name;
 static fd_set fdset;
 
@@ -44,12 +46,14 @@ static int  Opt_flags;
 
 volatile sig_atomic_t  Bg;  /* FvwmCommand in background */
 
+static char hostname[MAXHOSTNAME];
 
 void err_msg( const char *msg );
 void err_quit( const char *msg );
 void sendit( char *cmd );
 void receive( void );
 static RETSIGTYPE sig_ttin(int);
+/* void sig_pipe( int ); */
 static RETSIGTYPE sig_quit(int);
 void usage(void);
 int  read_f(int fd, char *p, int len);
@@ -86,7 +90,7 @@ int main ( int argc, char *argv[])
   int  Rc;
   struct timeval tv2;
   extern char *optarg;
-  extern int  optind, opterr, optopt;
+  extern int  optind;
 
 #ifdef HAVE_SIGACTION
   {
@@ -117,18 +121,16 @@ int main ( int argc, char *argv[])
     sigaction(SIGHUP, &sigact, NULL);
     sigaction(SIGQUIT, &sigact, NULL);
     sigaction(SIGTERM, &sigact, NULL);
-    sigaction(SIGPIPE, &sigact, NULL);
   }
 #else
 #ifdef USE_BSD_SIGNALS
   fvwmSetSignalMask( sigmask(SIGINT) | sigmask(SIGHUP) |
-                     sigmask(SIGQUIT) | sigmask(SIGTERM) );
+		     sigmask(SIGQUIT) | sigmask(SIGTERM) );
 #endif
   signal(SIGINT, sig_quit);
   signal(SIGHUP, sig_quit);
   signal(SIGQUIT, sig_quit);
   signal(SIGTERM, sig_quit);
-  signal(SIGPIPE, sig_quit);
   signal(SIGTTIN, sig_ttin);
   signal(SIGTTOU, sig_ttin);
 #ifdef HAVE_SIGINTERRUPT
@@ -136,7 +138,6 @@ int main ( int argc, char *argv[])
   siginterrupt(SIGHUP, 1);
   siginterrupt(SIGQUIT, 1);
   siginterrupt(SIGTERM, 1);
-  siginterrupt(SIGPIPE, 1);
   siginterrupt(SIGTTIN, 0);
   siginterrupt(SIGTTOU, 0);
 #endif
@@ -200,12 +201,29 @@ int main ( int argc, char *argv[])
 
   if( f_stem == NULL )
   {
-    if ((f_stem = fifos_get_default_name()) == NULL)
+    char *dpy_name;
+
+    /* default name */
+    dpy_name = getenv("DISPLAY");
+    if (!dpy_name)
+      dpy_name = ":0";
+    if (strncmp(dpy_name, "unix:", 5) == 0)
+      dpy_name += 4;
+    f_stem = safemalloc(11 + strlen(F_NAME) + MAXHOSTNAME + strlen(dpy_name));
+
+    if ((stat("/var/tmp", &stat_buf) == 0) && (stat_buf.st_mode & S_IFDIR))
+      strcpy (f_stem, "/var/tmp/");
+    else
+      strcpy (f_stem, "/tmp/");
+    strcat (f_stem, F_NAME);
+
+    /* Make it unique */
+    if (!dpy_name[0] || ':' == dpy_name[0])
     {
-       fprintf (stderr, "\n%s can't decide on fifo-name. "
-	      "Make sure that $FVWM_USERDIR is set.\n",
-	      MYNAME );
+      gethostname(hostname, MAXHOSTNAME);
+      strcat(f_stem, hostname);  /* Put hostname before dpy if not there */
     }
+    strcat(f_stem, dpy_name);
   }
 
   /* create 2 fifos */
@@ -219,29 +237,22 @@ int main ( int argc, char *argv[])
   strcpy(Fr_name,f_stem);
   strcat(Fr_name, "R");
 
-  Fdrun = open(Fr_name, O_WRONLY | O_CREAT | O_EXCL | O_NOFOLLOW, 0600);
-  if (Fdrun < 0)
+  if ((Frun = fopen (Fr_name,"r" )) !=NULL)
   {
-    FILE *f;
-    if ((f = fopen (Fr_name,"r" )) != NULL)
+    if (fgets (cmd, 20, Frun) != NULL)
     {
-      *cmd = 0;
-      fgets(cmd, 20, f);
-      fclose(f);
       fprintf (stderr, "\nFvwmCommand lock file %sR is detected. "
 	       "This may indicate another FvwmCommand is running. "
 	       "It appears to be running under process ID:\n%s\n",
-	       f_stem, (*cmd) ? cmd : "(unknown)" );
+	       f_stem, cmd );
       fprintf (stderr, "You may either kill the process or run FvwmCommand "
 	       "with another FIFO set using option -S and -f. "
 	       "If the process doesn't exist, simply remove file:\n%sR\n\n",
 	       f_stem);
       exit(1);
     }
-    else
-    {
-      err_quit ("writing lock file");
-    }
+    fclose (Frun);
+    unlink (Fr_name);
   }
 
   if( Opt_Serv )
@@ -250,17 +261,19 @@ int main ( int argc, char *argv[])
     system (cmd);
   }
 
+  if ((Frun = fopen (Fr_name,"w" )) != NULL)
   {
-    char buf[64];
-
-    sprintf(buf, "%d\n", (int)getpid());
-    write(Fdrun, buf, strlen(buf));
-    close(Fdrun);
+    fprintf (Frun, "%d\n", (int) getpid());
+    fclose (Frun);
+  }
+  else
+  {
+      err_quit ("writing lock file");
   }
 
   Fdr = Fdw = -1;
   count = 0;
-  while ((Fdr=open (fm_name, O_RDONLY | O_NOFOLLOW)) < 0)
+  while ((Fdr=open (fm_name, O_RDONLY)) < 0)
   {
     if (count++>5)
     {
@@ -269,7 +282,7 @@ int main ( int argc, char *argv[])
     sleep(1);
   }
   count = 0;
-  while ((Fdw=open (fc_name, O_WRONLY | O_NOFOLLOW)) < 0)
+  while ((Fdw=open (fc_name, O_WRONLY)) < 0)
   {
     if (count++>2)
     {
@@ -319,32 +332,32 @@ int main ( int argc, char *argv[])
       FD_SET(Fdr, &fdset);
       if( Bg == 0 )
       {
-        FD_SET(STDIN_FILENO, &fdset);
+	FD_SET(STDIN_FILENO, &fdset);
       }
       ncnt = fvwmSelect(FD_SETSIZE, &fdset, 0, 0, NULL);
 
       /* message from fvwm */
       if (FD_ISSET(Fdr, &fdset))
       {
-        process_message();
+	process_message();
       }
 
       if( Bg == 0 )
       {
-        /* command input */
-        if( FD_ISSET(STDIN_FILENO, &fdset) )
-        {
-          if( fgets( cmd, MAX_MODULE_INPUT_TEXT_LEN - 1, stdin ) == 0 )
-          {
-            if( Bg == 0 )
-            {
-              /* other than SIGTTIN */
-              break;
-            }
-            continue;
-          }
-          sendit( cmd );
-        }
+	/* command input */
+	if( FD_ISSET(STDIN_FILENO, &fdset) )
+	{
+	  if( fgets( cmd, MAX_MODULE_INPUT_TEXT_LEN - 1, stdin ) == 0 )
+	  {
+	    if( Bg == 0 )
+	    {
+	      /* other than SIGTTIN */
+	      break;
+	    }
+	    continue;
+	  }
+	  sendit( cmd );
+	}
       }
     }
   }
@@ -355,7 +368,7 @@ int main ( int argc, char *argv[])
       strncpy( cmd, argv[i], MAX_MODULE_INPUT_TEXT_LEN - 1 );
       sendit( cmd );
       if (Opt_info >= 0)
-        receive();
+	receive();
     }
   }
   close_fifos();
@@ -604,7 +617,7 @@ void process_message( void )
       if( Opt_info >=2 )
       {
 	switch(type)
-        {
+	{
 
 	case M_CONFIGURE_WINDOW:
 	  list_configure( body);
@@ -615,9 +628,9 @@ void process_message( void )
 
 	default:
 	  if( Opt_info >= 3 )
-          {
+	  {
 	    switch( type )
-            {
+	    {
 	    case M_NEW_PAGE:
 	      list_new_page(body);
 	      break;
@@ -699,9 +712,9 @@ void list_configure(unsigned long *body)
     printf( "do_circulate_skip_shaded: %d\n",
 	    DO_SKIP_SHADED_CIRCULATE( cfgpacket ) );
     printf( "do_grab_focus_when_created: %d\n",
-	    DO_GRAB_FOCUS( cfgpacket ) );
+	    FP_DO_GRAB_FOCUS( FW_FOCUS_POLICY(cfgpacket) ) );
     printf( "do_grab_focus_when_transient_created: %d\n",
-	    DO_GRAB_FOCUS_TRANSIENT( cfgpacket ) );
+	    FP_DO_GRAB_FOCUS_TRANSIENT( FW_FOCUS_POLICY(cfgpacket) ) );
     printf( "do_ignore_restack: %d\n",
 	    DO_IGNORE_RESTACK( cfgpacket ) );
     printf( "do_lower_transient: %d\n",
@@ -709,11 +722,7 @@ void list_configure(unsigned long *body)
     printf( "do_not_show_on_map: %d\n",
 	    DO_NOT_SHOW_ON_MAP( cfgpacket ) );
     printf( "do_not_pass_click_focus_click: %d\n",
-	    DO_NOT_PASS_CLICK_FOCUS_CLICK( cfgpacket ) );
-    printf( "do_not_raise_click_focus_click: %d\n",
-	    DO_NOT_RAISE_CLICK_FOCUS_CLICK( cfgpacket ) );
-    printf( "do_raise_mouse_focus_click: %d\n",
-	    DO_RAISE_MOUSE_FOCUS_CLICK( cfgpacket ) );
+	    FP_DO_PASS_FOCUS_CLICK( FW_FOCUS_POLICY(cfgpacket) ) );
     printf( "do_raise_transient: %d\n",
 	    DO_RAISE_TRANSIENT( cfgpacket ) );
     printf( "do_resize_opaque: %d\n",
@@ -722,14 +731,8 @@ void list_configure(unsigned long *body)
 	    DO_SHRINK_WINDOWSHADE( cfgpacket ) );
     printf( "do_stack_transient_parent: %d\n",
 	    DO_STACK_TRANSIENT_PARENT( cfgpacket ) );
-    printf( "do_start_iconic: %d\n",
-	    DO_START_ICONIC( cfgpacket ) );
     printf( "do_window_list_skip: %d\n",
 	    DO_SKIP_WINDOW_LIST( cfgpacket ) );
-    printf( "focus_mode: %d\n",
-	    GET_FOCUS_MODE( cfgpacket ) );
-    printf( "has_bottom_title: %d\n",
-	    HAS_BOTTOM_TITLE( cfgpacket ) );
     printf( "has_depressable_border: %d\n",
 	    HAS_DEPRESSABLE_BORDER( cfgpacket ) );
     printf( "has_mwm_border: %d\n",
@@ -751,7 +754,7 @@ void list_configure(unsigned long *body)
     printf( "is_icon_suppressed: %d\n",
 	    IS_ICON_SUPPRESSED( cfgpacket ) );
     printf( "is_lenient: %d\n",
-	    IS_LENIENT( cfgpacket ) );
+	    FP_IS_LENIENT( FW_FOCUS_POLICY(cfgpacket) ) );
     printf( "does_wm_delete_window: %d\n",
 	    WM_DELETES_WINDOW( cfgpacket ) );
     printf( "does_wm_take_focus: %d\n",
@@ -761,11 +764,11 @@ void list_configure(unsigned long *body)
     printf( "do_reuse_destroyed: %d\n",
 	    DO_REUSE_DESTROYED( cfgpacket ) );
     printf( "has_border: %d\n",
-	    HAS_BORDER( cfgpacket ) );
+	    !HAS_NO_BORDER( cfgpacket ) );
     printf( "has_title: %d\n",
 	    HAS_TITLE( cfgpacket ) );
-    printf( "is_deiconify_pending: %d\n",
-	    IS_DEICONIFY_PENDING( cfgpacket ) );
+    printf( "is_iconify_pending: %d\n",
+	    IS_ICONIFY_PENDING( cfgpacket ) );
     printf( "is_fully_visible: %d\n",
 	    IS_FULLY_VISIBLE( cfgpacket ) );
     printf( "is_iconified: %d\n",
@@ -812,9 +815,11 @@ void list_configure(unsigned long *body)
 	    IS_WINDOW_FONT_LOADED( cfgpacket ) );
     printf( "is_window_shaded %d\n",
 	    IS_SHADED( cfgpacket ) );
+    printf( "title_dir: %d\n",
+	    GET_TITLE_DIR( cfgpacket ) );
   }
 
-  printf( "0x%08lx %-20s %ld\n",
+  printf( "0x%08lx %-20s %hd\n",
 	  cfgpacket->w, "title height", cfgpacket->title_height);
   printf( "0x%08lx %-20s %hd\n",
 	  cfgpacket->w, "border width", cfgpacket->border_width);
