@@ -21,7 +21,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <sys/wait.h>
 
 #if HAVE_GETOPT_H
@@ -29,13 +29,11 @@
 #endif
 
 #include <errno.h>
-#include <string.h>
 
 #include <libs/Module.h>
 #include <libs/fvwmlib.h>
 #include <fvwm/fvwm.h>
 #include <libs/vpacket.h>
-#include <libs/fvwm_sys_stat.h>
 
 #ifndef HAVE_STRERROR
 extern char *sys_errlist[];
@@ -53,6 +51,3 @@ extern char *sys_errlist[];
 #ifndef HAVE_MKFIFO
 #define mkfifo(path, mode) ((errno = ENOSYS) - ENOSYS - 1)
 #endif
-
-
-char * fifos_get_default_name();
