@@ -28,9 +28,8 @@
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <string.h>
 #include <sys/wait.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -97,7 +96,7 @@ pid_t spawn_xtee(void)
 /***********************************************************************
  *
  *  Procedure:
- *	main - start of module
+ *      main - start of module
  *
  ***********************************************************************/
 int main(int argc, char **argv)
@@ -153,7 +152,7 @@ int main(int argc, char **argv)
 	verbose++;
       else
       {
-        fprintf(stderr,"%s Version %s received two filename args!\n",
+	fprintf(stderr,"%s Version %s received two filename args!\n",
 		MyName,VERSION);
 	exit(1);
       }
@@ -198,7 +197,7 @@ int main(int argc, char **argv)
 /***********************************************************************
  *
  *  Procedure:
- *	Loop - wait for data to process
+ *      Loop - wait for data to process
  *
  ***********************************************************************/
 void Loop(const int *fd)
@@ -206,7 +205,7 @@ void Loop(const int *fd)
     while (1) {
       FvwmPacket* packet = ReadFvwmPacket(fd[1]);
       if ( packet == NULL )
-        exit(0);
+	exit(0);
       process_message( packet->type, packet->body );
     }
 }
@@ -215,7 +214,7 @@ void Loop(const int *fd)
 /***********************************************************************
  *
  *  Procedure:
- *	Process message - examines packet types, and takes appropriate action
+ *      Process message - examines packet types, and takes appropriate action
  *
  ***********************************************************************/
 void process_message(unsigned long type, const unsigned long *body)
@@ -340,7 +339,7 @@ void process_message(unsigned long type, const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	SIGPIPE handler - SIGPIPE means fvwm is dying
+ *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
  ***********************************************************************/
 void DeadPipe(int nonsense)
@@ -353,7 +352,7 @@ void DeadPipe(int nonsense)
 /***********************************************************************
  *
  *  Procedure:
- *	list_old_configure - displays packet contents to output
+ *      list_old_configure - displays packet contents to output
  *
  ***********************************************************************/
 void list_old_configure(const unsigned long *body)
@@ -386,7 +385,7 @@ void list_old_configure(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_page - displays packet contents to output
+ *      list_new_page - displays packet contents to output
  *
  ***********************************************************************/
 void list_new_page(const unsigned long *body)
@@ -400,7 +399,7 @@ void list_new_page(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_desk - displays packet contents to output
+ *      list_new_desk - displays packet contents to output
  *
  ***********************************************************************/
 void list_new_desk(const unsigned long *body)
@@ -412,7 +411,7 @@ void list_new_desk(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_focus - displays packet contents to output
+ *      list_focus - displays packet contents to output
  *
  ***********************************************************************/
 void list_focus(const unsigned long *body)
@@ -426,7 +425,7 @@ void list_focus(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_winid - displays packet contents to for three-field
+ *      list_winid - displays packet contents to for three-field
  *                   window ID-bearing packets
  *
  ***********************************************************************/
@@ -442,7 +441,7 @@ void list_winid(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_unknown - handles an unrecognized packet.
+ *      list_unknown - handles an unrecognized packet.
  *
  ***********************************************************************/
 void list_unknown(const unsigned long *body)
@@ -455,7 +454,7 @@ void list_unknown(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_iconify - displays packet contents to output
+ *      list_iconify - displays packet contents to output
  *
  ***********************************************************************/
 void list_icon(const unsigned long *body)
@@ -473,7 +472,7 @@ void list_icon(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_window_name - displays packet contents to output
+ *      list_window_name - displays packet contents to output
  *
  ***********************************************************************/
 
@@ -489,7 +488,7 @@ void list_window_name(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_restack - displays packet contents to output
+ *      list_restack - displays packet contents to output
  *
  ***********************************************************************/
 
@@ -516,7 +515,7 @@ void list_restack(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_error - displays packet contents to output
+ *      list_error - displays packet contents to output
  *
  ***********************************************************************/
 
@@ -529,7 +528,7 @@ void list_error(const unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_configure - displays packet contents to output
+ *      list_configure - displays packet contents to output
  *
  ***********************************************************************/
 void list_configure(const unsigned long *body)
@@ -545,7 +544,7 @@ void list_configure(const unsigned long *body)
   fprintf(output,"\t frame h %ld\n",p->frame_height);
   fprintf(output,"\t desk %ld\n",p->desk);
   fprintf(output,"\t layer %ld\n",p->layer);
-  fprintf(output,"\t title height %ld\n",p->title_height);
+  fprintf(output,"\t title height %hd\n",p->title_height);
   fprintf(output,"\t border width %hd\n",p->border_width);
   fprintf(output,"\t window base width %ld\n",p->hints_base_width);
   fprintf(output,"\t window base height %ld\n",p->hints_base_height);
@@ -571,27 +570,21 @@ void list_configure(const unsigned long *body)
 	    DO_SKIP_ICON_CIRCULATE(p));
     fprintf(output,"\t\tdo_circulate_skip_shaded: %d\n",
 	    DO_SKIP_SHADED_CIRCULATE(p));
-    fprintf(output,"\t\tdo_grab_focus_when_created: %d\n", DO_GRAB_FOCUS(p));
+    fprintf(output,"\t\tdo_grab_focus_when_created: %d\n",
+	    FP_DO_GRAB_FOCUS(FW_FOCUS_POLICY(p)));
     fprintf(output,"\t\tdo_grab_focus_when_transient_created: %d\n",
-	    DO_GRAB_FOCUS_TRANSIENT(p));
+	    FP_DO_GRAB_FOCUS_TRANSIENT(FW_FOCUS_POLICY(p)));
     fprintf(output,"\t\tdo_ignore_restack: %d\n", DO_IGNORE_RESTACK(p));
     fprintf(output,"\t\tdo_lower_transient: %d\n", DO_LOWER_TRANSIENT(p));
     fprintf(output,"\t\tdo_not_show_on_map: %d\n", DO_NOT_SHOW_ON_MAP(p));
     fprintf(output,"\t\tdo_not_pass_click_focus_click: %d\n",
-	    DO_NOT_PASS_CLICK_FOCUS_CLICK(p));
-    fprintf(output,"\t\tdo_not_raise_click_focus_click: %d\n",
-	    DO_NOT_RAISE_CLICK_FOCUS_CLICK(p));
-    fprintf(output,"\t\tdo_raise_mouse_focus_click: %d\n",
-	    DO_RAISE_MOUSE_FOCUS_CLICK(p));
+	    FP_DO_PASS_FOCUS_CLICK( FW_FOCUS_POLICY(p) ) );
     fprintf(output,"\t\tdo_raise_transient: %d\n", DO_RAISE_TRANSIENT(p));
     fprintf(output,"\t\tdo_resize_opaque: %d\n", DO_RESIZE_OPAQUE(p));
     fprintf(output,"\t\tdo_shrink_windowshade: %d\n", DO_SHRINK_WINDOWSHADE(p));
     fprintf(output,"\t\tdo_stack_transient_parent: %d\n",
 	    DO_STACK_TRANSIENT_PARENT(p));
-    fprintf(output,"\t\tdo_start_iconic: %d\n", DO_START_ICONIC(p));
     fprintf(output,"\t\tdo_window_list_skip: %d\n", DO_SKIP_WINDOW_LIST(p));
-    fprintf(output,"\t\tfocus_mode: %d\n", GET_FOCUS_MODE(p));
-    fprintf(output,"\t\thas_bottom_title: %d\n", HAS_BOTTOM_TITLE(p));
     fprintf(output,"\t\thas_depressable_border: %d\n",
 	    HAS_DEPRESSABLE_BORDER(p));
     fprintf(output,"\t\thas_mwm_border: %d\n", HAS_MWM_BORDER(p));
@@ -603,14 +596,15 @@ void list_configure(const unsigned long *body)
     fprintf(output,"\t\tis_fixed: %d\n", IS_FIXED(p));
     fprintf(output,"\t\tis_sticky_icon: %d\n", IS_ICON_STICKY(p));
     fprintf(output,"\t\tis_icon_suppressed: %d\n", IS_ICON_SUPPRESSED(p));
-    fprintf(output,"\t\tis_lenient: %d\n", IS_LENIENT(p));
+    fprintf(output,"\t\tis_lenient: %d\n",
+	    FP_IS_LENIENT( FW_FOCUS_POLICY(p) ));
     fprintf(output,"\t\tdoes_wm_delete_window: %d\n", WM_DELETES_WINDOW(p));
     fprintf(output,"\t\tdoes_wm_take_focus: %d\n", WM_TAKES_FOCUS(p));
     fprintf(output,"\t\tdo_iconify_after_map: %d\n", DO_ICONIFY_AFTER_MAP(p));
     fprintf(output,"\t\tdo_reuse_destroyed: %d\n", DO_REUSE_DESTROYED(p));
-    fprintf(output,"\t\thas_border: %d\n", HAS_BORDER(p));
+    fprintf(output,"\t\thas_border: %d\n", !HAS_NO_BORDER(p));
     fprintf(output,"\t\thas_title: %d\n", HAS_TITLE(p));
-    fprintf(output,"\t\tis_deiconify_pending: %d\n", IS_DEICONIFY_PENDING(p));
+    fprintf(output,"\t\tis_iconify_pending: %d\n", IS_ICONIFY_PENDING(p));
     fprintf(output,"\t\tis_fully_visible: %d\n", IS_FULLY_VISIBLE(p));
     fprintf(output,"\t\tis_iconified: %d\n", IS_ICONIFIED(p));
     fprintf(output,"\t\tis_iconfied_by_parent: %d\n",
@@ -637,6 +631,7 @@ void list_configure(const unsigned long *body)
 	    IS_WINDOW_BEING_MOVED_OPAQUE(p));
     fprintf(output,"\t\tis_window_font_loaded: %d\n", IS_WINDOW_FONT_LOADED(p));
     fprintf(output,"\t\tis_window_shaded %d\n", IS_SHADED(p));
+    fprintf(output,"\t\ttitle_dir: %d\n", GET_TITLE_DIR(p));
     fflush(output);
   }
 }
