@@ -38,7 +38,6 @@
 
 #include <libs/Module.h>
 #include <libs/fvwmlib.h>
-#include <libs/Picture.h>
 #include <fvwm/fvwm.h>
 #include <libs/vpacket.h>
 
@@ -106,9 +105,9 @@ destroy (int argc, char **argv)
   if (w != NULL)
     {
       if (gtk_widget_get_toplevel (current) == w)
-        {
-          current = NULL;
-        }
+	{
+	  current = NULL;
+	}
 
       g_hash_table_remove (widgets, argv[0]);
       gtk_object_unref (GTK_OBJECT (w));
@@ -342,6 +341,7 @@ process_message (unsigned long type,
   int button = 0;
   GtkWidget *widget;
   window_list_options *opts;
+
   char name[128];
 
   switch (type)
@@ -353,14 +353,14 @@ process_message (unsigned long type,
       widget = g_hash_table_lookup (widgets, name);
       if (!widget)
 	{
-          widget_not_found (name);
+	  widget_not_found (name);
 	}
       else if (GTK_IS_MENU (widget))
 	{
-          opts = (window_list_options *)
+	  opts = (window_list_options *)
 	    gtk_object_get_data (GTK_OBJECT (widget), "window_list");
-          if (opts)
-            {
+	  if (opts)
+	    {
 	      char *argv[1];
 
 	      argv[0] = name;
@@ -369,11 +369,11 @@ process_message (unsigned long type,
 	      widget = current;
 	      gtk_object_set_data (GTK_OBJECT (current), "window_list", opts);
 	      construct_window_list ();
-            }
+	    }
 
 	  gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
 			  button, timestamp);
-        }
+	}
       else if (GTK_IS_WINDOW (widget))
 	{
 	  gtk_widget_show (GTK_WIDGET (widget));
@@ -423,7 +423,17 @@ process_message (unsigned long type,
       break;
     case M_MINI_ICON:
       {
+/*
+	MiniIconPacket *mip = (MiniIconPacket *) body;
+	window_list_entry *wle = lookup_window_list_entry (mip->w);
+
+	if (wle->mini_icon)
+	  free(wle->mini_icon);
+	wle->mini_icon = safestrdup (mip->name);
+*/
+
 	window_list_entry *wle = lookup_window_list_entry (body[0]);
+
 	if (wle->mini_icon)
 	  free(wle->mini_icon);
 	wle->mini_icon = safestrdup ((char*) (&body[8]));
