@@ -20,6 +20,7 @@
  *************************************************************************/
 #include "libs/fvwmlib.h"
 #include <libs/vpacket.h>
+#include "libs/Flocale.h"
 
 struct icon_info;
 void MySendFvwmPipe(int *fd, char *message, unsigned long window);
@@ -38,8 +39,7 @@ extern int    My_XNextEvent(Display *dpy, XEvent *event);
 extern void   DeadPipe(int nonsense);
 extern void   CreateIconWindow(struct icon_info *item);
 extern void   DrawIconWindow(struct icon_info *item);
-extern void   GetBitmapFile(struct icon_info *item);
-extern void   GetXPMFile(struct icon_info *item);
+extern void   GetIconFromFile(struct icon_info *item);
 extern void GetIconWindow(struct icon_info *item);
 extern void GetIconBitmap(struct icon_info *item);
 extern void process_message(unsigned long type,unsigned long *body);
@@ -86,7 +86,7 @@ extern int window_cond(struct icon_info *item);
 extern int itemcmp(struct icon_info *item1, struct icon_info *item2);
 
 
-extern Display *dpy;			/* which display are we talking to */
+extern Display *dpy;                    /* which display are we talking to */
 extern Window Root;
 extern Window main_win;
 extern Window holder_win;
@@ -95,8 +95,8 @@ extern int screen;
 extern Pixel fore_pix, back_pix, icon_fore_pix, icon_back_pix;
 extern GC  NormalGC;
 extern GC  ReliefGC;
+extern FlocaleFont *Ffont;
 extern int ButtonWidth,ButtonHeight;
-extern XFontStruct *font;
 extern int num_rows;
 extern int num_columns;
 extern int Lines;
@@ -120,8 +120,9 @@ struct icon_info
   int y;
   int icon_w;
   int icon_h;
-  Pixmap iconPixmap;		/* pixmap for the icon */
+  Pixmap iconPixmap;            /* pixmap for the icon */
   Pixmap icon_maskPixmap;
+  Pixmap icon_alphaPixmap;
   Window IconWin;
   Window icon_pixmap_w;
   XWMHints *wmhints;
