@@ -23,9 +23,6 @@
 #include <libs/FScreen.h>
 #include <libs/Module.h>
 
-static char const rcsid[] =
-  "$Id: fvwm.c,v 1.38 2002/01/19 21:33:15 domivogt Exp $";
-
 static WinData *fvwm_focus_win = NULL;
 
 typedef struct {
@@ -57,19 +54,19 @@ typedef struct {
   } name;
 } m_name_data;
 
-#ifdef MINI_ICONS
+typedef struct {
+  Ulong arg, toggle, id;
+} m_property_data;
 
 typedef struct {
   Ulong app_id, frame_id, dbase_entry;
-  Ulong width, height, depth, picture, mask;
+  Ulong width, height, depth, picture, mask, alpha;
   union {
     Ulong name_long[1];
     Uchar name[4];
   } name;
 } m_mini_icon_data;
 
-#endif
-
 typedef union {
   m_toggle_paging_data toggle_paging_data;
   m_new_desk_data      new_desk_data;
@@ -78,9 +75,8 @@ typedef union {
   m_minimal_data       minimal_data;
   m_icon_data          icon_data;
   m_name_data          name_data;
-#ifdef MINI_ICONS
   m_mini_icon_data     mini_icon_data;
-#endif
+  m_property_data      property_data;
 } FvwmPacketBody;
 
 /* only used by count_nonsticky_in_hashtab */
@@ -225,35 +221,28 @@ static void set_win_configuration (WinData *win, FvwmPacketBody *body)
 
 static void handle_config_info (unsigned long *body)
 {
-  char *tline, *token;
+  char *tline, *token, *rest;
   int color;
+  extern void process_dynamic_config_line(char *line);
 
   tline = (char*)&(body[3]);
-  token = PeekToken(tline, &tline);
+  token = PeekToken(tline, &rest);
   if (StrEquals(token, "Colorset"))
   {
-    color = LoadColorset(tline);
+    color = LoadColorset(rest);
     change_colorset(color);
   }
   else if (StrEquals(token, XINERAMA_CONFIG_STRING))
   {
-    FScreenConfigureModule(tline);
+    FScreenConfigureModule(rest);
   }
   else if (StrEquals(token, "IgnoreModifiers"))
   {
-    sscanf(tline, "%d", &mods_unused);
+    sscanf(rest, "%d", &mods_unused);
   }
-  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+  else if (strncasecmp(tline, Module, ModuleLen) == 0)
   {
-    int j;
-    WinManager *man;
-
-    for (j = 0; j < globals.num_managers; j++)
-    {
-      man = &globals.managers[j];
-      if (man->pixmap[DEFAULT] == ParentRelative)
-	XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
-    }
+    process_dynamic_config_line(tline);
   }
 }
 
@@ -484,16 +473,20 @@ static void destroy_window (FvwmPacketBody *body)
   free_windata (win);
 }
 
-#ifdef MINI_ICONS
 static void mini_icon (FvwmPacketBody *body)
 {
   Ulong app_id = body->mini_icon_data.app_id;
   WinData *win;
 
+  if (!FMiniIconsSupported)
+  {
+    return;
+  }
   win = id_to_win (app_id);
   set_win_picture (win, body->mini_icon_data.picture,
-		   body->mini_icon_data.mask, body->mini_icon_data.depth,
-		   body->mini_icon_data.width, body->mini_icon_data.height);
+		   body->mini_icon_data.mask, body->mini_icon_data.alpha,
+		   body->mini_icon_data.depth, body->mini_icon_data.width,
+		   body->mini_icon_data.height);
 
 
   ConsoleDebug (FVWM, "mini_icon: 0x%lx 0x%lx %dx%dx%d\n",
@@ -501,7 +494,6 @@ static void mini_icon (FvwmPacketBody *body)
 		(unsigned long) win->pic.mask,
 		win->pic.width, win->pic.height, win->pic.depth);
 }
-#endif
 
 static void iconify (FvwmPacketBody *body, int dir)
 {
@@ -516,20 +508,24 @@ static void iconify (FvwmPacketBody *body, int dir)
   check_in_window (win);
 }
 
-/* only used by new_desk */
+/* only used by remanage_winlist */
 
-static void update_win_in_hashtab (void *arg)
+static void update_win_in_hashtab(void *arg)
 {
-  WinData *p = (WinData *)arg;
-  check_in_window (p);
+	WinData *p = (WinData *)arg;
+	check_in_window(p);
+}
+
+void remanage_winlist()
+{
+	walk_hashtab(update_win_in_hashtab);
+	draw_managers();
 }
 
 static void new_desk (FvwmPacketBody *body)
 {
-  globals.desknum = body->new_desk_data.desknum;
-  walk_hashtab (update_win_in_hashtab);
-
-  draw_managers ();
+	globals.desknum = body->new_desk_data.desknum;
+	remanage_winlist();
 }
 
 static void sendtomodule (FvwmPacketBody *body)
@@ -542,6 +538,38 @@ static void sendtomodule (FvwmPacketBody *body)
   execute_function ((char *)string);
 }
 
+static void property_change (FvwmPacketBody *body)
+{
+  WinManager *man = NULL;
+  int j;
+
+  if (body->property_data.id != 0)
+  {
+    if (!(man = find_windows_manager(body->property_data.id)))
+      return;
+  }
+
+  if (body->property_data.arg == MX_PROPERTY_CHANGE_BACKGROUND)
+  {
+    if (man != NULL && man->pixmap[DEFAULT] == ParentRelative)
+    {
+      XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
+    }
+    if (man != NULL)
+      return;
+    for (j = 0; j < globals.num_managers; j++)
+    {
+      man = &globals.managers[j];
+      if (man->pixmap[DEFAULT] == ParentRelative)
+	XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
+    }
+  }
+  else if  (body->property_data.arg == MX_PROPERTY_CHANGE_SWALLOW && man != NULL)
+  {
+    man->swallowed = body->property_data.toggle;
+  }
+}
+
 static void ProcessMessage (Ulong type, FvwmPacketBody *body)
 {
   int i;
@@ -589,12 +617,10 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     destroy_window (body);
     break;
 
-#ifdef MINI_ICONS
   case M_MINI_ICON:
     ConsoleDebug (FVWM, "DEBUG::M_MINI_ICON\n");
     mini_icon (body);
     break;
-#endif
 
   case M_WINDOW_NAME:
     ConsoleDebug (FVWM, "DEBUG::M_WINDOW_NAME\n");
@@ -658,7 +684,7 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     globals.y = body->new_page_data.y;
     globals.desknum = body->new_page_data.desknum;
     if (fvwm_focus_win && fvwm_focus_win->manager &&
-        fvwm_focus_win->manager->followFocus)
+	fvwm_focus_win->manager->followFocus)
     {
       /* need to set the focus on a page change */
       add_win_state (fvwm_focus_win, FOCUS_CONTEXT);
@@ -673,6 +699,11 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     sendtomodule (body);
     break;
 
+  case MX_PROPERTY_CHANGE:
+    ConsoleDebug (FVWM, "DEBUG::MX_PROPERTY_CHANGE\n");
+    property_change(body);
+    break;
+
   default:
     break;
   }
