@@ -18,15 +18,13 @@
 #include <stdlib.h>
 #include "FvwmIconMan.h"
 #include "readconfig.h"
+#include "xmanager.h"
 #include <libs/defaults.h>
 #include <libs/fvwmlib.h>
 #include <libs/FScreen.h>
 #include <libs/FShape.h>
 #include <libs/Module.h>
 
-static char const rcsid[] =
-  "$Id: readconfig.c,v 1.39 2002/01/07 12:30:23 domivogt Exp $";
-
 /************************************************************************
  *
  * Builtin functions:
@@ -68,7 +66,7 @@ FunctionType builtin_functions[] = {
   { "gotobutton",  builtin_gotobutton,  1, { ButtonArg } },
   { "gotomanager", builtin_gotomanager, 1, { ManagerArg } },
   { "jmp",         builtin_jmp,         1, { JmpArg } },
-  { "label",	   builtin_label,	1, { StringArg } },
+  { "label",       builtin_label,       1, { StringArg } },
   { "print",       builtin_print,       1, { StringArg } },
   { "printdebug",  builtin_printdebug,  0, {0} },
   { "quit",        builtin_quit,        0, {0} },
@@ -727,7 +725,7 @@ Binding *ParseMouseEntry (char *tline)
   find_context(modifiers,&mods,key_modifiers,tline);
   if((mods & AnyModifier)&&(mods&(~AnyModifier))) {
     ConsoleMessage ("Binding specified AnyModifier and other modifers too. "
-                    "Excess modifiers will be ignored.");
+		    "Excess modifiers will be ignored.");
   }
 
   new = (Binding *)safemalloc(sizeof(Binding));
@@ -934,7 +932,8 @@ static char *read_next_cmd (ReadOption flag)
 {
   static ReadOption status;
   static char *buffer;
-  static char *retstring, displaced, *cur_pos;
+  static char *retstring, *cur_pos;
+  char *end_pos;
 
   retstring = NULL;
   if (flag != READ_LINE && !(flag & status))
@@ -946,59 +945,60 @@ static char *read_next_cmd (ReadOption flag)
       cur_pos = buffer;
       skip_space (&cur_pos);
       if (!strncasecmp (Module, cur_pos, ModuleLen)) {
-        retstring = cur_pos;
-        cur_pos += ModuleLen;
-        displaced = *cur_pos;
-        if (displaced == '*')
-          status = READ_OPTION;
-        else if (displaced == '\0')
-          status = READ_LINE;
-        else if (iswhite (displaced))
-          status = READ_ARG;
-        else
-          status = READ_LINE;
-        break;
+	retstring = cur_pos;
+	cur_pos += ModuleLen;
+
+	if (*cur_pos == '*')
+	  cur_pos++;
+	skip_space(&cur_pos);
+
+	if (isalnum(*cur_pos))
+	  status = READ_OPTION;
+	else
+	  status = READ_LINE;
+	break;
       }
     }
     break;
 
   case READ_OPTION:
-    *cur_pos = displaced;
-    retstring = ++cur_pos;
+    retstring = cur_pos;
     while (*cur_pos != '*' && !iswhite (*cur_pos))
       cur_pos++;
-    displaced = *cur_pos;
-    *cur_pos = '\0';
-    if (displaced == '*')
+    end_pos = cur_pos;
+
+    if (*cur_pos == '*')
+      cur_pos++;
+    skip_space(&cur_pos);
+
+    if (!*cur_pos)
+      status = READ_LINE;
+    else if (isdigit(*retstring) || !strncmp(retstring, "transient", 9))
       status = READ_OPTION;
-    else if (displaced == '\0')
-      status = READ_LINE;
-    else if (iswhite (displaced))
+    else if (*cur_pos)
       status = READ_ARG;
-    else
-      status = READ_LINE;
+
+    *end_pos = '\0';
     break;
 
   case READ_ARG:
-    *cur_pos = displaced;
-    skip_space (&cur_pos);
     retstring = cur_pos;
     while (!iswhite (*cur_pos))
       cur_pos++;
-    displaced = *cur_pos;
-    *cur_pos = '\0';
-    if (displaced == '\0')
-      status = READ_LINE;
-    else if (iswhite (displaced))
+    end_pos = cur_pos;
+
+    skip_space(&cur_pos);
+
+    if (*cur_pos)
       status = READ_ARG;
     else
       status = READ_LINE;
+
+    *end_pos = '\0';
     break;
 
   case READ_REST_OF_LINE:
     status = READ_LINE;
-    *cur_pos = displaced;
-    skip_space (&cur_pos);
     retstring = cur_pos;
     break;
   }
@@ -1127,6 +1127,38 @@ static void handle_button_config (int manager, int context, char *option)
 	       copy_string (&globals.managers[id].backColorName[context], p));
 }
 
+static void add_weighted_sort(WinManager *man, WeightedSort *weighted_sort) {
+  WeightedSort *p;
+  int i;
+
+  if (man->weighted_sorts_len == man->weighted_sorts_size) {
+    i = man->weighted_sorts_size;
+    man->weighted_sorts_size += 16;
+    man->weighted_sorts =
+      (WeightedSort *)saferealloc ((char *)man->weighted_sorts,
+			 man->weighted_sorts_size * sizeof (WeightedSort));
+  }
+  p = &man->weighted_sorts[man->weighted_sorts_len];
+  p->resname = NULL;
+  p->classname = NULL;
+  p->titlename = NULL;
+  p->iconname = NULL;
+  if (weighted_sort->resname) {
+    copy_string(&p->resname, weighted_sort->resname);
+  }
+  if (weighted_sort->classname) {
+    copy_string(&p->classname, weighted_sort->classname);
+  }
+  if (weighted_sort->titlename) {
+    copy_string(&p->titlename, weighted_sort->titlename);
+  }
+  if (weighted_sort->iconname) {
+    copy_string(&p->iconname, weighted_sort->iconname);
+  }
+  p->weight = weighted_sort->weight;
+  ++man->weighted_sorts_len;
+}
+
 void read_in_resources (char *file)
 {
   char *p, *q;
@@ -1134,6 +1166,7 @@ void read_in_resources (char *file)
   char *option1;
   Binding *binding = NULL;
   Resolution r;
+  Reverse rv;
 
   if (!init_config_file (file))
     return;
@@ -1293,7 +1326,7 @@ void read_in_resources (char *file)
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  continue;
 	}
-        for ( i = 0; i < NUM_CONTEXTS; i++ ) {
+	for ( i = 0; i < NUM_CONTEXTS; i++ ) {
 	  SET_MANAGER(manager, colorsets[i], n);
 	  AllocColorset(n);
 	}
@@ -1306,7 +1339,7 @@ void read_in_resources (char *file)
 	}
 	ConsoleDebug (CONFIG, "default background: %s\n", p);
 
-        for ( i = 0; i < NUM_CONTEXTS; i++ )
+	for ( i = 0; i < NUM_CONTEXTS; i++ )
 	  SET_MANAGER (manager, backColorName[i],
 	    conditional_copy_string (&globals.managers[id].backColorName[i],
 				     p));
@@ -1350,33 +1383,36 @@ void read_in_resources (char *file)
 	  Free(token);
       }
       else if (!strcasecmp (option1, "drawicons")) {
-#ifdef MINI_ICONS
-	p = read_next_cmd (READ_ARG);
-	if (!p) {
-	  ConsoleMessage ("Bad line: %s\n", current_line);
-	  ConsoleMessage ("Need argument to drawicons\n");
-	  continue;
+	if (!FMiniIconsSupported)
+	{
+	  ConsoleMessage ("DrawIcons support not compiled in\n");
 	}
-	if (!strcasecmp (p, "true")) {
-	  i = 1;
+	else
+	{
+	  p = read_next_cmd (READ_ARG);
+	  if (!p) {
+	    ConsoleMessage ("Bad line: %s\n", current_line);
+	    ConsoleMessage ("Need argument to drawicons\n");
+	    continue;
+	  }
+	  if (!strcasecmp (p, "true")) {
+	    i = 1;
+	  }
+	  /* [NFM 3 Dec 97] added support for drawicons "always" */
+	  else if (!strcasecmp (p, "always")) {
+	    i = 2;
+	  }
+	  else if (!strcasecmp (p, "false")) {
+	    i = 0;
+	  }
+	  else {
+	    ConsoleMessage ("Bad line: %s\n", current_line);
+	    ConsoleMessage ("What is this: %s?\n", p);
+	    continue;
+	  }
+	  ConsoleDebug (CONFIG, "Setting drawicons to: %d\n", i);
+	  SET_MANAGER (manager, draw_icons, i);
 	}
-    /* [NFM 3 Dec 97] added support for FvwmIconMan*drawicons "always" */
-	else if (!strcasecmp (p, "always")) {
-	  i = 2;
-	}
-	else if (!strcasecmp (p, "false")) {
-	  i = 0;
-	}
-	else {
-	  ConsoleMessage ("Bad line: %s\n", current_line);
-	  ConsoleMessage ("What is this: %s?\n", p);
-	  continue;
-	}
-	ConsoleDebug (CONFIG, "Setting drawicons to: %d\n", i);
-	SET_MANAGER (manager, draw_icons, i);
-#else
-	ConsoleMessage ("DrawIcons support not compiled in\n");
-#endif
       }
       else if (!strcasecmp (option1, "followfocus")) {
 	p = read_next_cmd (READ_ARG);
@@ -1402,35 +1438,38 @@ void read_in_resources (char *file)
       else if (!strcasecmp (option1, "showonlyiconic")) {
        p = read_next_cmd (READ_ARG);
        if (!p) {
-         ConsoleMessage ("Bad line: %s\n", current_line);
-         ConsoleMessage ("Need argument to showonlyiconic\n");
-         continue;
+	 ConsoleMessage ("Bad line: %s\n", current_line);
+	 ConsoleMessage ("Need argument to showonlyiconic\n");
+	 continue;
        }
        if (!strcasecmp (p, "true")) {
-         i = 1;
+	 i = 1;
        }
        else if (!strcasecmp (p, "false")) {
-         i = 0;
+	 i = 0;
        }
        else {
-         ConsoleMessage ("Bad line: %s\n", current_line);
-         ConsoleMessage ("What is this: %s?\n", p);
-         continue;
+	 ConsoleMessage ("Bad line: %s\n", current_line);
+	 ConsoleMessage ("What is this: %s?\n", p);
+	 continue;
        }
        ConsoleDebug (CONFIG, "Setting showonlyiconic to: %d\n", i);
        SET_MANAGER (manager, showonlyiconic, i);
       }
       else if (!strcasecmp (option1, "font")) {
+	char *f;
 	p = read_next_cmd (READ_REST_OF_LINE);
 	trim(p);
 	if (!p) {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  continue;
 	}
-	ConsoleDebug (CONFIG, "font: %s\n", p);
+	CopyStringWithQuotes(&f, p);
+	ConsoleDebug (CONFIG, "font: %s\n", f);
 
 	SET_MANAGER (manager, fontname,
-		     copy_string (&globals.managers[id].fontname, p));
+		     copy_string (&globals.managers[id].fontname, f));
+	free(f);
       }
       else if (!strcasecmp (option1, "foreground")) {
 	p = read_next_cmd (READ_ARG);
@@ -1440,9 +1479,9 @@ void read_in_resources (char *file)
 	}
 	ConsoleDebug (CONFIG, "default foreground: %s\n", p);
 
-        for ( i = 0; i < NUM_CONTEXTS; i++ )
+	for ( i = 0; i < NUM_CONTEXTS; i++ )
 	SET_MANAGER (manager, foreColorName[i],
-           conditional_copy_string (&globals.managers[id].foreColorName[i],
+	   conditional_copy_string (&globals.managers[id].foreColorName[i],
 				    p));
       }
       else if (!strcasecmp (option1, "format")) {
@@ -1536,18 +1575,18 @@ void read_in_resources (char *file)
 	}
 	ConsoleDebug (CONFIG, "reverse: %s\n", p);
 	if (!strcasecmp (p, "none"))
-	  r = REVERSE_NONE;
+	  rv = REVERSE_NONE;
 	else if (!strcasecmp (p, "icon"))
-	  r = REVERSE_ICON;
+	  rv = REVERSE_ICON;
 	else if (!strcasecmp (p, "normal"))
-	  r = REVERSE_NORMAL;
+	  rv = REVERSE_NORMAL;
 	else {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  ConsoleMessage ("What kind of reverse is this?\n");
 	  continue;
 	}
 
-	SET_MANAGER (manager, rev, r);
+	SET_MANAGER (manager, rev, rv);
       }
       else if (!strcasecmp (option1, "shape")) {
 	p = read_next_cmd (READ_ARG);
@@ -1626,9 +1665,12 @@ void read_in_resources (char *file)
 	else if (!strcasecmp (p, "none")) {
 	  i = SortNone;
 	}
+	else if (!strcasecmp (p, "weighted")) {
+	  i = SortWeighted;
+	}
 	else if (!strcasecmp (p, "false") || !strcasecmp (p, "true")) {
 	  /* Old options */
-	  ConsoleMessage ("FvwmIconMan*sort option no longer takes "
+	  ConsoleMessage ("FvwmIconMan: sort option no longer takes "
 			  "true or false value\n"
 			  "Please read the latest manpage\n");
 	  continue;
@@ -1641,6 +1683,60 @@ void read_in_resources (char *file)
 	ConsoleDebug (CONFIG, "Setting sort to: %d\n", i);
 	SET_MANAGER (manager, sort, i);
       }
+      else if (!strcasecmp (option1, "sortweight")) {
+	WeightedSort weighted_sort;
+	p = read_next_cmd (READ_ARG);
+	if (!p) {
+	  ConsoleMessage ("Bad line: %s\n", current_line);
+	  continue;
+	}
+	if (extract_int (p, &n) == 0) {
+	  ConsoleMessage ("This is not a number: %s\n", p);
+	  ConsoleMessage ("Bad line: %s\n", current_line);
+	  continue;
+	}
+	weighted_sort.resname = NULL;
+	weighted_sort.classname = NULL;
+	weighted_sort.titlename = NULL;
+	weighted_sort.iconname = NULL;
+	weighted_sort.weight = n;
+	p = read_next_cmd (READ_ARG);
+	while (p) {
+	  if (!strncasecmp(p, "resource=", 9)) {
+	    copy_string(&weighted_sort.resname, p + 9);
+	  } else if (!strncasecmp(p, "class=", 6)) {
+	    copy_string(&weighted_sort.classname, p + 6);
+	  } else if (!strncasecmp(p, "title=", 6)) {
+	    copy_string(&weighted_sort.titlename, p + 6);
+	  } else if (!strncasecmp(p, "icon=", 5)) {
+	    copy_string(&weighted_sort.iconname, p + 5);
+	  } else {
+	    ConsoleMessage ("Unknown sortweight field: %s\n", p);
+	    ConsoleMessage ("Bad line: %s\n", current_line);
+	  }
+	  p = read_next_cmd (READ_ARG);
+	}
+	if (manager == -1) {
+	  for (i = 0; i < globals.num_managers; i++) {
+	    add_weighted_sort (&globals.managers[i], &weighted_sort);
+	  }
+	}
+	else {
+	  add_weighted_sort (&globals.managers[manager], &weighted_sort);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.resname);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.classname);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.titlename);
+	}
+	if (weighted_sort.resname) {
+	  Free(weighted_sort.iconname);
+	}
+      }
       else if (!strcasecmp (option1, "NoIconAction")) {
 	char *token;
 	p = read_next_cmd (READ_REST_OF_LINE);
@@ -1815,3 +1911,71 @@ void read_in_resources (char *file)
   close_config_file();
 }
 
+void process_dynamic_config_line(char *line)
+{
+	int manager = 0;
+	char *token;
+	extern void remanage_winlist();
+
+	/* don't support dynamic config for transient */
+	if (globals.transient)
+	{
+		return;
+	}
+
+	line += ModuleLen;
+	line = GetNextToken(line, &token);
+	if (token && isdigit(*token))
+	{
+		if (extract_int(token, &manager) == 0
+			|| manager <= 0 || manager > globals.num_managers)
+		{
+			manager = 0;
+		}
+		free(token);
+		line = GetNextToken(line, &token);
+	}
+
+	if (!token)
+	{
+		return;
+	}
+	manager--;
+
+	/* currently support only "resolution" */
+	if (strcasecmp(token, "resolution") == 0)
+	{
+		int value;
+		line = GetNextToken(line, &token);
+		if (!token)
+		{
+			return;
+		}
+		if (!strcasecmp(token, "global"))
+			value = SHOW_GLOBAL;
+		else if (!strcasecmp(token, "desk"))
+			value = SHOW_DESKTOP;
+		else if (!strcasecmp(token, "page"))
+			value = SHOW_PAGE;
+		else if (!strcasecmp(token, "screen"))
+			value = SHOW_SCREEN;
+		else if (!strcasecmp(token, "!desk"))
+			value = NO_SHOW_DESKTOP;
+		else if (!strcasecmp(token, "!page"))
+			value = NO_SHOW_PAGE;
+		else if (!strcasecmp(token, "!screen"))
+			value = NO_SHOW_SCREEN;
+		else
+		{
+			fprintf(
+				stderr, "%s: unknown resolution %s.\n",
+				MyName, token);
+			free(token);
+			return;
+		}
+		free(token);
+
+		SET_MANAGER(manager, res, value);
+		remanage_winlist();
+	}
+}
