@@ -28,7 +28,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/wait.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <ctype.h>
 
 #ifdef HAVE_SYS_BSDTYPES_H
@@ -41,15 +41,14 @@
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
 #include <X11/keysym.h>
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#endif
 
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/FRenderInit.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
 #ifdef DEBUG
 #  define FVWM_DEBUG_MSGS   /* Do we need this? */
 #endif
@@ -64,7 +63,7 @@ int fd[2];
 
 PagerStringList *FindDeskStrings(int desk);
 PagerStringList *NewPagerStringItem(PagerStringList *last, int desk);
-extern XFontStruct *windowFont;
+extern FlocaleFont *FwindowFont;
 extern Pixmap default_pixmap;
 
 /*************************************************************************
@@ -76,7 +75,7 @@ ScreenInfo Scr;
 PagerWindow *Start = NULL;
 PagerWindow *FocusWin = NULL;
 
-Display *dpy;			/* which display are we talking to */
+Display *dpy;                   /* which display are we talking to */
 int x_fd;
 fd_set_size_t fd_width;
 
@@ -86,7 +85,7 @@ char *font_string = NULL;
 char *smallFont = NULL;
 char *HilightC = NULL;
 
-Picture *HilightPixmap = NULL;
+FvwmPicture *HilightPixmap = NULL;
 int HilightDesks = 1;
 
 char *WindowBack = NULL;
@@ -101,7 +100,7 @@ Bool WindowBorders3d = False;
 
 Bool UseSkipList = False;
 
-Picture *PixmapBack = NULL;
+FvwmPicture *PixmapBack = NULL;
 
 char *ImagePath = NULL;
 
@@ -151,6 +150,7 @@ PagerStringList string_list = { NULL, 0, -1, -1, -1, NULL, NULL, NULL };
 Bool is_transient = False;
 Bool do_ignore_next_button_release = False;
 Bool error_occured = False;
+Bool Swallowed = False;
 
 static void SetDeskLabel(int desk, const char *label);
 static RETSIGTYPE TerminateHandler(int);
@@ -159,7 +159,7 @@ void ExitPager(void);
 /***********************************************************************
  *
  *  Procedure:
- *	main - start of module
+ *      main - start of module
  *
  ***********************************************************************/
 int main(int argc, char **argv)
@@ -172,9 +172,8 @@ int main(int argc, char **argv)
   int JunkX, JunkY;
   unsigned JunkMask;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmPager");
+
   /* Save our program  name - for error messages */
   MyName = GetFileNameFromPath(argv[0]);
 
@@ -212,10 +211,10 @@ int main(int argc, char **argv)
   /* We don't have sigaction(), so fall back to less robust methods.  */
 #ifdef USE_BSD_SIGNALS
   fvwmSetSignalMask( sigmask(SIGPIPE) |
-                     sigmask(SIGTERM) |
-                     sigmask(SIGQUIT) |
-                     sigmask(SIGINT) |
-                     sigmask(SIGHUP) );
+		     sigmask(SIGTERM) |
+		     sigmask(SIGQUIT) |
+		     sigmask(SIGINT) |
+		     sigmask(SIGHUP) );
 #endif
   signal(SIGPIPE, TerminateHandler);
   signal(SIGTERM, TerminateHandler);
@@ -312,10 +311,11 @@ int main(int argc, char **argv)
       exit (1);
     }
   x_fd = XConnectionNumber(dpy);
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
+  FRenderInit(dpy);
 
   Scr.screen = DefaultScreen(dpy);
   Scr.Root = RootWindow(dpy, Scr.screen);
@@ -328,15 +328,15 @@ int main(int argc, char **argv)
 
   SetMessageMask(fd,
 		 M_VISIBLE_NAME |
-                 M_ADD_WINDOW|
-                 M_CONFIGURE_WINDOW|
-                 M_DESTROY_WINDOW|
-                 M_FOCUS_CHANGE|
-                 M_NEW_PAGE|
-                 M_NEW_DESK|
-                 M_RAISE_WINDOW|
-                 M_LOWER_WINDOW|
-                 M_ICONIFY|
+		 M_ADD_WINDOW|
+		 M_CONFIGURE_WINDOW|
+		 M_DESTROY_WINDOW|
+		 M_FOCUS_CHANGE|
+		 M_NEW_PAGE|
+		 M_NEW_DESK|
+		 M_RAISE_WINDOW|
+		 M_LOWER_WINDOW|
+		 M_ICONIFY|
 		 M_ICON_LOCATION|
 		 M_DEICONIFY|
 		 M_RES_NAME|
@@ -347,7 +347,8 @@ int main(int argc, char **argv)
 		 M_END_WINDOWLIST|
 		 M_RESTACK);
   SetMessageMask(fd,
-		 MX_VISIBLE_ICON_NAME);
+		 MX_VISIBLE_ICON_NAME|
+		 MX_PROPERTY_CHANGE);
 #ifdef DEBUG
   fprintf(stderr,"[main]: calling ParseOptions\n");
 #endif
@@ -383,15 +384,9 @@ int main(int argc, char **argv)
   if (WindowLabelFormat == NULL)
     WindowLabelFormat = safestrdup("%i");
 
-  if (font_string == NULL)
-    font_string = safestrdup("fixed");
-
   if ((HilightC == NULL) && (HilightPixmap == NULL))
     HilightDesks = 0;
 
-  if (BalloonFont == NULL)
-    BalloonFont = safestrdup("fixed");
-
   if (BalloonBorderColor == NULL)
     BalloonBorderColor = safestrdup("black");
 
@@ -462,7 +457,7 @@ int main(int argc, char **argv)
   if (debug_term_signal)
   {
     fprintf(stderr,"[main]: Terminated due to signal %d\n",
-                   debug_term_signal);
+		   debug_term_signal);
   }
 #endif
   return 0;
@@ -471,7 +466,7 @@ int main(int argc, char **argv)
 /***********************************************************************
  *
  *  Procedure:
- *	Loop - wait for data to process
+ *      Loop - wait for data to process
  *
  ***********************************************************************/
 void Loop(int *fd)
@@ -504,7 +499,7 @@ void Loop(int *fd)
 /***********************************************************************
  *
  *  Procedure:
- *	Process message - examines packet types, and takes appropriate action
+ *      Process message - examines packet types, and takes appropriate action
  *
  ***********************************************************************/
 void process_message( FvwmPacket* packet )
@@ -566,6 +561,8 @@ void process_message( FvwmPacket* packet )
     case M_CONFIG_INFO:
       list_config_info(body);
       break;
+    case MX_PROPERTY_CHANGE:
+      list_property_change(body);
     default:
       /* ignore unknown packet */
       break;
@@ -576,7 +573,7 @@ void process_message( FvwmPacket* packet )
 /***********************************************************************
  *
  *  Procedure:
- *	SIGPIPE handler - SIGPIPE means fvwm is dying
+ *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
  ***********************************************************************/
 static RETSIGTYPE
@@ -593,7 +590,7 @@ void DeadPipe(int nonsense)
 /***********************************************************************
  *
  *  Procedure:
- *	list_add - displays packet contents to stderr
+ *      list_add - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_add(unsigned long *body)
@@ -639,8 +636,8 @@ void list_add(unsigned long *body)
   }
   else
   {
-    (*prev)->text = body[22];
-    (*prev)->back = body[23];
+    (*prev)->text = cfgpacket->TextPixel;
+    (*prev)->back = cfgpacket->BackPixel;
   }
   AddNewWindow(*prev);
 }
@@ -648,7 +645,7 @@ void list_add(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_configure - displays packet contents to stderr
+ *      list_configure - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_configure(unsigned long *body)
@@ -688,8 +685,8 @@ void list_configure(unsigned long *body)
   }
   else
   {
-    t->text = body[22];
-    t->back = body[23];
+    t->text = cfgpacket->TextPixel;
+    t->back = cfgpacket->BackPixel;
   }
   if (IS_ICONIFIED(t))
   {
@@ -723,7 +720,7 @@ void list_configure(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_destroy - displays packet contents to stderr
+ *      list_destroy - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_destroy(unsigned long *body)
@@ -764,7 +761,7 @@ void list_destroy(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_focus - displays packet contents to stderr
+ *      list_focus - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_focus(unsigned long *body)
@@ -772,15 +769,24 @@ void list_focus(unsigned long *body)
   PagerWindow *t;
   Window target_w;
   extern Pixel focus_pix, focus_fore_pix;
-  target_w = body[0];
+  Bool do_force_update = False;
 
+  target_w = body[0];
   if (win_hi_pix_set)
   {
+    if (focus_pix != win_hi_back_pix || focus_fore_pix != win_hi_fore_pix)
+    {
+      do_force_update = True;
+    }
     focus_pix = win_hi_back_pix;
     focus_fore_pix = win_hi_fore_pix;
   }
   else
   {
+    if (focus_pix != body[4] || focus_fore_pix != body[3])
+    {
+      do_force_update = True;
+    }
     focus_pix = body[4];
     focus_fore_pix = body[3];
   }
@@ -789,12 +795,12 @@ void list_focus(unsigned long *body)
   {
     t = t->next;
   }
-  if(t != FocusWin)
+  if (t != FocusWin || do_force_update)
   {
-    if(FocusWin != NULL)
+    if (FocusWin != NULL)
       Hilight(FocusWin, False);
     FocusWin = t;
-    if(FocusWin != NULL)
+    if (FocusWin != NULL)
       Hilight(FocusWin, True);
   }
 }
@@ -802,7 +808,7 @@ void list_focus(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_page - displays packet contents to stderr
+ *      list_new_page - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_new_page(unsigned long *body)
@@ -833,7 +839,7 @@ void list_new_page(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_new_desk - displays packet contents to stderr
+ *      list_new_desk - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_new_desk(unsigned long *body)
@@ -876,10 +882,9 @@ void list_new_desk(unsigned long *body)
     XStoreName(dpy, Scr.Pager_w, Desks[0].label);
     XSetIconName(dpy, Scr.Pager_w, Desks[0].label);
 
-
     if (Desks[0].bgPixmap != NULL)
     {
-      DestroyPicture(dpy, Desks[0].bgPixmap);
+      PDestroyFvwmPicture(dpy, Desks[0].bgPixmap);
       Desks[0].bgPixmap = NULL;
     }
 
@@ -961,10 +966,28 @@ void list_new_desk(unsigned long *body)
     {
       change_colorset(change_highcs);
     }
-
     XClearWindow(dpy, Desks[0].w);
     XClearWindow(dpy, Desks[0].title_w);
   } /* if (fAlwaysCurrentDesk && oldDesk != Scr.CurrentDesk) */
+  else if (!fAlwaysCurrentDesk)
+  {
+    int i;
+    char *name;
+    char line[100];
+
+    i = Scr.CurrentDesk - desk1;
+    if (i >= 0 && i < ndesks && Desks[i].label != NULL)
+    {
+      name = Desks[i].label;
+    }
+    else
+    {
+      sprintf(line, "Desk %d", Scr.CurrentDesk);
+      name = &(line[0]);
+    }
+    XStoreName(dpy, Scr.Pager_w, name);
+    XSetIconName(dpy, Scr.Pager_w, name);
+  }
 
   MovePage(True);
   DrawGrid(oldDesk - desk1,1);
@@ -979,7 +1002,7 @@ void list_new_desk(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_raise - displays packet contents to stderr
+ *      list_raise - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_raise(unsigned long *body)
@@ -1005,7 +1028,7 @@ void list_raise(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_lower - displays packet contents to stderr
+ *      list_lower - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_lower(unsigned long *body)
@@ -1033,7 +1056,7 @@ void list_lower(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_iconify - displays packet contents to stderr
+ *      list_iconify - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_iconify(unsigned long *body)
@@ -1085,7 +1108,7 @@ void list_iconify(unsigned long *body)
 /***********************************************************************
  *
  *  Procedure:
- *	list_deiconify - displays packet contents to stderr
+ *      list_deiconify - displays packet contents to stderr
  *
  ***********************************************************************/
 
@@ -1145,23 +1168,23 @@ void list_window_name(unsigned long *body,unsigned long type)
     {
       switch (type) {
       case M_RES_CLASS:
-        if(t->res_class != NULL)
-          free(t->res_class);
-        CopyString(&t->res_class,(char *)(&body[3]));
-        break;
+	if(t->res_class != NULL)
+	  free(t->res_class);
+	CopyString(&t->res_class,(char *)(&body[3]));
+	break;
       case M_RES_NAME:
-        if(t->res_name != NULL)
-          free(t->res_name);
-        CopyString(&t->res_name,(char *)(&body[3]));
-        break;
+	if(t->res_name != NULL)
+	  free(t->res_name);
+	CopyString(&t->res_name,(char *)(&body[3]));
+	break;
       case M_VISIBLE_NAME:
-        if(t->window_name != NULL)
-          free(t->window_name);
-        CopyString(&t->window_name,(char *)(&body[3]));
-        break;
+	if(t->window_name != NULL)
+	  free(t->window_name);
+	CopyString(&t->window_name,(char *)(&body[3]));
+	break;
       }
       /* repaint by clearing window */
-      if ((windowFont != NULL) && (t->icon_name != NULL)
+      if ((FwindowFont != NULL) && (t->icon_name != NULL)
 	   && !(MiniIcons && t->mini_icon.picture)) {
 	if (t->PagerView)
 	  XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
@@ -1188,7 +1211,7 @@ void list_window_name(unsigned long *body,unsigned long type)
 /***********************************************************************
  *
  *  Procedure:
- *	list_icon_name - displays packet contents to stderr
+ *      list_icon_name - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_icon_name(unsigned long *body)
@@ -1208,7 +1231,7 @@ void list_icon_name(unsigned long *body)
 	free(t->icon_name);
       CopyString(&t->icon_name,(char *)(&body[3]));
       /* repaint by clearing window */
-      if ((windowFont != NULL) && (t->icon_name != NULL)
+      if ((FwindowFont != NULL) && (t->icon_name != NULL)
 	   && !(MiniIcons && t->mini_icon.picture)) {
 	if (t->PagerView)
 	  XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
@@ -1221,19 +1244,22 @@ void list_icon_name(unsigned long *body)
 
 void list_mini_icon(unsigned long *body)
 {
-  PagerWindow	*t;
+  PagerWindow   *t;
   Window target_w;
+  MiniIconPacket *mip = (MiniIconPacket *) body;
+
   target_w = body[0];
   t = Start;
-  while (t && (t->w != target_w))
+  while (t && (t->w != mip->w))
     t = t->next;
   if (t)
   {
-    t->mini_icon.width   = body[3];
-    t->mini_icon.height  = body[4];
-    t->mini_icon.depth   = body[5];
-    t->mini_icon.picture = body[6];
-    t->mini_icon.mask    = body[7];
+    t->mini_icon.width   = mip->width;
+    t->mini_icon.height  = mip->height;
+    t->mini_icon.depth   = mip->depth;
+    t->mini_icon.picture = mip->picture;
+    t->mini_icon.mask    = mip->mask;
+    t->mini_icon.alpha   = mip->alpha;
     /* repaint by clearing window */
     if (MiniIcons && t->mini_icon.picture) {
       if (t->PagerView)
@@ -1298,7 +1324,7 @@ void list_restack(unsigned long *body, unsigned long length)
 /***********************************************************************
  *
  *  Procedure:
- *	list_end - displays packet contents to stderr
+ *      list_end - displays packet contents to stderr
  *
  ***********************************************************************/
 void list_end(void)
@@ -1349,11 +1375,6 @@ void list_config_info(unsigned long *body)
   {
     FScreenConfigureModule(tline);
   }
-  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
-  {
-    if (default_pixmap == ParentRelative)
-      ReConfigure();
-  }
   else if (StrEquals(token, "DesktopName"))
   {
     int val;
@@ -1378,6 +1399,26 @@ void list_config_info(unsigned long *body)
   }
 }
 
+void list_property_change(unsigned long *body)
+{
+  if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND)
+  {
+
+    if (default_pixmap == ParentRelative &&
+	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
+    {
+      ReConfigure();
+    }
+    if (BalloonView != None)
+    {
+      XClearArea(dpy, Scr.balloon_w, 0, 0, 0, 0, True);
+    }
+  }
+  else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == Scr.Pager_w)
+  {
+    Swallowed = body[1];
+  }
+}
 
 /***************************************************************************
  *
@@ -1412,7 +1453,7 @@ int My_XNextEvent(Display *dpy, XEvent *event)
 	miss_counter++;
 #ifdef WORRY_ABOUT_MISSED_XEVENTS
 	if(miss_counter > 100)
-          DeadPipe(0);
+	  DeadPipe(0);
 #endif
     }
 
@@ -1543,14 +1584,19 @@ void ParseOptions(void)
   int desk;
   int dx = 3;
   int dy = 3;
-
+  FvwmPictureAttributes fpa;
   Scr.FvwmRoot = NULL;
   Scr.Hilite = NULL;
   Scr.VScale = 32;
 
   Scr.MyDisplayWidth = DisplayWidth(dpy, Scr.screen);
   Scr.MyDisplayHeight = DisplayHeight(dpy, Scr.screen);
-
+  
+  fpa.mask = 0;
+  if (Pdepth <= 8)
+  {
+	  fpa.mask |= FPAM_DITHER;
+  }
   InitGetConfigLine(fd,CatString3("*",MyName,0));
   for (GetConfigLine(fd,&tline); tline != NULL; GetConfigLine(fd,&tline))
   {
@@ -1687,19 +1733,19 @@ void ParseOptions(void)
       {
 	window_x = g_x;
 	usposition = 1;
-        if (flags & XNegative)
-        {
-          xneg = 1;
-        }
+	if (flags & XNegative)
+	{
+	  xneg = 1;
+	}
       }
       if (flags & YValue)
       {
 	window_y = g_y;
 	usposition = 1;
-        if (flags & YNegative)
-        {
-          yneg = 1;
-        }
+	if (flags & YNegative)
+	{
+	  yneg = 1;
+	}
       }
     }
     else if (StrEquals(resource, "IconGeometry"))
@@ -1718,18 +1764,18 @@ void ParseOptions(void)
       if (flags & XValue)
       {
 	icon_x = g_x;
-        if (flags & XNegative)
-        {
-          icon_xneg = 1;
-        }
+	if (flags & XNegative)
+	{
+	  icon_xneg = 1;
+	}
       }
       if (flags & YValue)
       {
 	icon_y = g_y;
-        if (flags & YNegative)
-        {
-          icon_yneg = 1;
-        }
+	if (flags & YNegative)
+	{
+	  icon_yneg = 1;
+	}
       }
     }
     else if (StrEquals(resource, "Label"))
@@ -1749,24 +1795,16 @@ void ParseOptions(void)
     {
       if (font_string)
 	free(font_string);
-      if (next && next[0] == '\"')
+      CopyStringWithQuotes(&font_string, next);
+      if(strncasecmp(font_string,"none",4) == 0)
       {
-	int l;
-
-	next++;
-	CopyString(&font_string, next);
-	l = strlen(font_string);
-	if (l > 0 && font_string[l - 1] == '\"')
+	uselabel = 0;
+	if (font_string)
 	{
-	  font_string[l - 1] = 0;
+	  free(font_string);
+	  font_string = NULL;
 	}
       }
-      else
-      {
-	CopyString(&font_string,next);
-      }
-      if(strncasecmp(font_string,"none",4) == 0)
-	uselabel = 0;
     }
     else if (StrEquals(resource, "Fore"))
     {
@@ -1851,32 +1889,29 @@ void ParseOptions(void)
 	{
 	  if (item->next->bgPixmap != NULL)
 	  {
-	    DestroyPicture(dpy, item->next->bgPixmap);
+	    PDestroyFvwmPicture(dpy, item->next->bgPixmap);
 	    item->next->bgPixmap = NULL;
 	  }
-	  item->next->bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					       ImagePath,
-					       arg2, 0);
+	  item->next->bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
 	else
 	{
 	  /* new Dcolor and desktop */
 	  item = NewPagerStringItem(item, desk);
-	  item->bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					 ImagePath,
-					 arg2, 0);
+	  item->bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
 	if (desk == Scr.CurrentDesk)
 	{
 	  if (Desks[0].bgPixmap != NULL)
 	  {
-	    DestroyPicture(dpy, Desks[0].bgPixmap);
+	    PDestroyFvwmPicture(dpy, Desks[0].bgPixmap);
 	    Desks[0].bgPixmap = NULL;
 	  }
 
-	  Desks[0].bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					    ImagePath,
-					    arg2, 0);
+	  Desks[0].bgPixmap = PCacheFvwmPicture(
+		  dpy, Scr.Pager_w, ImagePath, arg2, fpa);
 	}
       }
       else if((desk >= desk1)&&(desk <=desk2))
@@ -1885,12 +1920,11 @@ void ParseOptions(void)
 
 	if (Desks[dNr].bgPixmap != NULL)
 	{
-	  DestroyPicture(dpy, Desks[dNr].bgPixmap);
+	  PDestroyFvwmPicture(dpy, Desks[dNr].bgPixmap);
 	  Desks[dNr].bgPixmap = NULL;
 	}
-	Desks[dNr].bgPixmap = CachePicture (dpy, Scr.Pager_w,
-					    ImagePath,
-					    arg2, 0);
+	Desks[dNr].bgPixmap = PCacheFvwmPicture(
+		dpy, Scr.Pager_w, ImagePath, arg2, fpa);
       }
 
 #ifdef DEBUG
@@ -1904,13 +1938,12 @@ void ParseOptions(void)
       if(Pdepth > 1)
       {
 	if (PixmapBack) {
-	  DestroyPicture (dpy, PixmapBack);
+	  PDestroyFvwmPicture (dpy, PixmapBack);
 	  PixmapBack = NULL;
 	}
 
-	PixmapBack = CachePicture (dpy, Scr.Pager_w,
-				   ImagePath,
-				   arg1, 0);
+	PixmapBack = PCacheFvwmPicture(
+		dpy, Scr.Pager_w, ImagePath, arg1, fpa);
 #ifdef DEBUG
 	fprintf(stderr,
 		"[ParseOptions]: Global: bgPixmap = %s\n", arg1);
@@ -1923,13 +1956,12 @@ void ParseOptions(void)
       if(Pdepth > 1)
       {
 	if (HilightPixmap) {
-	  DestroyPicture (dpy, HilightPixmap);
+	  PDestroyFvwmPicture (dpy, HilightPixmap);
 	  HilightPixmap = NULL;
 	}
 
-	HilightPixmap = CachePicture (dpy, Scr.Pager_w,
-				      ImagePath,
-				      arg1, 0);
+	HilightPixmap = PCacheFvwmPicture (
+		dpy, Scr.Pager_w, ImagePath, arg1, fpa);
 
 #ifdef DEBUG
 	fprintf(stderr,
@@ -1959,22 +1991,7 @@ void ParseOptions(void)
     {
       if (smallFont)
 	free(smallFont);
-      if (next && next[0] == '\"')
-      {
-	int l;
-
-	next++;
-	CopyString(&smallFont, next);
-	l = strlen(smallFont);
-	if (l > 0 && smallFont[l - 1] == '\"')
-	{
-	  smallFont[l - 1] = 0;
-	}
-      }
-      else
-      {
-	CopyString(&smallFont,next);
-      }
+      CopyStringWithQuotes(&smallFont, next);
       if (strncasecmp(smallFont,"none",4) == 0)
       {
 	free(smallFont);
@@ -2138,7 +2155,7 @@ void ParseOptions(void)
     {
       if (BalloonFont)
 	free(BalloonFont);
-      CopyString(&BalloonFont, next);
+      CopyStringWithQuotes(&BalloonFont, next);
     }
 
     else if (StrEquals(resource, "BalloonBorderColor"))
@@ -2156,6 +2173,17 @@ void ParseOptions(void)
     else if (StrEquals(resource, "BalloonYOffset"))
     {
       sscanf(arg1, "%d", &BalloonYOffset);
+      if (BalloonYOffset == 0)
+      {
+	fprintf(stderr,
+		"%s: Warning:"
+		" you're not allowed BalloonYOffset 0; defaulting to +3\n",
+		MyName);
+	/* we don't allow yoffset of 0 because it allows direct transit from
+	 * pager window to balloon window, setting up a LeaveNotify/EnterNotify
+	 * event loop */
+	BalloonYOffset = 3;
+      }
     }
     else if (StrEquals(resource,"BalloonStringFormat"))
     {
