@@ -25,17 +25,20 @@
 #include "libs/FShape.h"
 #include "libs/fvwmsignal.h"
 #include "libs/Picture.h"
+#include "libs/PictureGraphics.h"
+#include "libs/FRender.h"
+#include "libs/FRenderInit.h"
 
-#ifdef MEMDEBUG			/* For debugging */
+#ifdef MEMDEBUG                 /* For debugging */
 #include <unchecked.h>
 #endif
 
 /* Variables utilise par l'analyseur syntaxique */
 extern ScriptProp *scriptprop;
-extern int nbobj;			/* Nombre d'objets */
-extern int numligne;			/* Numero de ligne */
-extern TabObj *tabobj;			/* Tableau d'objets, limite=1000 */
-extern char **TabVVar;			/* Tableau des variables du sript */
+extern int nbobj;                       /* Nombre d'objets */
+extern int numligne;                    /* Numero de ligne */
+extern TabObj *tabobj;                  /* Tableau d'objets, limite=1000 */
+extern char **TabVVar;                  /* Tableau des variables du sript */
 extern int TabIdObj[1001];
 extern Bloc **TabIObj;
 extern CaseObj *TabCObj;
@@ -51,14 +54,15 @@ extern int __bounds_debug_no_checking;
 
 
 /* Variables globales */
-char *ScriptName;	/* Nom du fichier contenat le script decrivant le GUI */
+char *ScriptName;       /* Nom du fichier contenat le script decrivant le GUI */
 char *ScriptBaseName;
 char *ScriptPath = "";
 char *ModuleName;
-int fd[2]; 			/* pipe pair */
+int fd[2];                      /* pipe pair */
 int fd_err;
-int x_fd;			/* fd for X */
+int x_fd;                       /* fd for X */
 Window ref;
+FlocaleWinString *FwinString;
 
 extern int yyparse(void);
 extern void (*TabCom[25]) (int NbArg,long *TabArg);
@@ -66,8 +70,8 @@ extern void (*TabCom[25]) (int NbArg,long *TabArg);
 Display *dpy;
 int screen;
 Window  Root;
-X11base *x11base;		/* Pour le serveur X */
-TypeBuffSend BuffSend;		/* Pour les communication entre script */
+X11base *x11base;               /* Pour le serveur X */
+TypeBuffSend BuffSend;          /* Pour les communication entre script */
 int grab_server = 0;
 struct XObj *tabxobj[1000];
 char *Scrapt;
@@ -96,14 +100,14 @@ ShutdownX(void)
   int NbEssai=0;
   struct timeval tv;
 
-#ifdef DEBUG			/* For debugging */
+#ifdef DEBUG                    /* For debugging */
   XSync(dpy,0);
 #endif
 
   /* execute the QuitFunc */
   if (x11base->quitfunc != NULL)
     ExecBloc(x11base->quitfunc);
-  
+
   if (is_dead_pipe)
     return;
 
@@ -243,25 +247,25 @@ void ParseOptions(void)
       else if (strncasecmp(tline,"*FvwmScriptDefaultBack",22) == 0)
       {
 	x11base->colorset = -1;
-        CopyString(&x11base->backcolor,&tline[22]);
+	CopyString(&x11base->backcolor,&tline[22]);
       }
       else if (strncasecmp(tline,"*FvwmScriptDefaultFore",22) == 0)
       {
 	x11base->colorset = -1;
-        CopyString(&x11base->forecolor,&tline[22]);
+	CopyString(&x11base->forecolor,&tline[22]);
       }
       else if (strncasecmp(tline,"*FvwmScriptDefaultHilight",25) == 0)
       {
 	x11base->colorset = -1;
-        CopyString(&x11base->hilicolor,&tline[25]);
+	CopyString(&x11base->hilicolor,&tline[25]);
       }
       else if (strncasecmp(tline,"*FvwmScriptDefaultShadow",24) == 0)
       {
 	x11base->colorset = -1;
-        CopyString(&x11base->shadcolor,&tline[24]);
+	CopyString(&x11base->shadcolor,&tline[24]);
       }
       else if (strncasecmp(tline,"*FvwmScriptDefaultFont",22) == 0)
-	CopyString(&x11base->font,&tline[22]);
+	CopyStringWithQuotes(&x11base->font,&tline[22]);
       else if (strncasecmp(tline,"ColorLimit",10) == 0)
 	save_color_limit = atoi(&tline[10]);
       else if (strncasecmp(tline,"Colorset",8) == 0)
@@ -281,7 +285,8 @@ static int myErrorHandler(Display *dpy, XErrorEvent *event)
   /* some errors are acceptable, mostly they're caused by
    * trying to use a deleted pixmap */
   if((event->error_code == BadDrawable) || (event->error_code == BadPixmap) ||
-     (event->error_code == BadWindow))
+     (event->error_code == BadWindow) ||
+     (event->error_code == FRenderGetErrorCodeBase() + FRenderBadPicture))
     return 0;
 
   PrintXErrorAndCoredump(dpy, event, x11base->title);
@@ -311,10 +316,12 @@ void Xinit(int IsFather)
   }
   screen = DefaultScreen(dpy);
   Root = RootWindow(dpy,screen);
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
+  FRenderInit(dpy);
+  FlocaleAllocateWinString(&FwinString);
   XSetErrorHandler(myErrorHandler);
 
 #ifdef MEMDEBUG
@@ -348,24 +355,75 @@ void Xinit(int IsFather)
 /***********************/
 void LoadIcon(struct XObj *xobj)
 {
-  Picture *pic;
+	char *path;
+	int depth;
+	FvwmPictureAttributes fpa;
 
-  if ((xobj->icon) != NULL)
-  {
-    pic = CachePicture(dpy,x11base->win,imagePath,xobj->icon,save_color_limit);
-    if (!pic)
-    {
-      fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to load pixmap %s\n",
-	      ScriptName,xobj->icon);
-      xobj->iconPixmap = None;
-      xobj->icon_maskPixmap = None;
-      return;
-    }
-    xobj->iconPixmap = pic->picture;
-    xobj->icon_maskPixmap = pic->mask;
-    xobj->icon_w = pic->width;
-    xobj->icon_h = pic->height;
-  }
+	fpa.mask = 0;
+	if (xobj->colorset >= 0 && Colorset[xobj->colorset].do_dither_icon)
+	{
+		fpa.mask |= FPAM_DITHER;
+	}
+	if (xobj->iconPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->iconPixmap);
+		xobj->iconPixmap = None;
+	}
+	if (xobj->icon_maskPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->icon_maskPixmap);
+		xobj->icon_maskPixmap = None;
+	}
+	if (xobj->icon_alphaPixmap != None)
+	{
+		XFreePixmap(dpy, xobj->icon_alphaPixmap);
+		xobj->icon_alphaPixmap = None;
+	}
+	if (xobj->alloc_pixels != NULL && xobj->nalloc_pixels > 0)
+	{
+		PictureFreeColors(
+			dpy, Pcmap, xobj->alloc_pixels, xobj->nalloc_pixels, 0,
+			False);
+		xobj->alloc_pixels = NULL;
+		xobj->nalloc_pixels = 0;
+	}
+	if ((xobj->icon) == NULL)
+		return;
+	if ((path = PictureFindImageFile(xobj->icon,imagePath,R_OK)) == NULL)
+	{
+		return;
+	}
+	if (!PImageLoadPixmapFromFile(
+		dpy, x11base->root, path, &xobj->iconPixmap,
+		&xobj->icon_maskPixmap, &xobj->icon_alphaPixmap, &xobj->icon_w,
+		&xobj->icon_h, &depth, &xobj->nalloc_pixels, &xobj->alloc_pixels,
+		fpa))
+	{
+		fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to "
+			"load pixmap %s\n",
+			ScriptName,xobj->icon);
+		free(path);
+		return;
+	}
+	if (depth <= 1)
+	{
+		XGCValues gcv;
+		GC m_gc;
+		Pixmap temp;
+
+		gcv.background= WhitePixel(dpy, screen);
+		gcv.foreground= BlackPixel(dpy, screen);
+		m_gc = fvwmlib_XCreateGC(dpy, x11base->win,
+				       GCForeground | GCBackground, &gcv);
+		temp = XCreatePixmap(dpy, x11base->win,
+				     xobj->icon_w, xobj->icon_h, Pdepth);
+		XCopyPlane(dpy, xobj->iconPixmap, temp, m_gc,
+			   0,0,xobj->icon_w, xobj->icon_h, 0,0,1);
+		XFreePixmap(dpy, xobj->iconPixmap);
+		XFreeGC(dpy,m_gc);
+		xobj->iconPixmap = temp;
+	}
+	free(path);
 }
 
 
@@ -552,7 +610,7 @@ void BuildGUI(int IsFather)
     else
       tabxobj[i]->title = (*tabobj)[i].title;
 
-    if ((*tabobj)[i].font == NULL)
+    if ((*tabobj)[i].font == NULL && x11base->font != NULL)
       tabxobj[i]->font = (char*)safestrdup(x11base->font);
     else
       tabxobj[i]->font = (*tabobj)[i].font;
@@ -591,7 +649,7 @@ void BuildGUI(int IsFather)
     tabxobj[i]->iconPixmap = None;
     tabxobj[i]->icon_maskPixmap = None;
 
-    LoadIcon(tabxobj[i]);	           /* Chargement de l'icone du widget */
+    LoadIcon(tabxobj[i]);                  /* Chargement de l'icone du widget */
 
     tabxobj[i]->InitObj(tabxobj[i]);
   }
@@ -600,7 +658,7 @@ void BuildGUI(int IsFather)
   x11base->periodictasks = scriptprop->periodictasks;
   /* the QuitFunc */
   x11base->quitfunc = scriptprop->quitfunc;
-   
+
   /*Si un bloc d'initialisation du script existe, on l'execute ici */
   if (scriptprop->initbloc != NULL)
   {
@@ -745,9 +803,9 @@ void UpdateRootTransparency(void)
     for (i=0; i<nbobj; i++)
     {
       if (Rectangle != tabxobj[i]->TypeWidget &&
-	  SwallowExec != tabxobj[i]->TypeWidget && 
-	  tabxobj[i]->colorset > 0 &&
-	  Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)
+	  SwallowExec != tabxobj[i]->TypeWidget &&
+	  ((tabxobj[i]->colorset > 0 &&
+	   Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
       {
 	XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
       }
@@ -823,7 +881,7 @@ void ReadXServer (void)
       /* check for bindings defined by the Key instruction */
       if ((action = CheckBinding(BindingsList, STROKE_ARG(0)
 				 event.xkey.keycode,
-				 event.xkey.state, LockMask, 
+				 event.xkey.state, LockMask,
 				 C_WINDOW, KEY_BINDING)) != NULL)
       {
 	SendMsgAndString(action, "CheckBinding");
@@ -1035,17 +1093,37 @@ void MainLoop (void)
 	      SetWindowBackground(dpy, x11base->win, x11base->size.width,
 				  x11base->size.height, &Colorset[n], Pdepth,
 				  x11base->gc, True);
+	      for (i=0; i<nbobj; i++)
+	      {
+		if (Rectangle != tabxobj[i]->TypeWidget &&
+		    SwallowExec != tabxobj[i]->TypeWidget &&
+		    ((tabxobj[i]->colorset > 0 &&
+		      Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)))
+		{
+		  XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
+		}
+	      }
 	    }
 	  }
 	  else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
 	    FScreenConfigureModule(line);
 	  }
-	  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+	  if (token)
+	    free(token);
+	}
+	else if (packet->type == MX_PROPERTY_CHANGE)
+	{
+	  if (packet->body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
+	      ((!x11base->swallowed && packet->body[2] == 0) ||
+	       (x11base->swallowed && packet->body[2] == x11base->win)))
 	  {
 	    UpdateRootTransparency();
 	  }
-	  if (token)
-	    free(token);
+	  else if  (packet->body[0] == MX_PROPERTY_CHANGE_SWALLOW &&
+		    packet->body[2] == x11base->win)
+	  {
+	    x11base->swallowed = packet->body[1];
+	  }
 	}
 	else if (packet->type == M_STRING) {
 	  char *action, *token;
@@ -1077,19 +1155,19 @@ void ReadFvwmScriptArg(int argc, char **argv,int IsFather)
   Atom myatom;
   int FisrtArg;
 
-  BuffSend.NbMsg=0;			/* Aucun message dans le buffer */
+  BuffSend.NbMsg=0;                     /* Aucun message dans le buffer */
 
   for (i=2; i<98; i++)
     x11base->TabScriptId[i]=NULL;
 
-  if (IsFather)			/* Cas du pere */
+  if (IsFather)                 /* Cas du pere */
   {
     myatom = XInternAtom(dpy,x11base->TabScriptId[1],True);
     XSetSelectionOwner(dpy, myatom, x11base->win, CurrentTime);
     FisrtArg = 9;
   }
   else
-  {				/* Cas du fils */
+  {                             /* Cas du fils */
     x11base->TabScriptId[0] = (char*)safecalloc(sizeof(char),strlen(argv[7]));
     x11base->TabScriptId[0] = strncpy(x11base->TabScriptId[0],argv[7],
 				    strlen(argv[7])-2);
@@ -1115,9 +1193,7 @@ int main (int argc, char **argv)
   int IsFather;
   int i;
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmScript");
 
   ModuleName = GetFileNameFromPath(argv[0]);
 
@@ -1148,19 +1224,20 @@ int main (int argc, char **argv)
   SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST| M_STRING |
 		 M_MAP|  M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
 		 M_END_CONFIG_INFO| M_WINDOW_NAME | M_SENDCONFIG);
-
+  SetMessageMask(fd, MX_PROPERTY_CHANGE);
   /* Enregistrement des arguments du script */
   x11base = (X11base*) safecalloc(1,sizeof(X11base));
   x11base->TabArg[0] = ModuleName;
   for (i=8-IsFather; i<argc; i++)
     x11base->TabArg[i-7+IsFather] = argv[i];
   /* Couleurs et fontes par defaut */
-  x11base->font = safestrdup("fixed");
+  x11base->font = NULL;
   x11base->forecolor = safestrdup("black");
   x11base->backcolor = safestrdup("grey85");
   x11base->shadcolor = safestrdup("grey55");
   x11base->hilicolor = safestrdup("grey100");
   x11base->colorset = -1;
+  x11base->swallowed = False;
 
   /* Initialisation du serveur X et de la fenetre */
   Xinit(IsFather);
@@ -1169,7 +1246,7 @@ int main (int argc, char **argv)
 
   SendText(fd,"Send_WindowList",0);
 
-  ReadConfig(ScriptName);	/* Lecture et analyse du script */
+  ReadConfig(ScriptName);       /* Lecture et analyse du script */
 
   /* Fonction d'initialisation de TabCom et TabFunc   */
   InitCom();
@@ -1199,10 +1276,10 @@ int main (int argc, char **argv)
 #else
 #ifdef USE_BSD_SIGNALS
   fvwmSetSignalMask( sigmask(SIGPIPE) |
-                     sigmask(SIGINT) |
-                     sigmask(SIGHUP) |
-                     sigmask(SIGTERM) |
-                     sigmask(SIGQUIT) );
+		     sigmask(SIGINT) |
+		     sigmask(SIGHUP) |
+		     sigmask(SIGTERM) |
+		     sigmask(SIGQUIT) );
 #endif
   signal (SIGPIPE, TerminateHandler);
   signal (SIGINT, TerminateHandler);  /* cleanup on other ways of closing too */
