@@ -15,8 +15,7 @@
 
 #include "types.h"
 #include "libs/fvwmsignal.h"
-#include <time.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #ifdef HAVE_GETPWUID
 #  include <pwd.h>
 #endif
@@ -138,7 +137,7 @@ static char *CalcArg (long *TabArg,int *Ix)
   char *TmpStr;
   int i;
 
-  if (TabArg[*Ix]>100000)	/* Cas du codage d'un nombre */
+  if (TabArg[*Ix]>100000)       /* Cas du codage d'un nombre */
   {
     i = (int)TabArg[*Ix] - 200000;
     TmpStr = (char*)safecalloc(1,sizeof(char)*10);
@@ -150,11 +149,11 @@ static char *CalcArg (long *TabArg,int *Ix)
    TmpStr = (char*)safecalloc(1,sizeof(char)*10);
    sprintf(TmpStr,"%d",i);
   }
-  else if (TabArg[*Ix] < -100000)	/* Cas d'un id de fonction */
+  else if (TabArg[*Ix] < -100000)       /* Cas d'un id de fonction */
   {
     TmpStr = TabFunc[TabArg[*Ix]+150000](Ix,TabArg);
   }
-  else				/* Cas d'une variable */
+  else                          /* Cas d'une variable */
   {
     TmpStr=safestrdup(TabVVar[TabArg[*Ix]]);
   }
@@ -171,7 +170,7 @@ static char *FuncGetValue(int *NbArg, long *TabArg)
   char *tmp;
   long Id;
 
-  (*NbArg)++;		/* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;           /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -186,7 +185,7 @@ static char *FuncGetMinValue(int *NbArg, long *TabArg)
   char *tmp;
   long Id;
 
-  (*NbArg)++;		   /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;              /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -201,7 +200,7 @@ static char *FuncGetMaxValue(int *NbArg, long *TabArg)
   char *tmp;
   long Id;
 
-  (*NbArg)++;	      /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;         /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -217,7 +216,7 @@ static char *FuncGetFore(int *NbArg, long *TabArg)
   long Id;
   XColor color;
 
-  (*NbArg)++;		 /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;            /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -236,7 +235,7 @@ static char *FuncGetBack(int *NbArg, long *TabArg)
   long Id;
   XColor color;
 
-  (*NbArg)++;		  /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;             /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -255,7 +254,7 @@ static char *FuncGetHili(int *NbArg, long *TabArg)
   long Id;
   XColor color;
 
-  (*NbArg)++;	    /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;       /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -274,7 +273,7 @@ static char *FuncGetShad(int *NbArg, long *TabArg)
   long Id;
   XColor color;
 
-  (*NbArg)++;	    /* La fonction GetValue n'a qu'un seul argument */
+  (*NbArg)++;       /* La fonction GetValue n'a qu'un seul argument */
   tmp = CalcArg(TabArg,NbArg);
   Id = atoi(tmp);
   free(tmp);
@@ -381,7 +380,7 @@ static char *FuncGetOutput(int *NbArg, long *TabArg)
     str = (char*)safecalloc(sizeof(char),255);
     sscanf(&BufCom[j],"%s",str);
   }
-  else		/* Lecture de la ligne complete */
+  else          /* Lecture de la ligne complete */
   {
     if (i != 2) j++;
     k=j;
@@ -1150,12 +1149,7 @@ static void ChangeFont (int NbArg,long *TabArg)
   int i=0;
   char *arg[2];
   int IdItem;
-#ifdef I18N_MB
-  char **ml;
-  XFontStruct **fs_list;
-#else
-  XFontStruct *xfont;
-#endif
+  FlocaleFont *Ffont;
 
   arg[0] = CalcArg(TabArg,&i);
   i++;
@@ -1164,32 +1158,20 @@ static void ChangeFont (int NbArg,long *TabArg)
   if (tabxobj[IdItem]->font)
     free(tabxobj[IdItem]->font);
   tabxobj[IdItem]->font = safestrdup(arg[1]);
-#ifdef I18N_MB
-  /* Hmm.. Fontset is not freed. However, original alogrithm does not consider
-   * the situation of font-loading-falure.
-  */
-  if ((tabxobj[IdItem]->xfontset = GetFontSetOrFixed(dpy,tabxobj[IdItem]->font))
-      == NULL) {
-    fprintf(stderr, "[%s][ChangeFont]: Couldn't load font %s. Exiting!\n",
-	    ScriptName,tabxobj[IdItem]->font);
-    exit(1);
-  }
-  XFontsOfFontSet(tabxobj[IdItem]->xfontset, &fs_list, &ml);
-  tabxobj[IdItem]->xfont = fs_list[0];
-  XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->xfont->fid);
-#else
-  if ((xfont = XLoadQueryFont(dpy,tabxobj[IdItem]->font)) == NULL)
-  {
-    fprintf(stderr,"[%s][ChangeFont]: Can't load font %s\n",
+
+  if ((Ffont =
+       FlocaleLoadFont(dpy, tabxobj[IdItem]->font, ScriptName)) == NULL) {
+    fprintf(stderr, "[%s][ChangeFont]: Couldn't load font %s\n",
 	    ScriptName,tabxobj[IdItem]->font);
   }
   else
   {
-    XFreeFont(dpy,tabxobj[IdItem]->xfont);
-    tabxobj[IdItem]->xfont = xfont;
-    XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->xfont->fid);
+    FlocaleUnloadFont(dpy, tabxobj[IdItem]->Ffont);
+    tabxobj[IdItem]->Ffont = Ffont;
+    if (Ffont->font != NULL)
+      XSetFont(dpy,tabxobj[IdItem]->gc,tabxobj[IdItem]->Ffont->font->fid);
   }
-#endif
+
   if (tabxobj[IdItem]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[IdItem]->win);
   tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
@@ -1280,7 +1262,8 @@ static void ChangeForeColor (int NbArg,long *TabArg)
 
   /* Liberation de la couleur */
   if (tabxobj[IdItem]->colorset < 0)
-    XFreeColors(dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[fore]),1,0);
+    PictureFreeColors(
+	    dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[fore]),1,0, True);
 
   tabxobj[IdItem]->forecolor=(char*)safecalloc(100,sizeof(char));
   sprintf(tabxobj[IdItem]->forecolor,"%s",arg[1]);
@@ -1316,7 +1299,8 @@ static void ChangeBackColor (int NbArg,long *TabArg)
 
   /* Liberation de la couleur */
   if (tabxobj[IdItem]->colorset < 0)
-    XFreeColors(dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[back]),1,0);
+    PictureFreeColors(
+	    dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[back]),1,0,True);
 
   tabxobj[IdItem]->backcolor = (char*)safecalloc(100,sizeof(char));
   sprintf(tabxobj[IdItem]->backcolor,"%s",arg[1]);
@@ -1346,10 +1330,10 @@ static void ChangeMainColorset (int i)
 {
  /* Liberation de la couleur */
   if (x11base->colorset < 0) {
-    XFreeColors(dpy,Pcmap,&x11base->TabColor[fore],1,0);
-    XFreeColors(dpy,Pcmap,&x11base->TabColor[back],1,0);
-    XFreeColors(dpy,Pcmap,&x11base->TabColor[hili],1,0);
-    XFreeColors(dpy,Pcmap,&x11base->TabColor[shad],1,0);
+    PictureFreeColors(dpy,Pcmap,&x11base->TabColor[fore],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&x11base->TabColor[back],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&x11base->TabColor[hili],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&x11base->TabColor[shad],1,0,True);
   }
   x11base->colorset = i;
   AllocColorset(i);
@@ -1381,10 +1365,10 @@ static void ChangeColorset (int NbArg,long *TabArg)
 
   /* Liberation de la couleur */
   if (tabxobj[IdItem]->colorset < 0) {
-    XFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[fore],1,0);
-    XFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[back],1,0);
-    XFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[hili],1,0);
-    XFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[shad],1,0);
+    PictureFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[fore],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[back],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[hili],1,0,True);
+    PictureFreeColors(dpy,Pcmap,&tabxobj[IdItem]->TabColor[shad],1,0,True);
   }
   sscanf(arg[1], "%d", &i);
   tabxobj[IdItem]->colorset = i;
@@ -1444,8 +1428,8 @@ static void SendSign (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* WarpPointer *******/
-static void WarpPointer(int NbArg,long *TabArg)
+/******* MyWarpPointer *******/
+static void MyWarpPointer(int NbArg,long *TabArg)
 {
   int i=0;
   char *arg;
@@ -1476,7 +1460,7 @@ static void IfThen (int NbArg,long *TabArg)
   /* Verification de la condition */
   for (j=0; j<NbArg-2; j++)
   {
-    if (TabArg[j] > 100000)	/* Cas du codage d'un nombre */
+    if (TabArg[j] > 100000)     /* Cas du codage d'un nombre */
     {
       i = (int)TabArg[j] - 200000;
       arg[CurrArg] = (char*)safecalloc(1,sizeof(char)*10);
@@ -1487,12 +1471,12 @@ static void IfThen (int NbArg,long *TabArg)
     {
       IdFuncComp = TabArg[j] + 250000;
     }
-    else if (TabArg[j] < -100000)	/* Cas d'un id de fonction */
+    else if (TabArg[j] < -100000)       /* Cas d'un id de fonction */
     {
       arg[CurrArg] = TabFunc[TabArg[j]+150000](&j,TabArg);
       CurrArg++;
     }
-    else				/* Cas d'une variable */
+    else                                /* Cas d'une variable */
     {
       arg[CurrArg] = safestrdup(TabVVar[TabArg[j]]);
       CurrArg++;
@@ -1525,17 +1509,17 @@ static void Loop (int NbArg,long *TabArg)
   /* Calcul des 2 autres arguments */
   for (i=1; i<NbArg-1; i++)
   {
-    if (TabArg[i] > 100000)	/* Cas du codage d'un nombre */
+    if (TabArg[i] > 100000)     /* Cas du codage d'un nombre */
     {
       i = (int)TabArg[i] - 200000;
       arg[CurrArg] = (char*)safecalloc(1,sizeof(char)*10);
       sprintf(arg[CurrArg],"%d",i);
     }
-    else if (TabArg[i] < -100000)	/* Cas d'un id de fonction */
+    else if (TabArg[i] < -100000)       /* Cas d'un id de fonction */
     {
       arg[CurrArg] = TabFunc[TabArg[i]+150000](&i,TabArg);
     }
-    else				/* Cas d'une variable */
+    else                                /* Cas d'une variable */
       arg[CurrArg] = safestrdup(TabVVar[TabArg[i]]);
     CurrArg++;
   }
@@ -1661,7 +1645,7 @@ static void WriteToFile (int NbArg,long *TabArg)
       memmove(&buf[CurrPos+strlen(arg[1])],&buf[CurrPos],strlen(buf)-CurrPos);
       memmove(&buf[CurrPos],arg[1],strlen(arg[1]));
     }
-    else	  /* Remplacement des anciennes commandes */
+    else          /* Remplacement des anciennes commandes */
     {
       CurrPos = CurrPos+strlen(StrBegin);
       CurrPos2 = CurrPos;
@@ -1829,7 +1813,7 @@ void InitCom()
   TabCom[14]=IfThen;
   TabCom[15]=Loop;
   TabCom[16]=While;
-  TabCom[17]=WarpPointer;
+  TabCom[17]=MyWarpPointer;
   TabCom[18]=WriteToFile;
   TabCom[19]=ChangeBackColor;
   TabCom[21]=ChangeValueMax;
