@@ -23,10 +23,9 @@ void InitTextField(struct XObj *xobj)
  unsigned long mask;
  XSetWindowAttributes Attr;
  int i;
- int asc,desc,dir;
- XCharStruct struc;
 
  /* Enregistrement des couleurs et de la police */
+ /* colors and fonts */
  if (xobj->colorset >= 0) {
   xobj->TabColor[fore] = Colorset[xobj->colorset].fg;
   xobj->TabColor[back] = Colorset[xobj->colorset].bg;
@@ -41,7 +40,7 @@ void InitTextField(struct XObj *xobj)
 
  mask=0;
  Attr.cursor=XCreateFontCursor(dpy,XC_xterm);
- mask|=CWCursor;		/* Curseur pour la fenetre */
+ mask|=CWCursor;                /* Curseur pour la fenetre / window cursor */
  Attr.background_pixel=xobj->TabColor[back];
  mask|=CWBackPixel;
 
@@ -51,13 +50,18 @@ void InitTextField(struct XObj *xobj)
 		mask,&Attr);
  xobj->gc=fvwmlib_XCreateGC(dpy,xobj->win,0,NULL);
  XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]);
- if ((xobj->xfont=XLoadQueryFont(dpy,xobj->font))==NULL)
-   fprintf(stderr,"Can't load font %s\n",xobj->font);
- else
-  XSetFont(dpy,xobj->gc,xobj->xfont->fid);
+
+ if ((xobj->Ffont = FlocaleLoadFont(dpy, xobj->font, ScriptName)) == NULL)
+ {
+   fprintf(stderr, "%s: Couldn't load font. Exiting!\n", ScriptName);
+   exit(1);
+ }
+ if (xobj->Ffont->font != NULL)
+   XSetFont(dpy, xobj->gc, xobj->Ffont->font->fid);
 
  XSetLineAttributes(dpy,xobj->gc,1,LineSolid,CapRound,JoinMiter);
  /* value2 reprsente la fin de la zone selectionnee */
+ /* value2 gives the end of the selected zone */
  if (xobj->value>strlen(xobj->title))
   xobj->value=strlen(xobj->title);
  xobj->value2=xobj->value;
@@ -65,39 +69,37 @@ void InitTextField(struct XObj *xobj)
  xobj->value3=0;
 
  /* Redimensionnement du widget */
- XTextExtents(xobj->xfont,"lp",strlen("lp"),&dir,&asc,&desc,&struc);
- xobj->height=asc+desc+10;
- i=XTextWidth(xobj->xfont,xobj->title,strlen(xobj->title))+40;
+ /* widget resizing */
+ xobj->height= xobj->Ffont->height + 10;
+ i = FlocaleTextWidth(xobj->Ffont,xobj->title,strlen(xobj->title))+40;
  if (xobj->width<i)
   xobj->width=i;
  XResizeWindow(dpy,xobj->win,xobj->width,xobj->height);
  if (xobj->colorset >= 0)
    SetWindowBackground(dpy, xobj->win, xobj->width, xobj->height,
-                       &Colorset[xobj->colorset], Pdepth,
-                       xobj->gc, True);
+		       &Colorset[xobj->colorset], Pdepth,
+		       xobj->gc, True);
  XSelectInput(dpy, xobj->win, ExposureMask);
 }
 
 void DestroyTextField(struct XObj *xobj)
 {
- XFreeFont(dpy,xobj->xfont);
+ FlocaleUnloadFont(dpy,xobj->Ffont);
  XFreeGC(dpy,xobj->gc);
  XDestroyWindow(dpy,xobj->win);
 }
 
 /* Dessin du curseur du texte */
+/* text cursor drawing */
 void DrawPointTxt(struct XObj *xobj,unsigned int pixel)
 {
 #define dec 2
  int x,y;
  XSegment segm[2];
- int asc,desc,dir;
- XCharStruct struc;
 
- XTextExtents(xobj->xfont,"lp",strlen("lp"),&dir,&asc,&desc,&struc);
- x=XTextWidth(xobj->xfont, xobj->title + xobj->value3, 
+ x = FlocaleTextWidth(xobj->Ffont, xobj->title + xobj->value3,
 	      xobj->value - xobj->value3)+5;
- y=asc+5;
+ y = xobj->Ffont->ascent + 5;
 
  segm[0].x1=x;
  segm[0].y1=y;
@@ -112,15 +114,15 @@ void DrawPointTxt(struct XObj *xobj,unsigned int pixel)
 }
 
 /* Dessin du contenu du champs texte */
+/* text field drawing */
 void DrawTextField(struct XObj *xobj)
 {
  int x1,y1;
  int x2,l;
- int desc,dir,asc;
- XCharStruct struc;
  int nl=0;
  int right=0;
 
+ y1 = xobj->Ffont->ascent;
  l=strlen(xobj->title);
  if (xobj->value>l)
   xobj->value=l;
@@ -129,49 +131,55 @@ void DrawTextField(struct XObj *xobj)
  DrawReliefRect(0,0,xobj->width,xobj->height,xobj,shad,hili);
  XClearArea(dpy,xobj->win,2,2,xobj->width-4,xobj->height-4,False);
  XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]);
- XTextExtents(xobj->xfont,"lp",strlen("lp"),&dir,&y1,&desc,&struc);
  /* calcul du premier caractere visible */
+ /* computation of the first visible character */
  while (l-nl >= 1 &&
-	 XTextWidth(xobj->xfont,xobj->title + nl,
-		    xobj->value - nl) > (xobj->width-10))
+	 FlocaleTextWidth(xobj->Ffont,xobj->title + nl,
+			  xobj->value - nl) > (xobj->width-10))
    { nl++; }
  if (nl>xobj->value3) { xobj->value3 = nl; }
  else if (xobj->value3>xobj->value) { xobj->value3--; }
  /* calcul de la longueur du titre visible */
- while ( l-xobj->value3-right >= 1 && 
-	 XTextWidth(xobj->xfont,xobj->title + xobj->value3,
-		    l - xobj->value3 - right) > (xobj->width-10))
+ /* computation of the length of the visible title */
+ while ( l-xobj->value3-right >= 1 &&
+	 FlocaleTextWidth(xobj->Ffont,xobj->title + xobj->value3,
+			  l - xobj->value3 - right) > (xobj->width-10))
  { right++; }
-#ifdef I18N_MB
+ FwinString->win = xobj->win;
+ FwinString->gc = xobj->gc;
+ FwinString->x = 5;
+ FwinString->y = y1+5;
+ FwinString->str = xobj->title + xobj->value3;
+ FwinString->len = strlen(xobj->title) - xobj->value3-right;
+ FlocaleDrawString(dpy, xobj->Ffont, FwinString, FWS_HAVE_LENGTH);
+#if 0
  XmbDrawString(dpy,xobj->win,xobj->xfontset,xobj->gc,5,y1+5,
 	       xobj->title + xobj->value3,
 	       strlen(xobj->title) - xobj->value3-right);
-#else
- XDrawString(dpy,xobj->win,xobj->gc,5,y1+5,xobj->title+xobj->value3,
-	     strlen(xobj->title) - xobj->value3-right);
 #endif
 
  /* Dessin de la zone selectionnee */
+ /* selected zone drawing */
  XSetFunction(dpy,xobj->gc,GXinvert);
- if (xobj->value2>xobj->value)		/* Curseur avant la souris */
+ if (xobj->value2>xobj->value)          /* Curseur avant la souris */
  {
-  x1=XTextWidth(xobj->xfont,&xobj->title[xobj->value3],
-		xobj->value - xobj->value3);
-  x2=XTextWidth(xobj->xfont,&xobj->title[xobj->value],
-		xobj->value2 - xobj->value);
+  x1=FlocaleTextWidth(xobj->Ffont,&xobj->title[xobj->value3],
+		      xobj->value - xobj->value3);
+  x2=FlocaleTextWidth(xobj->Ffont,&xobj->title[xobj->value],
+		      xobj->value2 - xobj->value);
  }
- else		/* Curseur apres la souris */
+ else           /* Curseur apres la souris / cursor after the mouse */
  {
-  x1=XTextWidth(xobj->xfont,&xobj->title[xobj->value3],
-		xobj->value2 - xobj->value3);
-  x2=XTextWidth(xobj->xfont,&xobj->title[xobj->value2],
-		xobj->value - xobj->value2);
+  x1=FlocaleTextWidth(xobj->Ffont,&xobj->title[xobj->value3],
+		      xobj->value2 - xobj->value3);
+  x2=FlocaleTextWidth(xobj->Ffont,&xobj->title[xobj->value2],
+		      xobj->value - xobj->value2);
  }
- XTextExtents(xobj->xfont,"lp",strlen("lp"),&dir,&asc,&desc,&struc);
- XFillRectangle(dpy,xobj->win,xobj->gc,x1+5,7,x2,y1+desc-2);
+ XFillRectangle(dpy,xobj->win,xobj->gc,x1+5,7,x2,y1+xobj->Ffont->descent-2);
  XSetFunction(dpy,xobj->gc,GXcopy);
 
  /* Dessin du point d'insertion */
+ /* insertion point drawing */
  DrawPointTxt(xobj,xobj->TabColor[fore]);
 }
 
@@ -194,6 +202,7 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
 
  /* On deplace le curseur a la position de la souris */
  /* On recupere la position de la souris */
+ /* We move the cursor at mouse position and we get the mouse position */
  switch (EvtButton->button)
  {
   case Button1:
@@ -201,7 +210,7 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
     x2=x2-xobj->x;
     PosCurs=0;
     while ((PosCurs<strlen(xobj->title+xobj->value3))&&
-	   (x2>XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)+8))
+	   (x2>FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)+8))
      PosCurs++;
     DrawPointTxt(xobj,xobj->TabColor[back]);
     xobj->value=PosCurs+xobj->value3;
@@ -219,29 +228,33 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
        x2=x2-xobj->x;
        PosCurs=0;
        while ((PosCurs<strlen(xobj->title+xobj->value3))&&
-	      (x2>XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)+8))
-     	PosCurs++;
+	      (x2 >
+	       FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)+8))
+	PosCurs++;
        /* Limitation de la zone de dessin */
+       /* limitation of the drawing zone */
        if (PosCurs>xobj->value2)
        {
-        rect.x=XTextWidth(xobj->xfont,xobj->title+xobj->value3,xobj->value2);
-        rect.y=0;
-        rect.width=XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs+1)
+	rect.x=
+	  FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,xobj->value2);
+	rect.y=0;
+	rect.width=
+	  FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs+1)
 	  -rect.x+1;
-        rect.height=xobj->height;
-        xobj->value2=PosCurs+xobj->value3;
-        DrawTextField(xobj);
+	rect.height=xobj->height;
+	xobj->value2=PosCurs+xobj->value3;
+	DrawTextField(xobj);
        }
        else
        if (PosCurs<xobj->value2)
        {
-        rect.x=XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)-1;
-        rect.y=0;
-        rect.width=XTextWidth(xobj->xfont,xobj->title+xobj->value3,
+	rect.x=FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)-1;
+	rect.y=0;
+	rect.width=FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,
 			      xobj->value2+1) - rect.x+2;
-        rect.height=xobj->height;
-        xobj->value2=PosCurs+xobj->value3;
-        DrawTextField(xobj);
+	rect.height=xobj->height;
+	xobj->value2=PosCurs+xobj->value3;
+	DrawTextField(xobj);
        }
 
       break;
@@ -252,6 +265,7 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
     }
     /* Enregistrement de la selection dans le presse papier */
     /* Le programme devient proprietaire de la selection */
+    /* selection stuff: get the selection */
     if (xobj->value!=xobj->value2)
     {
      str=(char*)GetText(xobj,xobj->value2);
@@ -266,14 +280,16 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
     }
    break;
 
-   case Button2:			/* Colle le texte */
-    /* Si l'application possede pas la selection, elle la demande */
-    /* sinon elle lit son presse papier */
+   case Button2:                        /* Colle le texte */
+     /* Si l'application possede pas la selection, elle la demande */
+     /* sinon elle lit son presse papier */
+     /* read the selection */
     if (!x11base->HaveXSelection)
     {
-     /* Demande de la selection */
+      /* Demande de la selection */
+      /* ask for the selection */
      XConvertSelection(dpy,XA_PRIMARY,XA_STRING,propriete,*xobj->ParentWin,
-    			EvtButton->time);
+			EvtButton->time);
      while (!(XCheckTypedEvent(dpy,SelectionNotify,&event)))
       ;
      if (event.xselection.property!=None)
@@ -285,11 +301,11 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
 			  &longueur,&octets_restant,&donnees);
        if (longueur>0)
        {
-        Scrapt=(char*)realloc((void*)Scrapt,(longueur+1)*sizeof(char));
-        Scrapt=strcpy(Scrapt,(char *)donnees);
-        XDeleteProperty(dpy,event.xselection.requestor,
+	Scrapt=(char*)realloc((void*)Scrapt,(longueur+1)*sizeof(char));
+	Scrapt=strcpy(Scrapt,(char *)donnees);
+	XDeleteProperty(dpy,event.xselection.requestor,
 			event.xselection.property);
-        XFree(donnees);
+	XFree(donnees);
        }
       }
      }
@@ -306,12 +322,12 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
     }
     break;
 
-   case Button3:		/* Appuie sur le troisieme bouton */
+   case Button3:                /* Appuie sur le troisieme bouton */
     XQueryPointer(dpy,*xobj->ParentWin,&Win1,&Win2,&x1,&y1,&x2,&y2,&modif);
     x2=x2-xobj->x;
     PosCurs=0;
     while ((PosCurs<strlen(xobj->title))&&
-	   (x2>XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)+8))
+	   (x2>FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)+8))
      PosCurs++;
     if ((PosCurs<xobj->value) && (xobj->value<xobj->value2))
       xobj->value=xobj->value2;
@@ -329,28 +345,31 @@ void EvtMouseTextField(struct XObj *xobj,XButtonEvent *EvtButton)
        XQueryPointer(dpy,*xobj->ParentWin,&Win1,&Win2,&x1,&y1,&x2,&y2,&modif);
        x2=x2-xobj->x;
        while ((PosCurs<strlen(xobj->title))&&
-	      (x2>XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)+8))
-     	PosCurs++;
+	      (x2 >
+	       FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)+8))
+	PosCurs++;
        if (PosCurs>xobj->value2)
        {
-        rect.x=XTextWidth(xobj->xfont,xobj->title+xobj->value3,xobj->value2);
-        rect.y=0;
-        rect.width=XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs+1)
+	rect.x=
+	  FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,xobj->value2);
+	rect.y=0;
+	rect.width=
+	  FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs+1)
 	  -rect.x+1;
-        rect.height=xobj->height;
-        xobj->value2=PosCurs;
-        DrawTextField(xobj);
+	rect.height=xobj->height;
+	xobj->value2=PosCurs;
+	DrawTextField(xobj);
        }
        else
        if (PosCurs<xobj->value2)
        {
-        rect.x=XTextWidth(xobj->xfont,xobj->title+xobj->value3,PosCurs)-1;
-        rect.y=0;
-        rect.width=XTextWidth(xobj->xfont,xobj->title+xobj->value3,
+	rect.x=FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,PosCurs)-1;
+	rect.y=0;
+	rect.width=FlocaleTextWidth(xobj->Ffont,xobj->title+xobj->value3,
 			      xobj->value2+1)-rect.x+2;
-        rect.height=xobj->height;
-        xobj->value2=PosCurs+xobj->value3;
-        DrawTextField(xobj);
+	rect.height=xobj->height;
+	xobj->value2=PosCurs+xobj->value3;
+	DrawTextField(xobj);
        }
        PosCurs=0;
       break;
@@ -422,9 +441,10 @@ void EvtKeyTextField(struct XObj *xobj,XKeyEvent *EvtKey)
     SendMsg(xobj,SingleClic);
    }
   }
-  else if (i!=0)	/* Cas d'un caractere normal */
+  else if (i!=0)        /* Cas d'un caractere normal */
   {
-   /* Insertion du caractere dans le titre */
+    /* Insertion du caractere dans le titre */
+    /* a normal character: insertion in the title */
    Size=strlen(xobj->title);
    xobj->title=(char*)realloc(xobj->title,(2+Size)*sizeof(char));
    memmove(&xobj->title[NewPos+1],&xobj->title[NewPos],
@@ -439,7 +459,7 @@ void EvtKeyTextField(struct XObj *xobj,XKeyEvent *EvtKey)
  if ((xobj->value!=NewPos)||(xobj->value2!=NewPos))
  {
   XSetForeground(dpy,xobj->gc,xobj->TabColor[back]);
-  x2=XTextWidth(xobj->xfont,xobj->title,strlen(xobj->title));
+  x2=FlocaleTextWidth(xobj->Ffont,xobj->title,strlen(xobj->title));
   XFillRectangle(dpy,xobj->win,xobj->gc,x2+4,4,xobj->width-x2-8,
 		 xobj->height-8);
   xobj->value=NewPos;
