@@ -18,11 +18,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <string.h>
 #include <fcntl.h>
 #include <pwd.h>
-#include <time.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <X11/Xlib.h>
@@ -54,21 +52,14 @@ extern Pixel back, fore;
 extern int colorset;
 extern int Clength;
 extern GC blackgc, hilite, shadow, checkered;
+extern FlocaleWinString *FwinString;
 
 GC statusgc = 0;
 GC tipsgc = 0;
-XFontStruct *StatusFont;
-#ifdef I18N_MB
-XFontSet StatusFontset;
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
+FlocaleFont *FStatusFont;
 int stwin_width = 100, goodies_width = 0;
 int anymail, unreadmail, newmail, mailcleared = 0;
-int fontheight, clock_width;
+int goodies_fontheight, clock_width;
 char *mailpath = NULL;
 char *clockfmt = NULL;
 char *datefmt = NULL;
@@ -86,7 +77,7 @@ char *TipsFore = "black",
 int tipscolorset = -1;
 int IgnoreOldMail = False;
 int ShowTips = False;
-char *statusfont_string = "fixed";
+char *statusfont_string = NULL;
 int last_date = -1;
 
 void cool_get_inboxstatus();
@@ -116,11 +107,15 @@ static void CreateOrUpdateGoodyGC(void)
     pfore = fore;
     pback = back;
   }
-  gcmask = GCForeground | GCBackground | GCFont | GCGraphicsExposures;
+  gcmask = GCForeground | GCBackground | GCGraphicsExposures;
   gcval.foreground = pfore;
   gcval.background = pback;
-  gcval.font = StatusFont->fid;
   gcval.graphics_exposures = False;
+  if (FStatusFont->font != NULL)
+  {
+    gcval.font = FStatusFont->font->fid;
+    gcmask = GCFont;
+  }
   if (statusgc)
     XChangeGC(dpy, statusgc, gcmask, &gcval);
   else
@@ -204,15 +199,15 @@ Bool GoodiesParseConfig(char *tline)
     do_check_mail = (has_mailpath && (mailcheck_interval > 0));
     break;
   case 2: /* Mailcheck */
-    do_check_mail = MAILCHECK_DEFAULT;
-    sscanf(rest, "%d", &do_check_mail);
+    mailcheck_interval = MAILCHECK_DEFAULT;
+    sscanf(rest, "%d", &mailcheck_interval);
     do_check_mail = (has_mailpath && (mailcheck_interval > 0));
     break;
   case 3: /* ClockFormat */
     UpdateString(&clockfmt, rest);
     break;
   case 4: /* StatusFont */
-    CopyString(&statusfont_string, rest);
+    CopyStringWithQuotes(&statusfont_string, rest);
     break;
   case 5: /* TipsFore */
     CopyString(&TipsFore, rest);
@@ -247,16 +242,21 @@ Bool GoodiesParseConfig(char *tline)
   return True;
 }
 
+void LoadGoodiesFont(void)
+{
+	if ((FStatusFont =
+	     FlocaleLoadFont(dpy, statusfont_string, Module)) == NULL)
+	{
+		fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+		exit(1);
+	}
+	goodies_fontheight = FStatusFont->height;
+}
+
 void InitGoodies(void)
 {
   struct passwd *pwent;
   char tmp[1024];
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
 
   if (mailpath == NULL) {
     strcpy(tmp, DEFAULT_MAIL_PATH);
@@ -265,29 +265,6 @@ void InitGoodies(void)
     UpdateString(&mailpath, tmp);
   }
 
-#ifdef I18N_MB
-  if ((StatusFontset=XCreateFontSet(dpy,statusfont_string,&ml,&mc,&ds))==NULL) {
-#ifdef STRICTLY_FIXED
-    if ((StatusFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds))==NULL) {
-#else
-    if ((StatusFontset=XCreateFontSet(dpy,"fixed,-*--14-*",&ml,&mc,&ds))==NULL) {
-#endif
-      fprintf(stderr, "%s: Couldn't load fixed fontset...exiting !\n",Module);
-      exit(1);
-    }
-  }
-  XFontsOfFontSet(StatusFontset,&fs_list,&ml);
-  StatusFont = fs_list[0];
-#else
-  if ((StatusFont = XLoadQueryFont(dpy, statusfont_string)) == NULL) {
-    if ((StatusFont = XLoadQueryFont(dpy, "fixed")) == NULL) {
-      fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n",Module);
-      exit(1);
-    }
-  }
-#endif
-
-  fontheight = StatusFont->ascent + StatusFont->descent;
   CreateOrUpdateGoodyGC();
   if (clockfmt)
   {
@@ -297,10 +274,10 @@ void InitGoodies(void)
     time(&timer);
     tms = localtime(&timer);
     strftime(str, 24, clockfmt, tms);
-    clock_width = XTextWidth(StatusFont, str, strlen(str)) + 4;
+    clock_width = FlocaleTextWidth(FStatusFont, str, strlen(str)) + 4;
   }
   else
-    clock_width = XTextWidth(StatusFont, "XX:XX", 5) + 4;
+    clock_width = FlocaleTextWidth(FStatusFont, "XX:XX", 5) + 4;
   goodies_width += clock_width;
   stwin_width = goodies_width;
 }
@@ -333,18 +310,26 @@ void DrawGoodies(void)
   }
 
   Draw3dBox(win, win_width - stwin_width, 0, stwin_width, RowHeight);
-#ifdef I18N_MB
-  XmbDrawString(dpy,win,StatusFontset,statusgc,
-#else
-  XDrawString(dpy,win,statusgc,
-#endif
-	      win_width - stwin_width + 4,
-	      ((RowHeight - fontheight) >> 1) +StatusFont->ascent,
-	      str, strlen(str));
+  FwinString->win = win;
+  FwinString->gc = statusgc;
+  FwinString->str = str;
+  FwinString->x = win_width - stwin_width + 4;
+  FwinString->y = ((RowHeight - goodies_fontheight) >> 1) + FStatusFont->ascent;
+  if (colorset >= 0)
+  {
+    FwinString->colorset = &Colorset[colorset];
+    FwinString->flags.has_colorset = True;
+  }
+  else
+  {
+    FwinString->flags.has_colorset = False;
+  }
+  FlocaleDrawString(dpy, FStatusFont, FwinString, 0);
 
   if (!do_check_mail)
     return;
-  if (timer - last_mail_check >= 10) {
+  if (timer - last_mail_check >= mailcheck_interval)
+  {
     cool_get_inboxstatus();
     last_mail_check = timer;
     if (newmail)
@@ -358,14 +343,14 @@ void DrawGoodies(void)
 	      ((RowHeight - minimail_height) >> 1));
   else if (!IgnoreOldMail /*&& !mailcleared*/ && anymail)
     XCopyArea(dpy, mailpix, win, statusgc, 0, 0,
-              minimail_width, minimail_height,
-              win_width - stwin_width + clock_width + 3,
-              ((RowHeight - minimail_height) >> 1));
+	      minimail_width, minimail_height,
+	      win_width - stwin_width + clock_width + 3,
+	      ((RowHeight - minimail_height) >> 1));
 
   if (Tip.open) {
     if (Tip.type == DATE_TIP)
       if (tms->tm_mday != last_date) /* reflect date change */
-        CreateDateWindow(); /* This automatically deletes any old window */
+	CreateDateWindow(); /* This automatically deletes any old window */
     last_date = tms->tm_mday;
     RedrawTipWindow();
   }
@@ -417,14 +402,28 @@ void CreateMailTipWindow()
 
 void RedrawTipWindow(void)
 {
+  /* FIXME: I am sure that we redraw to often */
   if (Tip.text) {
-#ifdef I18N_MB
-    XmbDrawString(dpy, Tip.win, StatusFontset, tipsgc, 3, Tip.th-4,
-#else
-    XDrawString(dpy, Tip.win, tipsgc, 3, Tip.th-4,
-#endif
-                     Tip.text, strlen(Tip.text));
-    XRaiseWindow(dpy, Tip.win);  /*****************/
+    FwinString->win = Tip.win;
+    FwinString->gc = tipsgc;
+    FwinString->str = Tip.text;
+    FwinString->x = 3;
+    FwinString->y = 2 + FStatusFont->ascent;
+    if (tipscolorset >= 0)
+    {
+       FwinString->colorset = &Colorset[tipscolorset];
+       FwinString->flags.has_colorset = True;
+    }
+    else
+    {
+       FwinString->flags.has_colorset = False;
+    }
+    if (FftSupport && FStatusFont->fftf.fftfont != NULL)
+    {
+       XClearArea(dpy, Tip.win, 0, 0, Tip.w, Tip.h, False);
+    }
+    FlocaleDrawString(dpy, FStatusFont, FwinString, 0);
+    XRaiseWindow(dpy, Tip.win);
   }
 }
 
@@ -438,8 +437,8 @@ void PopupTipWindow(int px, int py, const char *text)
   if (Tip.win != None)
     DestroyTipWindow();
 
-  Tip.tw = XTextWidth(StatusFont, text, strlen(text)) + 6;
-  Tip.th = StatusFont->ascent + StatusFont->descent + 4;
+  Tip.tw = FlocaleTextWidth(FStatusFont, (char *)text, strlen(text)) + 6;
+  Tip.th = FStatusFont->height + 4;
   XTranslateCoordinates(dpy, win, Root, px, py, &newx, &newy, &child);
 
   Tip.x = newx;
@@ -485,7 +484,7 @@ void CreateTipWindow(int x, int y, int w, int h)
 {
   unsigned long gcmask;
   unsigned long winattrmask = CWBackPixel | CWBorderPixel | CWEventMask |
-                              CWColormap |CWSaveUnder | CWOverrideRedirect;
+			      CWColormap |CWSaveUnder | CWOverrideRedirect;
   XSetWindowAttributes winattr;
   GC cgc = None;
   GC gc0 = None;
@@ -517,11 +516,15 @@ void CreateTipWindow(int x, int y, int w, int h)
   Tip.win = XCreateWindow(dpy, Root, x, y, w+4, h+4, 0, Pdepth, InputOutput,
 			  Pvisual, winattrmask, &winattr);
 
-  gcmask = GCForeground | GCBackground | GCFont | GCGraphicsExposures;
+  gcmask = GCForeground | GCBackground | GCGraphicsExposures;
   gcval.graphics_exposures = False;
   gcval.foreground = tip_fore;
   gcval.background = tip_back;
-  gcval.font = StatusFont->fid;
+  if (FStatusFont->font != NULL)
+  {
+    gcval.font = FStatusFont->font->fid;
+    gcmask |= GCFont;
+  }
   if (tipsgc)
     XChangeGC(dpy, tipsgc, gcmask, &gcval);
   else
@@ -531,7 +534,7 @@ void CreateTipWindow(int x, int y, int w, int h)
   pclip = XCreatePixmap(dpy, Tip.win, w+4, h+4, 1);
 
   gcmask = GCForeground | GCBackground | GCFillStyle | GCStipple |
-           GCGraphicsExposures;
+	   GCGraphicsExposures;
   gcval.foreground = 1;
   gcval.background = 0;
   gcval.fill_style = FillStippled;
@@ -622,21 +625,21 @@ void cool_get_inboxstatus()
       newsize = st.st_size;
 
       if (newsize > 0)
-         anymail = 1;
+	 anymail = 1;
       else
-         anymail = 0;
+	 anymail = 0;
 
       if (st.st_mtime >= st.st_atime && newsize > 0)
-         unreadmail = 1;
+	 unreadmail = 1;
       else
-         unreadmail = 0;
+	 unreadmail = 0;
 
       if (newsize > oldsize && unreadmail) {
-         newmail = 1;
-         mailcleared = 0;
+	 newmail = 1;
+	 mailcleared = 0;
       }
       else
-         newmail = 0;
+	 newmail = 0;
    }
 
    oldsize = newsize;
@@ -649,7 +652,7 @@ void HandleMailClick(XEvent event)
   static Time lastclick = 0;
   if (event.xbutton.time - lastclick < 250)
   {
-    SendFvwmPipe(Fvwm_fd, MailCmd, 0);
+    SendText(Fvwm_fd, MailCmd, 0);
   }
   lastclick = event.xbutton.time;
   mailcleared = 1;
