@@ -58,10 +58,8 @@
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <string.h>
 #include <sys/wait.h>
-#include <sys/time.h>
-#include <time.h>
+#include "libs/ftime.h"
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -73,8 +71,9 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/FRenderInit.h"
 #include "libs/Module.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 #include "libs/Colorset.h"
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -97,18 +96,18 @@
 /*
  * You may want to raise the following values if your machine is fast
  */
-#define ANIM_STEP	2   /* must be >= 1. Greater is smoother and slower */
-#define ANIM_STEP_MAIN	1   /* same for the main folder */
-#define ANIM_DELAY	10
+#define ANIM_STEP       2   /* must be >= 1. Greater is smoother and slower */
+#define ANIM_STEP_MAIN  1   /* same for the main folder */
+#define ANIM_DELAY      10
 
 #ifdef ENABLE_SOUND
-#define WHEV_PUSH		0
-#define WHEV_CLOSE_FOLDER	1
-#define WHEV_OPEN_FOLDER	2
-#define WHEV_CLOSE_MAIN		3
-#define WHEV_OPEN_MAIN		4
-#define WHEV_DROP		5
-#define MAX_EVENTS		6
+#define WHEV_PUSH               0
+#define WHEV_CLOSE_FOLDER       1
+#define WHEV_OPEN_FOLDER        2
+#define WHEV_CLOSE_MAIN         3
+#define WHEV_OPEN_MAIN          4
+#define WHEV_DROP               5
+#define MAX_EVENTS              6
 
 int SoundActive = 0;
 char *Sounds[6]={".",".",".",".",".","."};
@@ -187,10 +186,10 @@ int MaxColors=16;
 int Withdrawn;
 static int global_colorset = -1;
 
-#define DIR_TOLEFT	1
-#define DIR_TORIGHT	2
-#define DIR_TOUP	3
-#define DIR_TODOWN	4
+#define DIR_TOLEFT      1
+#define DIR_TORIGHT     2
+#define DIR_TOUP        3
+#define DIR_TODOWN      4
 
 #ifdef ENABLE_SOUND
 void waitchild(int bullshit)
@@ -210,7 +209,7 @@ static void configure_all_icon_windows(void);
 /***********************************************************************
  *
  *  Procedure:
- *	main - start of afterstep
+ *      main - start of afterstep
  *
  ***********************************************************************
  */
@@ -243,8 +242,9 @@ int main(int argc, char **argv)
       Buttons[i].icons[j].file = NULL;
       Buttons[i].icons[j].w = 0;
       Buttons[i].icons[j].h = 0;
-      Buttons[i].icons[j].mask = None;	/* pixmap for the icon mask */
+      Buttons[i].icons[j].mask = None;  /* pixmap for the icon mask */
       Buttons[i].icons[j].icon = None;
+      Buttons[i].icons[j].alpha = None;
       Buttons[i].icons[j].depth = 0;
     }
     Buttons[i].IconWin = None;
@@ -269,11 +269,12 @@ int main(int argc, char **argv)
 	    XDisplayName(display_name));
     exit (1);
   }
-  InitPictureCMap(dpy);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   /* Initialise default colorset */
   AllocColorset(0);
   FShapeInit(dpy);
+  FRenderInit(dpy);
 
   x_fd = XConnectionNumber(dpy);
 
@@ -377,7 +378,7 @@ int main(int argc, char **argv)
       goto Builtin;
     }
     Buttons[BACK_BUTTON].icons[0].file=BgPixmapFile;
-    if (GetXPMFile(BACK_BUTTON,0))
+    if (LoadIconFile(BACK_BUTTON,0))
       break;
     else
       goto Solid;
@@ -400,10 +401,7 @@ int main(int argc, char **argv)
   case TEXTURE_BUILTIN:
     Builtin:
     TextureType=TEXTURE_BUILTIN;
-    /* if (GetXPMData( BACK_BUTTON, button_xpm))
-       break;
-       pdg */
-  default:
+   default:
     Solid:
     TextureType=TEXTURE_SOLID;
     if (GetSolidXPM(BACK_BUTTON, BgColor))
@@ -467,7 +465,7 @@ int main(int argc, char **argv)
 /***********************************************************************
  *
  *  Procedure:
- *	Loop - wait for data to process
+ *      Loop - wait for data to process
  *
  ***********************************************************************/
 
@@ -515,7 +513,7 @@ void Loop(void)
 	if (Event.xbutton.button != Button1) {
 	  if (Event.xbutton.button == Button2) {
 	    static int LastX, LastY;
-            static int scr_x, scr_y, scr_w, scr_h;
+	    static int scr_x, scr_y, scr_w, scr_h;
 
 	    if (LastMapped != -1) {
 	      CloseFolder(LastMapped);
@@ -548,12 +546,12 @@ void Loop(void)
 	      }
 	      XTranslateCoordinates(
 		dpy,main_win,Root,LastX,LastY,&LastX,&LastY,&junk);
-              /*!!! Note: maybe it will be better to use (LastX+W/2,LastY+H/2)
-                in ScrRect query? That could help in pathological cases when
-                Wharf is not completely on a single screen. */
-    	      fscr.xypos.x = LastX;
+	      /*!!! Note: maybe it will be better to use (LastX+W/2,LastY+H/2)
+		in ScrRect query? That could help in pathological cases when
+		Wharf is not completely on a single screen. */
+	      fscr.xypos.x = LastX;
 	      fscr.xypos.y = LastY;
-              FScreenGetScrRect(
+	      FScreenGetScrRect(
 		&fscr, FSCREEN_XYPOS, &scr_x, &scr_y, &scr_w, &scr_h);
 	      if (num_rows<num_columns) { /* horizontal */
 		if (LastY > scr_y + scr_h / 2) {
@@ -576,7 +574,7 @@ void Loop(void)
 				    CornerX, CornerY,
 				    BUTTONWIDTH,BUTTONHEIGHT);
 		}
-	      } else {	/* vertical */
+	      } else {  /* vertical */
 		if (LastX > scr_x + scr_w/2) {
 		  CornerX = scr_x + scr_w - BUTTONWIDTH;
 		} else {
@@ -614,7 +612,7 @@ void Loop(void)
 	if (*CurrentWin!=main_win) {
 	  CurrentButton = CurrentBase + CurrentColumn*num_rows
 	    + CurrentRow*num_columns;
-	}	else {
+	}       else {
 	  CurrentButton = CurrentBase + CurrentColumn
 	    + CurrentRow*num_columns;
 	  if (CurrentButton>=num_buttons) {
@@ -707,7 +705,7 @@ void Loop(void)
 	  int dummy_x, dummy_y, base, pos_x, pos_y;
 	  unsigned int dummy;
 
-	  /*		    if (Event.xclient.data.l[0]!=DndFile ||
+	  /*                if (Event.xclient.data.l[0]!=DndFile ||
 			    Event.xclient.data.l[0]!=DndFiles ||
 			    Event.xclient.data.l[0]!=DndExe
 			    )
@@ -794,7 +792,7 @@ void Loop(void)
 	if (*CurrentWin!=main_win) {
 	  NewButton = CurrentBase + CurrentColumn*num_rows
 	    + CurrentRow*num_columns;
-	}	else {
+	}       else {
 	  NewButton = CurrentBase + CurrentColumn
 	    + CurrentRow*num_columns;
 	}
@@ -2337,12 +2335,12 @@ void process_message(unsigned long type,unsigned long *body)
  * ICCCM Client Messages - Section 4.2.8 of the ICCCM dictates that all
  * client messages will have the following form:
  *
- *     event type	ClientMessage
- *     message type	_XA_WM_PROTOCOLS
- *     window		tmp->w
- *     format		32
- *     data[0]		message atom
- *     data[1]		time stamp
+ *     event type       ClientMessage
+ *     message type     _XA_WM_PROTOCOLS
+ *     window           tmp->w
+ *     format           32
+ *     data[0]          message atom
+ *     data[1]          time stamp
  *
  ****************************************************************************/
 void my_send_clientmessage (Window w, Atom a, Time timestamp)
@@ -2475,8 +2473,8 @@ void FindLockMods(void)
     {
       for (i = 0; i < mm->max_keypermod; i++)
       {
-      	if ((kc = *kp++) &&
-      	    ((ks = XKeycodeToKeysym(dpy, kc, 0)) != NoSymbol))
+	if ((kc = *kp++) &&
+	    ((ks = XKeycodeToKeysym(dpy, kc, 0)) != NoSymbol))
 	{
 	  kn = XKeysymToString(ks);
 	  knl = strlen(kn);
